@@ -13,7 +13,7 @@ extern "C" {
 
 
 /** Defines the Baresip version string */
-#define BARESIP_VERSION "0.6.1"
+#define BARESIP_VERSION "1.0.0"
 
 
 #ifndef NET_MAX_NS
@@ -21,6 +21,11 @@ extern "C" {
 #endif
 
 
+/*
+ * Clock-rate for audio timestamp
+ */
+#define AUDIO_TIMEBASE 1000000U
+
 /*
  * Clock-rate for video timestamp
  */
@@ -54,6 +59,8 @@ struct account;
 
 int account_alloc(struct account **accp, const char *sipaddr);
 int account_debug(struct re_printf *pf, const struct account *acc);
+int account_json_api(struct odict *odacc, struct odict *odcfg,
+		 const struct account *acc);
 int account_set_auth_user(struct account *acc, const char *user);
 int account_set_auth_pass(struct account *acc, const char *pass);
 int account_set_outbound(struct account *acc, const char *ob, unsigned ix);
@@ -61,11 +68,15 @@ int account_set_sipnat(struct account *acc, const char *sipnat);
 int account_set_answermode(struct account *acc, enum answermode mode);
 int account_set_display_name(struct account *acc, const char *dname);
 int account_set_regint(struct account *acc, uint32_t regint);
+int account_set_stun_uri(struct account *acc, const char *uri);
 int account_set_stun_host(struct account *acc, const char *host);
 int account_set_stun_port(struct account *acc, uint16_t port);
+int account_set_stun_user(struct account *acc, const char *user);
+int account_set_stun_pass(struct account *acc, const char *pass);
 int account_set_mediaenc(struct account *acc, const char *mediaenc);
 int account_set_medianat(struct account *acc, const char *medianat);
 int account_set_audio_codecs(struct account *acc, const char *codecs);
+int account_set_video_codecs(struct account *acc, const char *codecs);
 int account_set_mwi(struct account *acc, const char *value);
 int account_set_call_transfer(struct account *acc, const char *value);
 int account_auth(const struct account *acc, char **username, char **password,
@@ -73,6 +84,7 @@ int account_auth(const struct account *acc, char **username, char **password,
 struct list *account_aucodecl(const struct account *acc);
 struct list *account_vidcodecl(const struct account *acc);
 struct sip_addr *account_laddr(const struct account *acc);
+struct uri *account_luri(const struct account *acc);
 uint32_t account_regint(const struct account *acc);
 uint32_t account_pubint(const struct account *acc);
 uint32_t account_ptime(const struct account *acc);
@@ -86,11 +98,32 @@ const char *account_sipnat(const struct account *acc);
 const char *account_stun_user(const struct account *acc);
 const char *account_stun_pass(const struct account *acc);
 const char *account_stun_host(const struct account *acc);
+const struct stun_uri *account_stun_uri(const struct account *acc);
 uint16_t account_stun_port(const struct account *acc);
 const char *account_mediaenc(const struct account *acc);
 const char *account_medianat(const struct account *acc);
 const char *account_mwi(const struct account *acc);
 const char *account_call_transfer(const struct account *acc);
+const char *account_extra(const struct account *acc);
+
+
+/*
+ * Audio frame
+ */
+
+/**
+ * Defines a frame of audio samples
+ */
+struct auframe {
+	int fmt;             /**< Sample format (enum aufmt)        */
+	void *sampv;         /**< Audio samples (must be mem_ref'd) */
+	size_t sampc;        /**< Total number of audio samples     */
+	uint64_t timestamp;  /**< Timestamp in AUDIO_TIMEBASE units */
+};
+
+void   auframe_init(struct auframe *af, int fmt, void *sampv, size_t sampc);
+size_t auframe_size(const struct auframe *af);
+void   auframe_mute(struct auframe *af);
 
 
 /*
@@ -102,7 +135,7 @@ const char *account_call_transfer(const struct account *acc);
 #define AULEVEL_MAX    (0.0)
 
 
-double aulevel_calc_dbov(const int16_t *sampv, size_t sampc);
+double aulevel_calc_dbov(int fmt, const void *sampv, size_t sampc);
 
 
 /*
@@ -120,6 +153,24 @@ enum call_event {
 	CALL_EVENT_MENC,
 };
 
+/** Call States */
+enum call_state {
+	CALL_STATE_IDLE = 0,
+	CALL_STATE_INCOMING,
+	CALL_STATE_OUTGOING,
+	CALL_STATE_RINGING,
+	CALL_STATE_EARLY,
+	CALL_STATE_ESTABLISHED,
+	CALL_STATE_TERMINATED,
+	CALL_STATE_UNKNOWN
+};
+
+/** Video mode */
+enum vidmode {
+	VIDMODE_OFF = 0,    /**< Video disabled                */
+	VIDMODE_ON,         /**< Video enabled                 */
+};
+
 struct call;
 
 typedef void (call_event_h)(struct call *call, enum call_event ev,
@@ -127,8 +178,12 @@ typedef void (call_event_h)(struct call *call, enum call_event ev,
 typedef void (call_dtmf_h)(struct call *call, char key, void *arg);
 
 int  call_connect(struct call *call, const struct pl *paddr);
+int  call_answer(struct call *call, uint16_t scode, enum vidmode vmode);
+int  call_progress(struct call *call);
+void call_hangup(struct call *call, uint16_t scode, const char *reason);
 int  call_modify(struct call *call);
 int  call_hold(struct call *call, bool hold);
+int  call_set_video_dir(struct call *call, enum sdp_dir dir);
 int  call_send_digit(struct call *call, char key);
 bool call_has_audio(const struct call *call);
 bool call_has_video(const struct call *call);
@@ -140,6 +195,7 @@ int  call_notify_sipfrag(struct call *call, uint16_t scode,
 void call_set_handlers(struct call *call, call_event_h *eh,
 		       call_dtmf_h *dtmfh, void *arg);
 uint16_t      call_scode(const struct call *call);
+enum call_state call_state(const struct call *call);
 uint32_t      call_duration(const struct call *call);
 uint32_t      call_setup_duration(const struct call *call);
 const char   *call_id(const struct call *call);
@@ -155,6 +211,7 @@ bool          call_is_outgoing(const struct call *call);
 void          call_enable_rtp_timeout(struct call *call, uint32_t timeout_ms);
 uint32_t      call_linenum(const struct call *call);
 struct call  *call_find_linenum(const struct list *calls, uint32_t linenum);
+struct call  *call_find_id(const struct list *calls, const char *id);
 void call_set_current(struct list *calls, struct call *call);
 const struct list *call_get_custom_hdrs(const struct call *call);
 
@@ -179,6 +236,7 @@ int custom_hdrs_apply(const struct list *hdrs, custom_hdrs_h *h, void *arg);
 typedef int (confline_h)(const struct pl *addr, void *arg);
 
 int  conf_configure(void);
+int  conf_configure_buf(const uint8_t *buf, size_t sz);
 int  conf_modules(void);
 void conf_path_set(const char *path);
 int  conf_path_get(char *path, size_t sz);
@@ -215,7 +273,6 @@ enum audio_mode {
 
 /** SIP User-Agent */
 struct config_sip {
-	uint32_t trans_bsize;   /**< SIP Transaction bucket size    */
 	char uuid[64];          /**< Universally Unique Identifier  */
 	char local[64];         /**< Local SIP Address              */
 	char cert[256];         /**< SIP Certificate                */
@@ -241,16 +298,15 @@ struct config_audio {
 	uint32_t srate_src;     /**< Opt. sampling rate for source  */
 	uint32_t channels_play; /**< Opt. channels for player       */
 	uint32_t channels_src;  /**< Opt. channels for source       */
-	bool src_first;         /**< Audio source opened first      */
 	enum audio_mode txmode; /**< Audio transmit mode            */
 	bool level;             /**< Enable audio level indication  */
 	int src_fmt;            /**< Audio source sample format     */
 	int play_fmt;           /**< Audio playback sample format   */
 	int enc_fmt;            /**< Audio encoder sample format    */
 	int dec_fmt;            /**< Audio decoder sample format    */
+	struct range buffer;    /**< Audio receive buffer in [ms]   */
 };
 
-#ifdef USE_VIDEO
 /** Video */
 struct config_video {
 	char src_mod[16];       /**< Video source module            */
@@ -263,7 +319,6 @@ struct config_video {
 	bool fullscreen;        /**< Enable fullscreen display      */
 	int enc_fmt;            /**< Encoder pixelfmt (enum vidfmt) */
 };
-#endif
 
 /** Audio/Video Transport */
 struct config_avt {
@@ -278,25 +333,15 @@ struct config_avt {
 
 /** Network Configuration */
 struct config_net {
+	int af;                 /**< AF_UNSPEC, AF_INET or AF_INET6 */
 	char ifname[64];        /**< Bind to interface (optional)   */
 	struct {
 		char addr[64];
+		bool fallback;
 	} nsv[NET_MAX_NS];      /**< Configured DNS nameservers     */
 	size_t nsc;             /**< Number of DNS nameservers      */
 };
 
-#ifdef USE_VIDEO
-/* BFCP */
-struct config_bfcp {
-	char proto[16];         /**< BFCP Transport (optional)      */
-};
-#endif
-
-/** SDP */
-struct config_sdp {
-	bool ebuacip;           /**< Enable EBU-ACIP parameters     */
-};
-
 
 /** Core configuration */
 struct config {
@@ -307,18 +352,10 @@ struct config {
 
 	struct config_audio audio;
 
-#ifdef USE_VIDEO
 	struct config_video video;
-#endif
 	struct config_avt avt;
 
 	struct config_net net;
-
-#ifdef USE_VIDEO
-	struct config_bfcp bfcp;
-#endif
-
-	struct config_sdp sdp;
 };
 
 int config_parse_conf(struct config *cfg, const struct conf *conf);
@@ -404,11 +441,11 @@ typedef void (message_recv_h)(struct ua *ua, const struct pl *peer,
 			      struct mbuf *body, void *arg);
 
 struct message;
-struct message_lsnr;
 
 int  message_init(struct message **messagep);
-int  message_listen(struct message_lsnr **lsnrp, struct message *message,
+int  message_listen(struct message *message,
 		    message_recv_h *h, void *arg);
+void message_unlisten(struct message *message, message_recv_h *recvh);
 int  message_send(struct ua *ua, const char *peer, const char *msg,
 		  sip_resp_h *resph, void *arg);
 
@@ -428,7 +465,7 @@ struct ausrc_prm {
 	int        fmt;         /**< Sample format (enum aufmt) */
 };
 
-typedef void (ausrc_read_h)(const void *sampv, size_t sampc, void *arg);
+typedef void (ausrc_read_h)(struct auframe *af, void *arg);
 typedef void (ausrc_error_h)(int err, const char *str, void *arg);
 
 typedef int  (ausrc_alloc_h)(struct ausrc_st **stp, const struct ausrc *ausrc,
@@ -515,7 +552,6 @@ struct aufilt_dec_st {
 struct aufilt_prm {
 	uint32_t srate;       /**< Sampling rate in [Hz]        */
 	uint8_t  ch;          /**< Number of channels           */
-	uint32_t ptime;       /**< Wanted packet-time in [ms]   */
 	int      fmt;         /**< Sample format (enum aufmt)   */
 };
 
@@ -523,13 +559,13 @@ typedef int (aufilt_encupd_h)(struct aufilt_enc_st **stp, void **ctx,
 			      const struct aufilt *af, struct aufilt_prm *prm,
 			      const struct audio *au);
 typedef int (aufilt_encode_h)(struct aufilt_enc_st *st,
-			      void *sampv, size_t *sampc);
+			      struct auframe *af);
 
 typedef int (aufilt_decupd_h)(struct aufilt_dec_st **stp, void **ctx,
 			      const struct aufilt *af, struct aufilt_prm *prm,
 			      const struct audio *au);
 typedef int (aufilt_decode_h)(struct aufilt_dec_st *st,
-			      void *sampv, size_t *sampc);
+			      struct auframe *af);
 
 struct aufilt {
 	struct le le;
@@ -564,6 +600,9 @@ struct log {
 
 void log_register_handler(struct log *logh);
 void log_unregister_handler(struct log *logh);
+void log_level_set(enum log_level level);
+enum log_level log_level_get(void);
+const char *log_level_name(enum log_level level);
 void log_enable_debug(bool enable);
 void log_enable_info(bool enable);
 void log_enable_stdout(bool enable);
@@ -572,7 +611,6 @@ void loglv(enum log_level level, const char *fmt, ...);
 void debug(const char *fmt, ...);
 void info(const char *fmt, ...);
 void warning(const char *fmt, ...);
-void error_msg(const char *fmt, ...);
 
 
 /*
@@ -591,23 +629,28 @@ enum menc_event {
 };
 
 
-typedef void (menc_error_h)(int err, void *arg);
+typedef void (menc_event_h)(enum menc_event event, const char *prm,
+			    struct stream *strm, void *arg);
 
-typedef void (menc_event_h)(enum menc_event event, const char *prm, void *arg);
+typedef void (menc_error_h)(int err, void *arg);
 
 typedef int  (menc_sess_h)(struct menc_sess **sessp, struct sdp_session *sdp,
 			   bool offerer, menc_event_h *eventh,
 			   menc_error_h *errorh, void *arg);
 
 typedef int  (menc_media_h)(struct menc_media **mp, struct menc_sess *sess,
-			    struct rtp_sock *rtp, int proto,
-			    void *rtpsock, void *rtcpsock,
-			    struct sdp_media *sdpm);
+			   struct rtp_sock *rtp,
+			   struct udp_sock *rtpsock, struct udp_sock *rtcpsock,
+			   const struct sa *raddr_rtp,
+			   const struct sa *raddr_rtcp,
+			   struct sdp_media *sdpm,
+			   const struct stream *strm);
 
 struct menc {
 	struct le le;
 	const char *id;
 	const char *sdp_proto;
+	bool wait_secure;
 	menc_sess_h *sessh;
 	menc_media_h *mediah;
 };
@@ -626,13 +669,17 @@ struct network;
 
 typedef void (net_change_h)(void *arg);
 
-int  net_alloc(struct network **netp, const struct config_net *cfg, int af);
-int  net_use_nameserver(struct network *net, const struct sa *ns);
+int  net_alloc(struct network **netp, const struct config_net *cfg);
+int  net_use_nameserver(struct network *net,
+			const struct sa *srvv, size_t srvc);
+int  net_set_address(struct network *net, const struct sa *ip);
 void net_change(struct network *net, uint32_t interval,
 		net_change_h *ch, void *arg);
 void net_force_change(struct network *net);
 bool net_check(struct network *net);
-int  net_af(const struct network *net);
+bool net_af_enabled(const struct network *net, int af);
+int  net_set_af(struct network *net, int af);
+int  net_dns_debug(struct re_printf *pf, const struct network *net);
 int  net_debug(struct re_printf *pf, const struct network *net);
 const struct sa *net_laddr_af(const struct network *net, int af);
 const char      *net_domain(const struct network *net);
@@ -647,10 +694,12 @@ struct play;
 struct player;
 
 int  play_file(struct play **playp, struct player *player,
-	       const char *filename, int repeat);
+	       const char *filename, int repeat,
+	       const char *play_mod, const char *play_dev);
 int  play_tone(struct play **playp, struct player *player,
 	       struct mbuf *tone,
-	       uint32_t srate, uint8_t ch, int repeat);
+	       uint32_t srate, uint8_t ch, int repeat,
+	       const char *play_mod, const char *play_dev);
 int  play_init(struct player **playerp);
 void play_set_path(struct player *player, const char *path);
 
@@ -680,21 +729,18 @@ enum ua_event {
 	UA_EVENT_CALL_TRANSFER_FAILED,
 	UA_EVENT_CALL_DTMF_START,
 	UA_EVENT_CALL_DTMF_END,
+	UA_EVENT_CALL_RTPESTAB,
 	UA_EVENT_CALL_RTCP,
 	UA_EVENT_CALL_MENC,
 	UA_EVENT_VU_TX,
 	UA_EVENT_VU_RX,
 	UA_EVENT_AUDIO_ERROR,
+	UA_EVENT_CALL_LOCAL_SDP,      /**< param: offer or answer */
+	UA_EVENT_CALL_REMOTE_SDP,     /**< param: offer or answer */
 
 	UA_EVENT_MAX,
 };
 
-/** Video mode */
-enum vidmode {
-	VIDMODE_OFF = 0,    /**< Video disabled                */
-	VIDMODE_ON,         /**< Video enabled                 */
-};
-
 /** Defines the User-Agent event handler */
 typedef void (ua_event_h)(struct ua *ua, enum ua_event ev,
 			  struct call *call, const char *prm, void *arg);
@@ -709,12 +755,12 @@ int  ua_connect(struct ua *ua, struct call **callp,
 		enum vidmode vmode);
 void ua_hangup(struct ua *ua, struct call *call,
 	       uint16_t scode, const char *reason);
-int  ua_answer(struct ua *ua, struct call *call);
-int  ua_progress(struct ua *ua, struct call *call);
-int  ua_hold_answer(struct ua *ua, struct call *call);
+int  ua_answer(struct ua *ua, struct call *call, enum vidmode vmode);
+int  ua_hold_answer(struct ua *ua, struct call *call, enum vidmode vmode);
 int  ua_options_send(struct ua *ua, const char *uri,
 		     options_resp_h *resph, void *arg);
 int  ua_debug(struct re_printf *pf, const struct ua *ua);
+int  ua_state_json_api(struct odict *od, const struct ua *ua);
 int  ua_print_calls(struct re_printf *pf, const struct ua *ua);
 int  ua_print_status(struct re_printf *pf, const struct ua *ua);
 int  ua_print_supported(struct re_printf *pf, const struct ua *ua);
@@ -722,6 +768,7 @@ int  ua_update_account(struct ua *ua);
 int  ua_register(struct ua *ua);
 void ua_unregister(struct ua *ua);
 bool ua_isregistered(const struct ua *ua);
+unsigned ua_destroy(struct ua *ua);
 void ua_pub_gruu_set(struct ua *ua, const struct pl *pval);
 const char     *ua_aor(const struct ua *ua);
 const char     *ua_cuser(const struct ua *ua);
@@ -747,16 +794,15 @@ int  ua_call_alloc(struct call **callp, struct ua *ua,
 
 
 /* One instance */
-int  ua_init(const char *software, bool udp, bool tcp, bool tls,
-	     bool prefer_ipv6);
+int  ua_init(const char *software, bool udp, bool tcp, bool tls);
 void ua_close(void);
 void ua_stop_all(bool forced);
 void uag_set_exit_handler(ua_exit_h *exith, void *arg);
+void uag_enable_sip_trace(bool enable);
 int  uag_reset_transp(bool reg, bool reinvite);
 int  uag_event_register(ua_event_h *eh, void *arg);
 void uag_event_unregister(ua_event_h *eh);
 void uag_set_sub_handler(sip_msg_h *subh);
-int  ua_print_sip_status(struct re_printf *pf, void *unused);
 int  uag_set_extra_params(const char *eprm);
 struct ua   *uag_find(const struct pl *cuser);
 struct ua   *uag_find_aor(const char *aor);
@@ -820,7 +866,6 @@ enum {
 struct cmd_arg {
 	char key;         /**< Which key was pressed  */
 	char *prm;        /**< Optional parameter     */
-	bool complete;    /**< True if complete       */
 	void *data;       /**< Application data       */
 };
 
@@ -861,8 +906,8 @@ struct vidsrc_st;
 
 /** Video Source parameters */
 struct vidsrc_prm {
-	int orient;       /**< Wanted picture orientation (enum vidorient) */
 	double fps;       /**< Wanted framerate                            */
+	int fmt;          /**< Wanted pixel format (enum vidfmt)           */
 };
 
 /**
@@ -915,7 +960,6 @@ struct vidisp_st;
 
 /** Video Display parameters */
 struct vidisp_prm {
-	void *view;       /**< Optional view (set by application or module) */
 	bool fullscreen;  /**< Enable fullscreen display                    */
 };
 
@@ -972,16 +1016,17 @@ typedef int (auenc_update_h)(struct auenc_state **aesp,
 			     const struct aucodec *ac,
 			     struct auenc_param *prm, const char *fmtp);
 typedef int (auenc_encode_h)(struct auenc_state *aes,
-			     uint8_t *buf, size_t *len,
+			     bool *marker, uint8_t *buf, size_t *len,
 			     int fmt, const void *sampv, size_t sampc);
 
 typedef int (audec_update_h)(struct audec_state **adsp,
 			     const struct aucodec *ac, const char *fmtp);
 typedef int (audec_decode_h)(struct audec_state *ads,
 			     int fmt, void *sampv, size_t *sampc,
-			     const uint8_t *buf, size_t len);
+			     bool marker, const uint8_t *buf, size_t len);
 typedef int (audec_plc_h)(struct audec_state *ads,
-			  int fmt, void *sampv, size_t *sampc);
+			  int fmt, void *sampv, size_t *sampc,
+			  const uint8_t *buf, size_t len);
 
 struct aucodec {
 	struct le le;
@@ -991,6 +1036,7 @@ struct aucodec {
 	uint32_t crate;             /* RTP Clock rate   */
 	uint8_t ch;
 	uint8_t pch;                /* RTP packet channels */
+	uint32_t ptime;             /* Packet time in [ms] (optional) */
 	const char *fmtp;
 	auenc_update_h *encupdh;
 	auenc_encode_h *ench;
@@ -1085,13 +1131,25 @@ struct vidfilt_dec_st {
 	struct le le;
 };
 
+/** Video Filter Parameters */
+struct vidfilt_prm {
+	unsigned width;   /**< Picture width              */
+	unsigned height;  /**< Picture height             */
+	int fmt;          /**< Pixel format (enum vidfmt) */
+	double fps;       /**< Video framerate            */
+};
+
 typedef int (vidfilt_encupd_h)(struct vidfilt_enc_st **stp, void **ctx,
-			       const struct vidfilt *vf);
+			       const struct vidfilt *vf,
+			       struct vidfilt_prm *prm,
+			       const struct video *vid);
 typedef int (vidfilt_encode_h)(struct vidfilt_enc_st *st,
 			       struct vidframe *frame, uint64_t *timestamp);
 
 typedef int (vidfilt_decupd_h)(struct vidfilt_dec_st **stp, void **ctx,
-			       const struct vidfilt *vf);
+			       const struct vidfilt *vf,
+			       struct vidfilt_prm *prm,
+			       const struct video *vid);
 typedef int (vidfilt_decode_h)(struct vidfilt_dec_st *st,
 			       struct vidframe *frame, uint64_t *timestamp);
 
@@ -1107,9 +1165,11 @@ struct vidfilt {
 void vidfilt_register(struct list *vidfiltl, struct vidfilt *vf);
 void vidfilt_unregister(struct vidfilt *vf);
 int vidfilt_enc_append(struct list *filtl, void **ctx,
-		       const struct vidfilt *vf);
+		       const struct vidfilt *vf, struct vidfilt_prm *prm,
+		       const struct video *vid);
 int vidfilt_dec_append(struct list *filtl, void **ctx,
-		       const struct vidfilt *vf);
+		       const struct vidfilt *vf, struct vidfilt_prm *prm,
+		       const struct video *vid);
 
 
 /*
@@ -1122,27 +1182,33 @@ struct mnat;
 struct mnat_sess;
 
 typedef void (audio_event_h)(int key, bool end, void *arg);
+typedef void (audio_level_h)(bool tx, double lvl, void *arg);
 typedef void (audio_err_h)(int err, const char *str, void *arg);
 
-int audio_alloc(struct audio **ap, const struct stream_param *stream_prm,
+int audio_alloc(struct audio **ap, struct list *streaml,
+		const struct stream_param *stream_prm,
 		const struct config *cfg,
-		struct call *call, struct sdp_session *sdp_sess, int label,
+		struct account *acc, struct sdp_session *sdp_sess, int label,
 		const struct mnat *mnat, struct mnat_sess *mnat_sess,
 		const struct menc *menc, struct menc_sess *menc_sess,
 		uint32_t ptime, const struct list *aucodecl, bool offerer,
-		audio_event_h *eventh, audio_err_h *errh, void *arg);
+		audio_event_h *eventh, audio_level_h *levelh,
+		audio_err_h *errh, void *arg);
 void audio_mute(struct audio *a, bool muted);
 bool audio_ismuted(const struct audio *a);
-void audio_set_devicename(struct audio *a, const char *src, const char *play);
+int  audio_set_devicename(struct audio *a, const char *src, const char *play);
 int  audio_set_source(struct audio *au, const char *mod, const char *device);
 int  audio_set_player(struct audio *au, const char *mod, const char *device);
-void audio_encoder_cycle(struct audio *audio);
+void audio_level_put(const struct audio *au, bool tx, double lvl);
 int  audio_level_get(const struct audio *au, double *level);
 int  audio_debug(struct re_printf *pf, const struct audio *a);
 struct stream *audio_strm(const struct audio *au);
+uint64_t audio_jb_current_value(const struct audio *au);
 int  audio_set_bitrate(struct audio *au, uint32_t bitrate);
 bool audio_rxaubuf_started(const struct audio *au);
 int  audio_start(struct audio *a);
+int  audio_start_source(struct audio *a, struct list *ausrcl,
+			struct list *aufiltl);
 void audio_stop(struct audio *a);
 bool audio_started(const struct audio *a);
 void audio_set_hold(struct audio *au, bool hold);
@@ -1151,6 +1217,8 @@ int  audio_encoder_set(struct audio *a, const struct aucodec *ac,
 int  audio_decoder_set(struct audio *a, const struct aucodec *ac,
 		       int pt_rx, const char *params);
 const struct aucodec *audio_codec(const struct audio *au, bool tx);
+struct config_audio *audio_config(struct audio *au);
+void audio_set_media_context(struct audio *au, struct media_ctx **ctx);
 
 
 /*
@@ -1159,20 +1227,35 @@ const struct aucodec *audio_codec(const struct audio *au, bool tx);
 
 struct video;
 
-void  video_mute(struct video *v, bool muted);
-void *video_view(const struct video *v);
+typedef void (video_err_h)(int err, const char *str, void *arg);
+
+int  video_alloc(struct video **vp, struct list *streaml,
+		 const struct stream_param *stream_prm,
+		 const struct config *cfg,
+		 struct sdp_session *sdp_sess, int label,
+		 const struct mnat *mnat, struct mnat_sess *mnat_sess,
+		 const struct menc *menc, struct menc_sess *menc_sess,
+		 const char *content, const struct list *vidcodecl,
+		 const struct list *vidfiltl, bool offerer,
+		 video_err_h *errh, void *arg);
+int  video_encoder_set(struct video *v, struct vidcodec *vc,
+		       int pt_tx, const char *params);
+int  video_start_source(struct video *v, struct media_ctx **ctx);
+int  video_start_display(struct video *v, const char *peer);
+void video_stop(struct video *v);
+void video_stop_display(struct video *v);
 int   video_set_fullscreen(struct video *v, bool fs);
-int   video_set_orient(struct video *v, int orient);
 void  video_vidsrc_set_device(struct video *v, const char *dev);
 int   video_set_source(struct video *v, const char *name, const char *dev);
 void  video_set_devicename(struct video *v, const char *src, const char *disp);
-void  video_encoder_cycle(struct video *video);
 int   video_debug(struct re_printf *pf, const struct video *v);
 uint64_t video_calc_rtp_timestamp_fix(uint64_t timestamp);
 double video_calc_seconds(uint64_t rtp_ts);
 struct stream *video_strm(const struct video *v);
 double video_timestamp_to_seconds(uint64_t timestamp);
 uint64_t video_calc_timebase_timestamp(uint64_t rtp_ts);
+const struct vidcodec *video_codec(const struct video *vid, bool tx);
+void video_sdp_attr_decode(struct video *v);
 
 
 /*
@@ -1181,13 +1264,19 @@ uint64_t video_calc_timebase_timestamp(uint64_t rtp_ts);
 
 /** Common parameters for media stream */
 struct stream_param {
-	bool use_rtp;
-	int af;
-	const char *cname;
+	bool use_rtp;       /**< Enable or disable RTP */
+	int af;             /**< Wanted address family */
+	const char *cname;  /**< Canonical name        */
 };
+
+typedef void (stream_mnatconn_h)(struct stream *strm, void *arg);
+typedef void (stream_rtpestab_h)(struct stream *strm, void *arg);
+typedef void (stream_rtcp_h)(struct stream *strm,
+			     struct rtcp_msg *msg, void *arg);
+typedef void (stream_error_h)(struct stream *strm, int err, void *arg);
+
 void stream_update(struct stream *s);
 const struct rtcp_stats *stream_rtcp_stats(const struct stream *strm);
-struct call *stream_call(const struct stream *strm);
 struct sdp_media *stream_sdpmedia(const struct stream *s);
 uint32_t stream_metric_get_tx_n_packets(const struct stream *strm);
 uint32_t stream_metric_get_tx_n_bytes(const struct stream *strm);
@@ -1195,6 +1284,43 @@ uint32_t stream_metric_get_tx_n_err(const struct stream *strm);
 uint32_t stream_metric_get_rx_n_packets(const struct stream *strm);
 uint32_t stream_metric_get_rx_n_bytes(const struct stream *strm);
 uint32_t stream_metric_get_rx_n_err(const struct stream *strm);
+void stream_set_secure(struct stream *strm, bool secure);
+bool stream_is_secure(const struct stream *strm);
+int  stream_start_mediaenc(struct stream *strm);
+int  stream_start(const struct stream *strm);
+void stream_set_session_handlers(struct stream *strm,
+				 stream_mnatconn_h *mnatconnh,
+				 stream_rtpestab_h *rtpestabh,
+				 stream_rtcp_h *rtcph,
+				 stream_error_h *errorh, void *arg);
+const char *stream_name(const struct stream *strm);
+int  stream_debug(struct re_printf *pf, const struct stream *s);
+
+
+/*
+ * STUN URI
+ */
+
+/** Defines the STUN uri scheme */
+enum stun_scheme {
+	STUN_SCHEME_STUN,  /**< STUN scheme        */
+	STUN_SCHEME_STUNS, /**< Secure STUN scheme */
+	STUN_SCHEME_TURN,  /**< TURN scheme        */
+	STUN_SCHEME_TURNS, /**< Secure TURN scheme */
+};
+
+/** Defines a STUN/TURN uri */
+struct stun_uri {
+	enum stun_scheme scheme;  /**< STUN Scheme            */
+	char *host;               /**< Hostname or IP-address */
+	uint16_t port;            /**< Port number            */
+};
+
+int stunuri_decode(struct stun_uri **sup, const struct pl *pl);
+int stunuri_set_host(struct stun_uri *su, const char *host);
+int stunuri_set_port(struct stun_uri *su, uint16_t port);
+int stunuri_print(struct re_printf *pf, const struct stun_uri *su);
+const char *stunuri_scheme_name(enum stun_scheme scheme);
 
 
 /*
@@ -1208,22 +1334,37 @@ struct mnat_media;
 typedef void (mnat_estab_h)(int err, uint16_t scode, const char *reason,
 			    void *arg);
 
-typedef int (mnat_sess_h)(struct mnat_sess **sessp, struct dnsc *dnsc,
-			  int af, const char *srv, uint16_t port,
+typedef void (mnat_connected_h)(const struct sa *raddr1,
+				const struct sa *raddr2, void *arg);
+
+
+typedef int (mnat_sess_h)(struct mnat_sess **sessp,
+			  const struct mnat *mnat, struct dnsc *dnsc,
+			  int af, const struct stun_uri *srv,
 			  const char *user, const char *pass,
 			  struct sdp_session *sdp, bool offerer,
 			  mnat_estab_h *estabh, void *arg);
 
 typedef int (mnat_media_h)(struct mnat_media **mp, struct mnat_sess *sess,
-			   int proto, void *sock1, void *sock2,
-			   struct sdp_media *sdpm);
+			   struct udp_sock *sock1, struct udp_sock *sock2,
+			   struct sdp_media *sdpm,
+			   mnat_connected_h *connh, void *arg);
 
 typedef int (mnat_update_h)(struct mnat_sess *sess);
 
-int mnat_register(struct mnat **mnatp, struct list *mnatl,
-		  const char *id, const char *ftag,
-		  mnat_sess_h *sessh, mnat_media_h *mediah,
-		  mnat_update_h *updateh);
+struct mnat {
+	struct le le;
+	const char *id;
+	const char *ftag;
+	bool wait_connected;
+	mnat_sess_h *sessh;
+	mnat_media_h *mediah;
+	mnat_update_h *updateh;
+};
+
+void mnat_register(struct list *mnatl, struct mnat *mnat);
+void mnat_unregister(struct mnat *mnat);
+const struct mnat *mnat_find(const struct list *mnatl, const char *id);
 
 
 /*
@@ -1233,9 +1374,6 @@ int mnat_register(struct mnat **mnatp, struct list *mnatl,
 bool sdp_media_has_media(const struct sdp_media *m);
 int  sdp_fingerprint_decode(const char *attr, struct pl *hash,
 			    uint8_t *md, size_t *sz);
-uint32_t sdp_media_rattr_u32(const struct sdp_media *sdpm, const char *name);
-const char *sdp_rattr(const struct sdp_session *s, const struct sdp_media *m,
-		      const char *name);
 
 
 /*
@@ -1319,7 +1457,7 @@ int h264_nal_send(bool first, bool last,
 const char *h264_nalunit_name(int type);
 static inline bool h264_is_keyframe(int type)
 {
-	return type == H264_NAL_SPS;
+	return type == H264_NAL_IDR_SLICE;
 }
 
 
@@ -1335,25 +1473,18 @@ static inline bool h264_is_keyframe(int type)
 
 
 int  module_preload(const char *module);
-int  module_load(const char *name);
+int  module_load(const char *path, const char *name);
 void module_unload(const char *name);
 void module_app_unload(void);
 
 
-/*
- * MOS (Mean Opinion Score)
- */
-
-double mos_calculate(double *r_factor, double rtt,
-		     double jitter, uint32_t num_packets_lost);
-
-
 /*
  * Generic event
  */
 
 int event_encode_dict(struct odict *od, struct ua *ua, enum ua_event ev,
 		      struct call *call, const char *prm);
+int event_add_au_jb_stat(struct odict *od_parent, const struct call *call);
 
 
 /*
@@ -1367,7 +1498,7 @@ uint64_t tmr_jiffies_usec(void);
  * Baresip instance
  */
 
-int  baresip_init(struct config *cfg, bool prefer_ipv6);
+int  baresip_init(struct config *cfg);
 void baresip_close(void);
 struct network *baresip_network(void);
 struct contacts *baresip_contacts(void);
