@@ -205,15 +205,17 @@ static int decode_update(struct audec_state **adsp,
 
 
 #ifdef AMR_WB
-static int encode_wb(struct auenc_state *st, uint8_t *buf, size_t *len,
+static int encode_wb(struct auenc_state *st,
+		     bool *marker, uint8_t *buf, size_t *len,
 		     int fmt, const void *sampv, size_t sampc)
 {
 	int n;
+	(void)marker;
 
 	if (sampc != L_FRAME16k)
 		return EINVAL;
 
-	if (*len < NB_SERIAL_MAX)
+	if (*len < (1+NB_SERIAL_MAX))
 		return ENOMEM;
 
 	if (fmt != AUFMT_S16LE)
@@ -234,11 +236,13 @@ static int encode_wb(struct auenc_state *st, uint8_t *buf, size_t *len,
 
 static int decode_wb(struct audec_state *st,
 		     int fmt, void *sampv, size_t *sampc,
-		     const uint8_t *buf, size_t len)
+		     bool marker, const uint8_t *buf, size_t len)
 {
+	(void)marker;
+
 	if (*sampc < L_FRAME16k)
 		return ENOMEM;
-	if (len > NB_SERIAL_MAX)
+	if (len > (1+NB_SERIAL_MAX))
 		return EINVAL;
 
 	if (fmt != AUFMT_S16LE)
@@ -254,10 +258,11 @@ static int decode_wb(struct audec_state *st,
 
 
 #ifdef AMR_NB
-static int encode_nb(struct auenc_state *st, uint8_t *buf,
+static int encode_nb(struct auenc_state *st, bool *marker, uint8_t *buf,
 		     size_t *len, int fmt, const void *sampv, size_t sampc)
 {
 	int r;
+	(void)marker;
 
 	if (!st || !buf || !len || !sampv || sampc != FRAMESIZE_NB)
 		return EINVAL;
@@ -281,8 +286,11 @@ static int encode_nb(struct auenc_state *st, uint8_t *buf,
 
 
 static int decode_nb(struct audec_state *st, int fmt, void *sampv,
-		     size_t *sampc, const uint8_t *buf, size_t len)
+		     size_t *sampc,
+		     bool marker, const uint8_t *buf, size_t len)
 {
+	(void)marker;
+
 	if (!st || !sampv || !sampc || !buf)
 		return EINVAL;
 
@@ -306,18 +314,32 @@ static int decode_nb(struct audec_state *st, int fmt, void *sampv,
 
 #ifdef AMR_WB
 static struct aucodec amr_wb = {
-	LE_INIT, NULL, "AMR-WB", 16000, 16000, 1, 1, NULL,
-	encode_update, encode_wb,
-	decode_update, decode_wb,
-	NULL, amr_fmtp_enc, amr_fmtp_cmp
+	.name      = "AMR-WB",
+	.srate     = 16000,
+	.crate     = 16000,
+	.ch        = 1,
+	.pch       = 1,
+	.encupdh   = encode_update,
+	.ench      = encode_wb,
+	.decupdh   = decode_update,
+	.dech      = decode_wb,
+	.fmtp_ench = amr_fmtp_enc,
+	.fmtp_cmph = amr_fmtp_cmp
 };
 #endif
 #ifdef AMR_NB
 static struct aucodec amr_nb = {
-	LE_INIT, NULL, "AMR", 8000, 8000, 1, 1, NULL,
-	encode_update, encode_nb,
-	decode_update, decode_nb,
-	NULL, amr_fmtp_enc, amr_fmtp_cmp
+	.name      = "AMR",
+	.srate     = 8000,
+	.crate     = 8000,
+	.ch        = 1,
+	.pch       = 1,
+	.encupdh   = encode_update,
+	.ench      = encode_nb,
+	.decupdh   = decode_update,
+	.dech      = decode_nb,
+	.fmtp_ench = amr_fmtp_enc,
+	.fmtp_cmph = amr_fmtp_cmp
 };
 #endif
 
