@@ -24,18 +24,18 @@ struct auplay_st {
 };
 
 
-extern struct hash *ht_device;
+extern struct hash *aubridge_ht_device;
 
 
-int play_alloc(struct auplay_st **stp, const struct auplay *ap,
+int aubridge_play_alloc(struct auplay_st **stp, const struct auplay *ap,
 	       struct auplay_prm *prm, const char *device,
 	       auplay_write_h *wh, void *arg);
-int src_alloc(struct ausrc_st **stp, const struct ausrc *as,
+int aubridge_src_alloc(struct ausrc_st **stp, const struct ausrc *as,
 	      struct media_ctx **ctx,
 	      struct ausrc_prm *prm, const char *device,
 	      ausrc_read_h *rh, ausrc_error_h *errh, void *arg);
 
 
-int  device_connect(struct device **devp, const char *device,
-		    struct auplay_st *auplay, struct ausrc_st *ausrc);
-void device_stop(struct device *dev);
+int  aubridge_device_connect(struct device **devp, const char *device,
+			     struct auplay_st *auplay, struct ausrc_st *ausrc);
+void aubridge_device_stop(struct device *dev);
