@@ -54,32 +54,36 @@ static void *device_thread(void *arg)
 {
 	uint64_t now, ts = tmr_jiffies();
 	struct device *dev = arg;
-	struct auresamp rs;
 	int16_t *sampv_in, *sampv_out;
 	size_t sampc_in;
 	size_t sampc_out;
-	int err;
+	size_t sampsz;
 
-	if (!dev->run) {
+	if (!dev->run)
+		return NULL;
+
+	if (dev->auplay->prm.srate != dev->ausrc->prm.srate ||
+	    dev->auplay->prm.ch != dev->ausrc->prm.ch ||
+	    dev->auplay->prm.fmt != dev->ausrc->prm.fmt) {
+
+		warning("aubridge: incompatible ausrc/auplay parameters\n");
 		return NULL;
 	}
 
+	info("aubridge: thread start: %u Hz, %u channels, format=%s\n",
+	     dev->auplay->prm.srate, dev->auplay->prm.ch,
+	     aufmt_name(dev->auplay->prm.fmt));
+
 	sampc_in = dev->auplay->prm.srate * dev->auplay->prm.ch * PTIME/1000;
 	sampc_out = dev->ausrc->prm.srate * dev->ausrc->prm.ch * PTIME/1000;
 
-	auresamp_init(&rs);
+	sampsz = aufmt_sample_size(dev->auplay->prm.fmt);
 
-	sampv_in  = mem_alloc(sizeof(int16_t) * sampc_in, NULL);
-	sampv_out = mem_alloc(sizeof(int16_t) * sampc_out, NULL);
+	sampv_in  = mem_alloc(sampsz * sampc_in, NULL);
+	sampv_out = mem_alloc(sampsz * sampc_out, NULL);
 	if (!sampv_in || !sampv_out)
 		goto out;
 
-	err = auresamp_setup(&rs,
-			     dev->auplay->prm.srate, dev->auplay->prm.ch,
-			     dev->ausrc->prm.srate, dev->ausrc->prm.ch);
-	if (err)
-		goto out;
-
 	while (dev->run) {
 
 		(void)sys_msleep(4);
@@ -92,30 +96,18 @@ static void *device_thread(void *arg)
 		if (ts > now)
 			continue;
 
-		if (dev->auplay && dev->auplay->wh) {
+		if (dev->auplay->wh) {
 			dev->auplay->wh(sampv_in, sampc_in, dev->auplay->arg);
 		}
 
-		if (rs.resample) {
-			err = auresamp(&rs,
-				       sampv_out, &sampc_out,
-				       sampv_in, sampc_in);
-			if (err) {
-				warning("aubridge: auresamp error"
-					" sampc_out=%zu, sampc_in=%zu (%m)\n",
-					sampc_out, sampc_in, err);
-			}
-
-			if (dev->ausrc && dev->ausrc->rh) {
-				dev->ausrc->rh(sampv_out, sampc_out,
-					       dev->ausrc->arg);
-			}
-		}
-		else {
-			if (dev->ausrc && dev->ausrc->rh) {
-				dev->ausrc->rh(sampv_in, sampc_in,
-					       dev->ausrc->arg);
-			}
+		if (dev->ausrc->rh) {
+			struct auframe af = {
+				.fmt   = dev->ausrc->prm.fmt,
+				.sampv = sampv_in,
+				.sampc = sampc_in,
+				.timestamp = ts * 1000
+			};
+			dev->ausrc->rh(&af, dev->ausrc->arg);
 		}
 
 		ts += PTIME;
@@ -183,11 +175,11 @@ void aubridge_device_stop(struct device *dev)
 	if (!dev)
 		return;
 
-	dev->auplay = NULL;
-	dev->ausrc = NULL;
-
 	if (dev->run) {
 		dev->run = false;
 		pthread_join(dev->thread, NULL);
 	}
+
+	dev->auplay = NULL;
+	dev->ausrc = NULL;
 }
