@@ -32,30 +32,38 @@
 
 /** Audio Loop */
 struct audio_loop {
-	struct aubuf *ab;
+	struct aubuf *aubuf;
 	struct ausrc_st *ausrc;
 	struct auplay_st *auplay;
-	const struct aucodec *ac;
-	struct auenc_state *enc;
-	struct audec_state *dec;
-	int16_t *sampv;
-	size_t sampc;
+	struct lock *lock;
 	struct tmr tmr;
 	uint32_t srate;
-	uint32_t ch;
+	uint8_t ch;
 	enum aufmt fmt;
 	bool started;
 
-	uint64_t n_read;
-	uint64_t n_write;
+	size_t aubuf_maxsz;
+	uint64_t aubuf_overrun;
+	uint64_t aubuf_underrun;
+
+	struct stats {
+		uint64_t n_samp;
+		uint64_t n_frames;
+	} stats_src, stats_play;
 };
 
 
 static struct audio_loop *gal = NULL;
-static char aucodec[64];
 
 
-static int print_summary(struct re_printf *pf, const struct audio_loop *al)
+static inline uint32_t calc_nsamp(uint32_t srate, uint8_t channels,
+				  uint16_t ptime)
+{
+	return srate * channels * ptime / 1000;
+}
+
+
+static int print_summary(struct re_printf *pf, struct audio_loop *al)
 {
 	const double scale = al->srate * al->ch;
 	int err;
@@ -67,33 +75,68 @@ static int print_summary(struct re_printf *pf, const struct audio_loop *al)
 	/* Source */
 	if (al->ausrc) {
 		struct ausrc *as = ausrc_get(al->ausrc);
+		const char *name = as->name;
+		const struct stats *stats = &al->stats_src;
+		double dur;
+
+		al->ausrc = mem_deref(al->ausrc);
+
+		dur = (double)stats->n_samp / scale;
 
 		err |= re_hprintf(pf,
 				  "* Source\n"
 				  "  module      %s\n"
 				  "  samples     %llu\n"
 				  "  duration    %.3f sec\n"
+				  "  frames      %llu (avg ptime %.2fms)\n"
 				  "\n"
 				  ,
-				  as->name,
-				  al->n_read,
-				  (double)al->n_read / scale);
+				  name,
+				  stats->n_samp,
+				  dur,
+				  al->stats_src.n_frames,
+				  1000.0*dur / (double)al->stats_src.n_frames
+				  );
+	}
+
+	if (al->aubuf) {
+
+		err |= re_hprintf(pf,
+				  "* Aubuf\n"
+				  "  overrun     %llu\n"
+				  "  underrun    %llu\n"
+				  "\n"
+				  ,
+				  al->aubuf_overrun,
+				  al->aubuf_underrun);
 	}
 
 	/* Player */
 	if (al->auplay) {
 		struct auplay *ap = auplay_get(al->auplay);
+		const char *name = ap->name;
+		const struct stats *stats = &al->stats_play;
+		double dur;
+
+		/* stop device first */
+		al->auplay = mem_deref(al->auplay);
+
+		dur = (double)stats->n_samp / scale;
 
 		err |= re_hprintf(pf,
 				  "* Player\n"
 				  "  module      %s\n"
 				  "  samples     %llu\n"
 				  "  duration    %.3f sec\n"
+				  "  frames      %llu (avg ptime %.2fms)\n"
 				  "\n"
 				  ,
-				  ap->name,
-				  al->n_write,
-				  (double)al->n_write / scale);
+				  name,
+				  stats->n_samp,
+				  dur,
+				  stats->n_frames,
+				  1000.0*dur / (double)stats->n_frames
+				  );
 	}
 
 	return err;
@@ -110,36 +153,36 @@ static void auloop_destructor(void *arg)
 	tmr_cancel(&al->tmr);
 	mem_deref(al->ausrc);
 	mem_deref(al->auplay);
-	mem_deref(al->sampv);
-	mem_deref(al->ab);
-	mem_deref(al->enc);
-	mem_deref(al->dec);
+	mem_deref(al->aubuf);
+	mem_deref(al->lock);
 }
 
 
 static void print_stats(struct audio_loop *al)
 {
 	double rw_ratio = 0.0;
-	double delay = (double)al->n_read - (double)al->n_write;
+	double delay;
 	const double scale = al->srate * al->ch;
 
-	if (al->n_write)
-		rw_ratio = 1.0 * (double)al->n_read / (double)al->n_write;
+	lock_read_get(al->lock);
+
+	delay = (double)al->stats_src.n_samp - (double)al->stats_play.n_samp;
+
+	rw_ratio = (double)al->stats_src.n_samp/(double)al->stats_play.n_samp;
 
 	(void)re_fprintf(stdout, "\r%uHz %dch %s "
 			 " n_read=%.3f n_write=%.3f rw_delay=%.3f [sec]"
 			 " rw_ratio=%f"
 			 ,
 			 al->srate, al->ch, aufmt_name(al->fmt),
-			 (double)al->n_read / scale,
-			 (double)al->n_write / scale,
+			 (double)al->stats_src.n_samp / scale,
+			 (double)al->stats_play.n_samp / scale,
 			 delay / scale, rw_ratio);
 
-	if (str_isset(aucodec))
-		(void)re_fprintf(stdout, " codec='%s'", aucodec);
-
 	(void)re_fprintf(stdout, "          \r");
 
+	lock_rel(al->lock);
+
 	fflush(stdout);
 }
 
@@ -153,44 +196,30 @@ static void tmr_handler(void *arg)
 }
 
 
-static int codec_read(struct audio_loop *al, int16_t *sampv, size_t sampc)
-{
-	uint8_t x[2560];
-	size_t xlen = sizeof(x);
-	int err;
-
-	aubuf_read_samp(al->ab, al->sampv, al->sampc);
-
-	err = al->ac->ench(al->enc, x, &xlen,
-			   AUFMT_S16LE, al->sampv, al->sampc);
-	if (err)
-		goto out;
-
-	if (al->ac->dech) {
-		err = al->ac->dech(al->dec, AUFMT_S16LE, sampv, &sampc,
-				   x, xlen);
-		if (err)
-			goto out;
-	}
-	else {
-		info("auloop: no decode handler\n");
-	}
-
- out:
-
-	return err;
-}
-
-
-static void read_handler(const void *sampv, size_t sampc, void *arg)
+static void src_read_handler(struct auframe *af, void *arg)
 {
 	struct audio_loop *al = arg;
-	size_t num_bytes = sampc * aufmt_sample_size(al->fmt);
+	struct stats *stats = &al->stats_src;
 	int err;
 
-	al->n_read += sampc;
+	if (af->fmt != (int)al->fmt) {
+		warning("auloop: format mismatch: exp=%d, actual=%d\n",
+			al->fmt, af->fmt);
+		return;
+	}
 
-	err = aubuf_write(al->ab, sampv, num_bytes);
+	lock_write_get(al->lock);
+
+	stats->n_samp   += af->sampc;
+	stats->n_frames += 1;
+
+	if (aubuf_cur_size(al->aubuf) >= al->aubuf_maxsz) {
+		++al->aubuf_overrun;
+	}
+
+	lock_rel(al->lock);
+
+	err = aubuf_write(al->aubuf, af->sampv, auframe_size(af));
 	if (err) {
 		warning("auloop: aubuf_write: %m\n", err);
 	}
@@ -201,21 +230,21 @@ static void write_handler(void *sampv, size_t sampc, void *arg)
 {
 	struct audio_loop *al = arg;
 	size_t num_bytes = sampc * aufmt_sample_size(al->fmt);
-	int err;
+	struct stats *stats = &al->stats_play;
 
-	al->n_write += sampc;
+	lock_write_get(al->lock);
+
+	stats->n_samp   += sampc;
+	stats->n_frames += 1;
+
+	if (stats->n_samp && aubuf_cur_size(al->aubuf) < num_bytes) {
+		++al->aubuf_underrun;
+	}
+
+	lock_rel(al->lock);
 
 	/* read from beginning */
-	if (al->ac) {
-		err = codec_read(al, sampv, sampc);
-		if (err) {
-			warning("auloop: codec_read error "
-				"on %zu samples (%m)\n", sampc, err);
-		}
-	}
-	else {
-		aubuf_read(al->ab, sampv, num_bytes);
-	}
+	aubuf_read(al->aubuf, sampv, num_bytes);
 }
 
 
@@ -227,39 +256,12 @@ static void error_handler(int err, const char *str, void *arg)
 }
 
 
-static void start_codec(struct audio_loop *al, const char *name,
-			uint32_t srate, uint32_t ch)
-{
-	struct auenc_param prm = {PTIME, 0};
-	int err;
-
-	al->ac = aucodec_find(baresip_aucodecl(), name, srate, ch);
-	if (!al->ac) {
-		warning("auloop: could not find codec: %s\n", name);
-		return;
-	}
-
-	if (al->ac->encupdh) {
-		err = al->ac->encupdh(&al->enc, al->ac, &prm, NULL);
-		if (err) {
-			warning("auloop: encoder update failed: %m\n", err);
-		}
-	}
-
-	if (al->ac->decupdh) {
-		err = al->ac->decupdh(&al->dec, al->ac, NULL);
-		if (err) {
-			warning("auloop: decoder update failed: %m\n", err);
-		}
-	}
-}
-
-
-static int auloop_reset(struct audio_loop *al, uint32_t srate, uint32_t ch)
+static int auloop_reset(struct audio_loop *al, uint32_t srate, uint8_t ch)
 {
 	struct auplay_prm auplay_prm;
 	struct ausrc_prm ausrc_prm;
 	const struct config *cfg = conf_config();
+	size_t min_sz, sampsz;
 	int err;
 
 	if (!cfg)
@@ -273,37 +275,24 @@ static int auloop_reset(struct audio_loop *al, uint32_t srate, uint32_t ch)
 
 	al->fmt = cfg->audio.src_fmt;
 
-	/* Optional audio codec */
-	if (str_isset(aucodec)) {
-		if (cfg->audio.src_fmt != AUFMT_S16LE) {
-			warning("auloop: only s16 supported with codec\n");
-			return EINVAL;
-		}
-
-		start_codec(al, aucodec, srate, ch);
-	}
-
 	/* audio player/source must be stopped first */
 	al->auplay = mem_deref(al->auplay);
 	al->ausrc  = mem_deref(al->ausrc);
 
-	al->sampv  = mem_deref(al->sampv);
-	al->ab     = mem_deref(al->ab);
+	al->aubuf  = mem_deref(al->aubuf);
 
 	al->srate = srate;
 	al->ch    = ch;
 
-	if (str_isset(aucodec)) {
-		al->sampc = al->srate * al->ch * PTIME / 1000;
-		al->sampv = mem_alloc(al->sampc * 2, NULL);
-		if (!al->sampv)
-			return ENOMEM;
-	}
-
 	info("Audio-loop: %uHz, %dch, %s\n", al->srate, al->ch,
 	     aufmt_name(al->fmt));
 
-	err = aubuf_alloc(&al->ab, 320, 0);
+	sampsz = aufmt_sample_size(al->fmt);
+
+	min_sz = sampsz * calc_nsamp(al->srate, al->ch, PTIME);
+	al->aubuf_maxsz = sampsz * calc_nsamp(al->srate, al->ch, PTIME*5);
+
+	err = aubuf_alloc(&al->aubuf, min_sz, al->aubuf_maxsz);
 	if (err)
 		return err;
 
@@ -325,10 +314,11 @@ static int auloop_reset(struct audio_loop *al, uint32_t srate, uint32_t ch)
 	ausrc_prm.ch         = al->ch;
 	ausrc_prm.ptime      = PTIME;
 	ausrc_prm.fmt        = al->fmt;
+
 	err = ausrc_alloc(&al->ausrc, baresip_ausrcl(),
 			  NULL, cfg->audio.src_mod,
 			  &ausrc_prm, cfg->audio.src_dev,
-			  read_handler, error_handler, al);
+			  src_read_handler, error_handler, al);
 	if (err) {
 		warning("auloop: ausrc %s,%s failed: %m\n", cfg->audio.src_mod,
 			cfg->audio.src_dev, err);
@@ -340,7 +330,7 @@ static int auloop_reset(struct audio_loop *al, uint32_t srate, uint32_t ch)
 
 
 static int audio_loop_alloc(struct audio_loop **alp,
-			    uint32_t srate, uint32_t ch)
+			    uint32_t srate, uint8_t ch)
 {
 	struct audio_loop *al;
 	int err;
@@ -349,6 +339,10 @@ static int audio_loop_alloc(struct audio_loop **alp,
 	if (!al)
 		return ENOMEM;
 
+	err = lock_alloc(&al->lock);
+	if (err)
+		goto out;
+
 	tmr_start(&al->tmr, 100, tmr_handler, al);
 
 	err = auloop_reset(al, srate, ch);
@@ -393,7 +387,7 @@ static int auloop_start(struct re_printf *pf, void *arg)
 	if (!srate || !ch)
 		return re_hprintf(pf, "invalid samplerate or channels\n");
 
-	err = audio_loop_alloc(&gal, srate, ch);
+	err = audio_loop_alloc(&gal, srate, (uint8_t)ch);
 	if (err) {
 		warning("auloop: alloc failed %m\n", err);
 	}
@@ -423,8 +417,6 @@ static const struct cmd cmdv[] = {
 
 static int module_init(void)
 {
-	conf_get_str(conf_cur(), "auloop_codec", aucodec, sizeof(aucodec));
-
 	return cmd_register(baresip_commands(), cmdv, ARRAY_SIZE(cmdv));
 }
 
