@@ -9,8 +9,8 @@
 int av1_encode_update(struct videnc_state **vesp, const struct vidcodec *vc,
 		      struct videnc_param *prm, const char *fmtp,
 		      videnc_packet_h *pkth, void *arg);
-int av1_encode(struct videnc_state *ves, bool update,
-	       const struct vidframe *frame, uint64_t timestamp);
+int av1_encode_packet(struct videnc_state *ves, bool update,
+		      const struct vidframe *frame, uint64_t timestamp);
 
 
 /* Decode */
