@@ -62,15 +62,14 @@ static void call_event_handler(struct call *call, enum call_event ev,
 	case CALL_EVENT_ESTABLISHED:
 		debug("b2bua: CALL_ESTABLISHED: peer_uri=%s\n",
 		      call_peeruri(call));
-		ua_answer(call_get_ua(call2), call2);
+		call_answer(call2, 200,
+			    call_has_video(call) ? VIDMODE_ON : VIDMODE_OFF);
 		break;
 
 	case CALL_EVENT_CLOSED:
 		debug("b2bua: CALL_CLOSED: %s\n", str);
 
-		mem_ref(call2);
-
-		ua_hangup(call_get_ua(call2), call2, call_scode(call), "");
+		call_hangup(call2, call_scode(call), "");
 		mem_deref(sess);
 		break;
 
@@ -137,6 +136,7 @@ static void ua_event_handler(struct ua *ua, enum ua_event ev,
 			     struct call *call, const char *prm, void *arg)
 {
 	int err;
+	(void)ua;
 	(void)prm;
 	(void)arg;
 
@@ -148,7 +148,7 @@ static void ua_event_handler(struct ua *ua, enum ua_event ev,
 
 		err = new_session(call);
 		if (err) {
-			ua_hangup(ua, call, 500, "Server Error");
+			call_hangup(call, 500, "Server Error");
 		}
 		break;
 
