@@ -21,64 +21,6 @@ static struct auplay *auplay;
 static struct ausrc *ausrc;
 
 
-#if TARGET_OS_IPHONE && __IPHONE_OS_VERSION_MIN_REQUIRED >= __IPHONE_2_0
-static void interruptionListener(void *data, UInt32 inInterruptionState)
-{
-	(void)data;
-
-	if (inInterruptionState == kAudioSessionBeginInterruption) {
-		debug("coreaudio: player interrupt: Begin\n");
-	}
-	else if (inInterruptionState == kAudioSessionEndInterruption) {
-		debug("coreaudio: player interrupt: End\n");
-	}
-}
-
-
-int audio_session_enable(void)
-{
-	OSStatus res;
-	UInt32 category;
-
-	res = AudioSessionInitialize(NULL, NULL, interruptionListener, 0);
-	if (res && res != 1768843636)
-		return ENODEV;
-
-	category = kAudioSessionCategory_PlayAndRecord;
-	res = AudioSessionSetProperty(kAudioSessionProperty_AudioCategory,
-				      sizeof(category), &category);
-	if (res) {
-		warning("coreaudio: Audio Category: %d\n", res);
-		return ENODEV;
-	}
-
-	res = AudioSessionSetActive(true);
-	if (res) {
-		warning("coreaudio: AudioSessionSetActive: %d\n", res);
-		return ENODEV;
-	}
-
-	return 0;
-}
-
-
-void audio_session_disable(void)
-{
-	AudioSessionSetActive(false);
-}
-#else
-int audio_session_enable(void)
-{
-	return 0;
-}
-
-
-void audio_session_disable(void)
-{
-}
-#endif
-
-
 int coreaudio_enum_devices(const char *name, struct list *dev_list,
 			    CFStringRef *uid, Boolean is_input)
 {
@@ -150,6 +92,8 @@ int coreaudio_enum_devices(const char *name, struct list *dev_list,
 		CFStringRef deviceUID = NULL;
 		CFStringRef deviceName = NULL;
 		const char *name_str;
+		/* fallback if CFStringGetCStringPtr fails */
+		char name_buf[64];
 
 		propertyAddress.mSelector   = kAudioDevicePropertyStreams;
 		status = AudioObjectGetPropertyDataSize(audioDevices[i],
@@ -195,6 +139,20 @@ int coreaudio_enum_devices(const char *name, struct list *dev_list,
 		name_str = CFStringGetCStringPtr(deviceName,
 						 kCFStringEncodingUTF8);
 
+		/* CFStringGetCStringPtr can and does fail
+		 * (documented behavior) */
+		if (0 == name_str) {
+			if (!CFStringGetCString(deviceName,
+						name_buf,
+						sizeof(name_buf),
+						kCFStringEncodingUTF8)) {
+				warning("CFStringGetCString "
+					" failed: %i\n", status);
+				continue;
+			}
+			name_str = name_buf;
+		}
+
 		if (uid) {
 			if (0 == str_casecmp(name, name_str)) {
 				*uid = deviceUID;
@@ -220,6 +178,7 @@ uint32_t coreaudio_aufmt_to_formatflags(enum aufmt fmt)
 	switch (fmt) {
 
 	case AUFMT_S16LE:  return kLinearPCMFormatFlagIsSignedInteger;
+	case AUFMT_S24_3LE:return kLinearPCMFormatFlagIsSignedInteger;
 	case AUFMT_FLOAT:  return kLinearPCMFormatFlagIsFloat;
 	default: return 0;
 	}
