@@ -5,10 +5,6 @@
  */
 
 
-int  audio_session_enable(void);
-void audio_session_disable(void);
-
-
 int coreaudio_player_alloc(struct auplay_st **stp, const struct auplay *ap,
 			   struct auplay_prm *prm, const char *device,
 			   auplay_write_h *wh, void *arg);
