@@ -12,11 +12,7 @@
 
 
 /* This value can be tuned */
-#if TARGET_OS_IPHONE
-#define BUFC 20
-#else
 #define BUFC 6
-#endif
 
 
 struct auplay_st {
@@ -39,8 +35,6 @@ static void auplay_destructor(void *arg)
 	st->wh = NULL;
 	pthread_mutex_unlock(&st->mutex);
 
-	audio_session_disable();
-
 	if (st->queue) {
 		AudioQueuePause(st->queue);
 		AudioQueueStop(st->queue, true);
@@ -108,10 +102,6 @@ int coreaudio_player_alloc(struct auplay_st **stp, const struct auplay *ap,
 	if (err)
 		goto out;
 
-	err = audio_session_enable();
-	if (err)
-		goto out;
-
 	fmt.mSampleRate       = (Float64)prm->srate;
 	fmt.mFormatID         = kAudioFormatLinearPCM;
 	fmt.mFormatFlags      = coreaudio_aufmt_to_formatflags(prm->fmt) |
