@@ -17,10 +17,13 @@
 
 struct ausrc_st {
 	const struct ausrc *as;      /* inheritance */
+
 	AudioQueueRef queue;
 	AudioQueueBufferRef buf[BUFC];
 	pthread_mutex_t mutex;
+	struct ausrc_prm prm;
 	uint32_t sampsz;
+	int fmt;
 	ausrc_read_h *rh;
 	void *arg;
 };
@@ -35,8 +38,6 @@ static void ausrc_destructor(void *arg)
 	st->rh = NULL;
 	pthread_mutex_unlock(&st->mutex);
 
-	audio_session_disable();
-
 	if (st->queue) {
 		AudioQueuePause(st->queue);
 		AudioQueueStop(st->queue, true);
@@ -59,6 +60,7 @@ static void record_handler(void *userData, AudioQueueRef inQ,
 			   const AudioStreamPacketDescription *inPacketDesc)
 {
 	struct ausrc_st *st = userData;
+	struct auframe af;
 	ausrc_read_h *rh;
 	void *arg;
 	(void)inStartTime;
@@ -73,7 +75,12 @@ static void record_handler(void *userData, AudioQueueRef inQ,
 	if (!rh)
 		return;
 
-	rh(inQB->mAudioData, inQB->mAudioDataByteSize/st->sampsz, arg);
+	auframe_init(&af, st->fmt, inQB->mAudioData,
+		     inQB->mAudioDataByteSize/st->sampsz);
+
+	af.timestamp = AUDIO_TIMEBASE*inStartTime->mSampleTime / st->prm.srate;
+
+	rh(&af, arg);
 
 	AudioQueueEnqueueBuffer(inQ, inQB, 0, NULL);
 }
@@ -112,15 +119,13 @@ int coreaudio_recorder_alloc(struct ausrc_st **stp, const struct ausrc *as,
 
 	sampc = prm->srate * prm->ch * prm->ptime / 1000;
 	bytc  = sampc * st->sampsz;
+	st->fmt = prm->fmt;
+	st->prm = *prm;
 
 	err = pthread_mutex_init(&st->mutex, NULL);
 	if (err)
 		goto out;
 
-	err = audio_session_enable();
-	if (err)
-		goto out;
-
 	fmt.mSampleRate       = (Float64)prm->srate;
 	fmt.mFormatID         = kAudioFormatLinearPCM;
 	fmt.mFormatFlags      = coreaudio_aufmt_to_formatflags(prm->fmt) |
