@@ -16,12 +16,13 @@
  */
 
 
-static int pcmu_encode(struct auenc_state *aes, uint8_t *buf,
+static int pcmu_encode(struct auenc_state *aes, bool *marker, uint8_t *buf,
 		       size_t *len, int fmt, const void *sampv, size_t sampc)
 {
 	const int16_t *p = sampv;
 
 	(void)aes;
+	(void)marker;
 
 	if (!buf || !len || !sampv)
 		return EINVAL;
@@ -42,11 +43,13 @@ static int pcmu_encode(struct auenc_state *aes, uint8_t *buf,
 
 
 static int pcmu_decode(struct audec_state *ads, int fmt, void *sampv,
-		       size_t *sampc, const uint8_t *buf, size_t len)
+		       size_t *sampc, bool marker,
+		       const uint8_t *buf, size_t len)
 {
 	int16_t *p = sampv;
 
 	(void)ads;
+	(void)marker;
 
 	if (!sampv || !sampc || !buf)
 		return EINVAL;
@@ -66,12 +69,13 @@ static int pcmu_decode(struct audec_state *ads, int fmt, void *sampv,
 }
 
 
-static int pcma_encode(struct auenc_state *aes, uint8_t *buf,
+static int pcma_encode(struct auenc_state *aes, bool *marker, uint8_t *buf,
 		       size_t *len, int fmt, const void *sampv, size_t sampc)
 {
 	const int16_t *p = sampv;
 
 	(void)aes;
+	(void)marker;
 
 	if (!buf || !len || !sampv)
 		return EINVAL;
@@ -92,11 +96,13 @@ static int pcma_encode(struct auenc_state *aes, uint8_t *buf,
 
 
 static int pcma_decode(struct audec_state *ads, int fmt, void *sampv,
-		       size_t *sampc, const uint8_t *buf, size_t len)
+		       size_t *sampc, bool marker,
+		       const uint8_t *buf, size_t len)
 {
 	int16_t *p = sampv;
 
 	(void)ads;
+	(void)marker;
 
 	if (!sampv || !sampc || !buf)
 		return EINVAL;
@@ -117,13 +123,25 @@ static int pcma_decode(struct audec_state *ads, int fmt, void *sampv,
 
 
 static struct aucodec pcmu = {
-	LE_INIT, "0", "PCMU", 8000, 8000, 1, 1, NULL,
-	NULL, pcmu_encode, NULL, pcmu_decode, NULL, NULL, NULL
+	.pt    = "0",
+	.name  = "PCMU",
+	.srate = 8000,
+	.crate = 8000,
+	.ch    = 1,
+	.pch   = 1,
+	.ench  = pcmu_encode,
+	.dech  = pcmu_decode,
 };
 
 static struct aucodec pcma = {
-	LE_INIT, "8", "PCMA", 8000, 8000, 1, 1, NULL,
-	NULL, pcma_encode, NULL, pcma_decode, NULL, NULL, NULL
+	.pt    = "8",
+	.name  = "PCMA",
+	.srate = 8000,
+	.crate = 8000,
+	.ch    = 1,
+	.pch   = 1,
+	.ench  = pcma_encode,
+	.dech  = pcma_decode,
 };
 
 
