@@ -12,7 +12,8 @@ struct g7221_aucodec {
 /* Encode */
 int g7221_encode_update(struct auenc_state **aesp, const struct aucodec *ac,
 			struct auenc_param *prm, const char *fmtp);
-int g7221_encode(struct auenc_state *aes, uint8_t *buf, size_t *len,
+int g7221_encode(struct auenc_state *aes,
+		 bool *marker, uint8_t *buf, size_t *len,
 		 int fmt, const void *sampv, size_t sampc);
 
 
@@ -21,7 +22,7 @@ int g7221_decode_update(struct audec_state **adsp, const struct aucodec *ac,
 			const char *fmtp);
 int g7221_decode(struct audec_state *ads,
 		 int fmt, void *sampv, size_t *sampc,
-		 const uint8_t *buf, size_t len);
+		 bool marker, const uint8_t *buf, size_t len);
 
 
 /* SDP */
