@@ -1,5 +1,5 @@
 /**
- * @file gst_video/gst_video.c  Video codecs using Gstreamer
+ * @file gst_video/gst_video.c  Video codecs using Gstreamer 1.0
  *
  * Copyright (C) 2010 Creytiv.com
  * Copyright (C) 2014 Fadeev Alexander
@@ -15,7 +15,7 @@
 /**
  * @defgroup gst_video gst_video
  *
- * This module implements video codecs using Gstreamer.
+ * This module implements video codecs using Gstreamer 1.0
  *
  * Currently only H.264 encoding is supported, but this can be extended
  * if needed. No decoding is done by this module, so that must be done by
@@ -29,7 +29,7 @@
 static struct vidcodec h264 = {
 	.name      = "H264",
 	.variant   = "packetization-mode=0",
-	.encupdh   = gst_video_encode_update,
+	.encupdh   = gst_video_encoder_set,
 	.ench      = gst_video_encode,
 	.fmtp_ench = gst_video_fmtp_enc,
 	.fmtp_cmph = gst_video_fmtp_cmp,
@@ -42,7 +42,7 @@ static int module_init(void)
 
 	vidcodec_register(baresip_vidcodecl(), &h264);
 
-	info("gst_video: using gstreamer H.264 encoder\n");
+	info("gst_video: using gstreamer (%s)\n", gst_version_string());
 
 	return 0;
 }
@@ -52,6 +52,8 @@ static int module_close(void)
 {
 	vidcodec_unregister(&h264);
 
+	gst_deinit();
+
 	return 0;
 }
 
