@@ -9,12 +9,12 @@
 /* Encode */
 struct videnc_state;
 
-int gst_video_encode_update(struct videnc_state **vesp,
-			    const struct vidcodec *vc,
-			    struct videnc_param *prm, const char *fmtp,
-			    videnc_packet_h *pkth, void *arg);
+int gst_video_encoder_set(struct videnc_state **stp,
+			  const struct vidcodec *vc,
+			  struct videnc_param *prm, const char *fmtp,
+			  videnc_packet_h *pkth, void *arg);
 int gst_video_encode(struct videnc_state *st, bool update,
-		     const struct vidframe *frame, uint64_t timestamp);
+		      const struct vidframe *frame, uint64_t timestamp);
 
 
 /* SDP */
