@@ -9,7 +9,7 @@
 #include "gst_video.h"
 
 
-static const uint8_t h264_level_idc = 0x0c;
+static const uint8_t gst_video_h264_level_idc = 0x0c;
 
 
 uint32_t gst_video_h264_packetization_mode(const char *fmtp)
@@ -43,7 +43,8 @@ int gst_video_fmtp_enc(struct mbuf *mb, const struct sdp_format *fmt,
 			   " packetization-mode=0"
 			   ";profile-level-id=%02x%02x%02x"
 			   "\r\n",
-			   fmt->id, profile_idc, profile_iop, h264_level_idc);
+			   fmt->id, profile_idc, profile_iop,
+			   gst_video_h264_level_idc);
 }
 
 
