@@ -19,6 +19,50 @@ struct dial_dialog {
 };
 
 
+static int clean_number(char* str)
+{
+	int i = 0, k = 0;
+
+	/* only clean numeric numbers
+	 * In other cases trust the user input
+	 */
+	int err = re_regex(str, sizeof(str), "[A-Za-z]");
+	if (err == 0)
+		return -1;
+
+	/* remove (0) which is in some mal-formated numbers
+	 * but only if trailed by another character
+	 */
+	if (str[0] == '+' || (str[0] == '0' && str[1] == '0'))
+		while (str[i]) {
+			if (str[i] == '('
+			 && str[i+1] == '0'
+			 && str[i+2] == ')'
+			 && (str[i+3] == ' '
+				 || (str[i+3] >= '0' && str[i+3] <= '9')
+			    )
+			) {
+				str[i+1] = ' ';
+				break;
+			}
+			++i;
+		}
+	i = 0;
+	while (str[i]) {
+		if (str[i] == ' '
+		 || str[i] == '.'
+		 || str[i] == '-'
+		 || str[i] == '/'
+		 || str[i] == '('
+		 || str[i] == ')')
+			++i;
+		else
+			str[k++] = str[i++];
+	}
+	str[k] = '\0';
+	return k;
+}
+
 static void dial_dialog_on_response(GtkDialog *dialog, gint response_id,
 				    gpointer arg)
 {
@@ -27,6 +71,12 @@ static void dial_dialog_on_response(GtkDialog *dialog, gint response_id,
 
 	if (response_id == GTK_RESPONSE_ACCEPT) {
 		uri = (char *)uri_combo_box_get_text(dd->uri_combobox);
+		if (gtk_mod_clean_number(dd->mod)) {
+			int length = clean_number(uri);
+			if (length >= 0)
+				uri_combo_box_set_text(dd->uri_combobox,
+					uri, length);
+		}
 		gtk_mod_connect(dd->mod, uri);
 	}
 
