@@ -42,7 +42,6 @@ struct gtk_mod {
 	pthread_t thread;
 	bool run;
 	bool contacts_inited;
-	struct message_lsnr *message;
 	struct mqueue *mq;
 	GApplication *app;
 	GtkStatusIcon *status_icon;
@@ -54,6 +53,7 @@ struct gtk_mod {
 	struct dial_dialog *dial_dialog;
 	GSList *call_windows;
 	GSList *incoming_call_menus;
+	bool clean_number;
 };
 
 static struct gtk_mod mod_obj;
@@ -72,26 +72,17 @@ static void reject_activated(GSimpleAction *, GVariant *, gpointer);
 static void denotify_incoming_call(struct gtk_mod *, struct call *);
 
 static GActionEntry app_entries[] = {
-	{"answer", answer_activated, "x", NULL, NULL, {0} },
-	{"reject", reject_activated, "x", NULL, NULL, {0} },
+	{"answer", answer_activated, "s", NULL, NULL, {0} },
+	{"reject", reject_activated, "s", NULL, NULL, {0} },
 };
 
 
 static struct call *get_call_from_gvariant(GVariant *param)
 {
-	gint64 call_ptr;
-	struct call *call;
 	struct list *calls = ua_calls(uag_current());
-	struct le *le;
+	const gchar *call_ptr = g_variant_get_string(param, NULL);
 
-	call_ptr = g_variant_get_int64(param);
-	call = GINT_TO_POINTER(call_ptr);
-
-	for (le = list_head(calls); le; le = le->next)
-		if (le->data == call)
-			return call;
-
-	return NULL;
+	return call_find_id(calls, call_ptr);
 }
 
 
@@ -310,12 +301,23 @@ static void accounts_menu_set_status(struct gtk_mod *mod,
 static void notify_incoming_call(struct gtk_mod *mod,
 		struct call *call)
 {
-	static const char *title = "Incoming call";
+	char title[128];
 	const char *msg = call_peeruri(call);
 	GtkWidget *call_menu;
 	GtkWidget *menu_item;
+
 #if defined(USE_LIBNOTIFY)
 	NotifyNotification *notification;
+#elif GLIB_CHECK_VERSION(2,40,0)
+	char id[64];
+	GVariant *target;
+	GNotification *notification;
+#endif
+
+	re_snprintf(title, sizeof title, "Incoming call from %s",
+					call_peername(call));
+
+#if defined(USE_LIBNOTIFY)
 
 	if (!notify_is_initted())
 		return;
@@ -325,9 +327,7 @@ static void notify_incoming_call(struct gtk_mod *mod,
 	g_object_unref(notification);
 
 #elif GLIB_CHECK_VERSION(2,40,0)
-	char id[64];
-	GVariant *target;
-	GNotification *notification = g_notification_new(title);
+	notification = g_notification_new(title);
 
 	re_snprintf(id, sizeof id, "incoming-call-%p", call);
 	id[sizeof id - 1] = '\0';
@@ -339,7 +339,7 @@ static void notify_incoming_call(struct gtk_mod *mod,
 	g_notification_set_urgent(notification, TRUE);
 #endif
 
-	target = g_variant_new_int64(GPOINTER_TO_INT(call));
+	target = g_variant_new_string(call_id(call));
 	g_notification_set_body(notification, msg);
 	g_notification_add_button_with_target_value(notification,
 			"Answer", "app.answer", target);
@@ -633,6 +633,13 @@ void gtk_mod_connect(struct gtk_mod *mod, const char *uri)
 	mqueue_push(mod->mq, MQ_CONNECT, (char *)uri);
 }
 
+bool gtk_mod_clean_number(struct gtk_mod *mod)
+{
+	if (!mod)
+		return false;
+	return mod->clean_number;
+}
+
 
 static void warning_dialog(const char *title, const char *fmt, ...)
 {
@@ -701,7 +708,7 @@ static void mqueue_handler(int id, void *data, void *arg)
 
 	case MQ_ANSWER:
 		call = data;
-		err = ua_answer(ua, call);
+		err = ua_answer(ua, call, VIDMODE_ON);
 		if (err) {
 			gdk_threads_enter();
 			warning_dialog("Call failed",
@@ -960,22 +967,22 @@ static int vu_decode_update(struct aufilt_dec_st **stp, void **ctx,
 }
 
 
-static int vu_encode(struct aufilt_enc_st *st, void *sampv, size_t *sampc)
+static int vu_encode(struct aufilt_enc_st *st, struct auframe *af)
 {
 	struct vumeter_enc *vu = (struct vumeter_enc *)st;
 
-	vu->avg_rec = calc_avg_s16(sampv, *sampc);
+	vu->avg_rec = calc_avg_s16(af->sampv, af->sampc);
 	vu->started = true;
 
 	return 0;
 }
 
 
-static int vu_decode(struct aufilt_dec_st *st, void *sampv, size_t *sampc)
+static int vu_decode(struct aufilt_dec_st *st, struct auframe *af)
 {
 	struct vumeter_dec *vu = (struct vumeter_dec *)st;
 
-	vu->avg_play = calc_avg_s16(sampv, *sampc);
+	vu->avg_play = calc_avg_s16(af->sampv, af->sampc);
 	vu->started = true;
 
 	return 0;
@@ -983,9 +990,11 @@ static int vu_decode(struct aufilt_dec_st *st, void *sampv, size_t *sampc)
 
 
 static struct aufilt vumeter = {
-	LE_INIT, "gtk_vumeter",
-	vu_encode_update, vu_encode,
-	vu_decode_update, vu_decode
+	.name    = "gtk_vumeter",
+	.encupdh = vu_encode_update,
+	.ench    = vu_encode,
+	.decupdh = vu_decode_update,
+	.dech    = vu_decode
 };
 
 
@@ -1007,7 +1016,10 @@ static const struct cmd cmdv[] = {
 
 static int module_init(void)
 {
-	int err = 0;
+	int err;
+
+	mod_obj.clean_number = false;
+	conf_get_bool(conf_cur(), "gtk_clean_number", &mod_obj.clean_number);
 
 	err = mqueue_alloc(&mod_obj.mq, mqueue_handler, &mod_obj);
 	if (err)
@@ -1016,7 +1028,7 @@ static int module_init(void)
 	aufilt_register(baresip_aufiltl(), &vumeter);
 
 #ifdef USE_NOTIFICATIONS
-	err = message_listen(&mod_obj.message, baresip_message(),
+	err = message_listen(baresip_message(),
 			     message_handler, &mod_obj);
 	if (err) {
 		warning("gtk: message_init failed (%m)\n", err);
@@ -1050,7 +1062,7 @@ static int module_close(void)
 		pthread_join(mod_obj.thread, NULL);
 	mod_obj.mq = mem_deref(mod_obj.mq);
 	aufilt_unregister(&vumeter);
-	mod_obj.message = mem_deref(mod_obj.message);
+	message_unlisten(baresip_message(), message_handler);
 
 #ifdef USE_LIBNOTIFY
 	if (notify_is_initted())
