@@ -49,6 +49,7 @@ struct menc_sess {
 
 struct menc_media {
 	Stream *stream;
+	const struct stream *strm;  /**< pointer to parent */
 };
 
 
@@ -101,15 +102,18 @@ static int session_alloc(struct menc_sess **sessp, struct sdp_session *sdp,
 
 static int media_alloc(struct menc_media **stp, struct menc_sess *sess,
                        struct rtp_sock *rtp,
-                       int proto, void *rtpsock, void *rtcpsock,
-                       struct sdp_media *sdpm)
+                       struct udp_sock *rtpsock, struct udp_sock *rtcpsock,
+ 		       const struct sa *raddr_rtp,
+		       const struct sa *raddr_rtcp,
+		       struct sdp_media *sdpm,
+		       const struct stream *strm)
 {
 	struct menc_media *st;
 	int err = 0;
 	StreamMediaType med_type;
 	const char *med_name;
 
-	if (!stp || !sess || !sess->session || proto != IPPROTO_UDP)
+	if (!stp || !sess || !sess->session)
 		return EINVAL;
 
 	st = *stp;
@@ -134,6 +138,7 @@ static int media_alloc(struct menc_media **stp, struct menc_sess *sess,
 	else
 		med_type = MT_UNKNOWN;
 
+	st->strm = strm;
 	st->stream = sess->session->create_stream(
 	                       *s_zrtp_config,
 	                       (struct udp_sock *)rtpsock,
@@ -168,7 +173,10 @@ static int media_alloc(struct menc_media **stp, struct menc_sess *sess,
 
 
 static struct menc menc_zrtp = {
-	LE_INIT, "zrtp", "RTP/AVP", session_alloc, media_alloc
+	.id        = "zrtp",
+	.sdp_proto = "RTP/AVP",
+	.sessh     = session_alloc,
+	.mediah    = media_alloc
 };
 
 
