@@ -14,7 +14,7 @@ struct auplay_st {
 	const struct auplay *ap;  /* pointer to base-class (inheritance) */
 
 	struct auplay_prm prm;
-	int16_t *sampv;
+	float *sampv;
 	size_t sampc;             /* includes number of channels */
 	auplay_write_h *wh;
 	void *arg;
@@ -25,16 +25,6 @@ struct auplay_st {
 };
 
 
-static inline float ausamp_short2float(int16_t in)
-{
-	float out;
-
-	out = (float) (in / (1.0 * 0x8000));
-
-	return out;
-}
-
-
 /**
  * The process callback for this JACK application is called in a
  * special realtime thread once for each audio cycle.
@@ -64,8 +54,8 @@ static int process_handler(jack_nframes_t nframes, void *arg)
 		buffer = jack_port_get_buffer(st->portv[ch], st->nframes);
 
 		for (j = 0; j < nframes; j++) {
-			int16_t samp = st->sampv[j*st->prm.ch + ch];
-			buffer[j] = ausamp_short2float(samp);
+			float samp = st->sampv[j*st->prm.ch + ch];
+			buffer[j] = samp;
 		}
 	}
 
@@ -86,9 +76,9 @@ static void auplay_destructor(void *arg)
 	mem_deref(st->portv);
 }
 
-
 static int start_jack(struct auplay_st *st)
 {
+	struct conf *conf = conf_cur();
 	const char **ports;
 	const char *client_name = "baresip";
 	const char *server_name = NULL;
@@ -97,10 +87,25 @@ static int start_jack(struct auplay_st *st)
 	unsigned ch;
 	jack_nframes_t engine_srate;
 
+	bool jack_connect_ports = true;
+	(void)conf_get_bool(conf, "jack_connect_ports",
+				  &jack_connect_ports);
+
 	/* open a client connection to the JACK server */
+	size_t len = jack_client_name_size();
+	char *conf_name = mem_alloc(len+1, NULL);
+
+	if (!conf_get_str(conf, "jack_client_name",
+			conf_name, len)) {
+		st->client = jack_client_open(conf_name, options,
+						&status, server_name);
+	}
+	else {
+		st->client = jack_client_open(client_name,
+			options, &status, server_name);
+	}
+	mem_deref(conf_name);
 
-	st->client = jack_client_open(client_name, options,
-				      &status, server_name);
 	if (st->client == NULL) {
 		warning("jack: jack_client_open() failed, "
 			"status = 0x%2.0x\n", status);
@@ -113,10 +118,8 @@ static int start_jack(struct auplay_st *st)
 	if (status & JackServerStarted) {
 		info("jack: JACK server started\n");
 	}
-	if (status & JackNameNotUnique) {
-		client_name = jack_get_client_name(st->client);
-		info("jack: unique name `%s' assigned\n", client_name);
-	}
+	client_name = jack_get_client_name(st->client);
+	info("jack: source unique name `%s' assigned\n", client_name);
 
 	jack_set_process_callback(st->client, process_handler, st);
 
@@ -134,7 +137,7 @@ static int start_jack(struct auplay_st *st)
 	}
 
 	st->sampc = st->nframes * st->prm.ch;
-	st->sampv = mem_alloc(st->sampc * sizeof(int16_t), NULL);
+	st->sampv = mem_alloc(st->sampc * sizeof(float), NULL);
 	if (!st->sampv)
 		return ENOMEM;
 
@@ -169,22 +172,35 @@ static int start_jack(struct auplay_st *st)
 	 * it.
 	 */
 
-	ports = jack_get_ports (st->client, NULL, NULL,
-				JackPortIsInput);
-	if (ports == NULL) {
-		warning("jack: no physical playback ports\n");
-		return ENODEV;
-	}
-
-	for (ch=0; ch<st->prm.ch; ch++) {
-
-		if (jack_connect (st->client, jack_port_name (st->portv[ch]),
-				  ports[ch])) {
-			warning("jack: cannot connect output ports\n");
+	if (jack_connect_ports) {
+		info("jack: connecting default input ports\n");
+		ports = jack_get_ports (st->client, NULL, NULL,
+					JackPortIsInput | JackPortIsPhysical);
+		if (ports == NULL) {
+			warning("jack: no physical playback ports\n");
+			return ENODEV;
 		}
-	}
 
-	jack_free(ports);
+		/* Connect all physical ports. In case of for example mono
+		 * audio with 2 physical playback ports, connect the
+		 * single registered port to both physical port.
+		 */
+		ch = 0;
+		for (unsigned i = 0; ports[i] != NULL; i++) {
+			if (jack_connect (st->client,
+					jack_port_name (st->portv[ch]),
+						ports[i])) {
+				warning("jack: cannot connect output ports\n");
+			}
+
+			++ch;
+			if (ch >= st->prm.ch) {
+				ch = 0;
+			}
+		}
+
+		jack_free(ports);
+	}
 
 	return 0;
 }
@@ -204,7 +220,7 @@ int jack_play_alloc(struct auplay_st **stp, const struct auplay *ap,
 
 	info("jack: play %uHz,%uch\n", prm->srate, prm->ch);
 
-	if (prm->fmt != AUFMT_S16LE) {
+	if (prm->fmt != AUFMT_FLOAT) {
 		warning("jack: playback: unsupported sample format (%s)\n",
 			aufmt_name(prm->fmt));
 		return ENOTSUP;
