@@ -15,7 +15,7 @@ struct ausrc_st {
 	const struct ausrc *as;  /* pointer to base-class (inheritance) */
 
 	struct ausrc_prm prm;
-	int16_t *sampv;
+	float *sampv;
 	size_t sampc;             /* includes number of channels */
 	ausrc_read_h *rh;
 	void *arg;
@@ -26,31 +26,15 @@ struct ausrc_st {
 };
 
 
-static inline int16_t ausamp_float2short(float in)
-{
-	double scaled_value;
-	int16_t out;
-
-	scaled_value = in * (8.0 * 0x10000000);
-
-	if (scaled_value >= (1.0 * 0x7fffffff)) {
-		out = 32767;
-	}
-	else if (scaled_value <= (-8.0 * 0x10000000)) {
-		out = -32768;
-	}
-	else
-		out = (short) (lrint (scaled_value) >> 16);
-
-	return out;
-}
-
-
 static int process_handler(jack_nframes_t nframes, void *arg)
 {
 	struct ausrc_st *st = arg;
+	struct auframe af;
 	size_t sampc = nframes * st->prm.ch;
 	size_t ch, j;
+	uint64_t ts;
+
+	ts = jack_frames_to_time(st->client, jack_last_frame_time(st->client));
 
 	/* 2. convert from 16-bit to float and copy to Jack */
 
@@ -62,14 +46,18 @@ static int process_handler(jack_nframes_t nframes, void *arg)
 		buffer = jack_port_get_buffer(st->portv[ch], st->nframes);
 
 		for (j = 0; j < nframes; j++) {
-			int16_t samp;
-			samp = ausamp_float2short(buffer[j]);
+			float samp = buffer[j];
 			st->sampv[j*st->prm.ch + ch] = samp;
 		}
 	}
 
+	af.fmt   = st->prm.fmt;
+	af.sampv = st->sampv;
+	af.sampc = sampc;
+	af.timestamp = ts;
+
 	/* 1. read data from app (signed 16-bit) interleaved */
-	st->rh(st->sampv, sampc, st->arg);
+	st->rh(&af, st->arg);
 
 	return 0;
 }
@@ -91,6 +79,7 @@ static void ausrc_destructor(void *arg)
 
 static int start_jack(struct ausrc_st *st)
 {
+	struct conf *conf = conf_cur();
 	const char **ports;
 	const char *client_name = "baresip";
 	const char *server_name = NULL;
@@ -99,10 +88,25 @@ static int start_jack(struct ausrc_st *st)
 	unsigned ch;
 	jack_nframes_t engine_srate;
 
+	bool jack_connect_ports = true;
+	(void)conf_get_bool(conf, "jack_connect_ports",
+				&jack_connect_ports);
+
 	/* open a client connection to the JACK server */
+	size_t len = jack_client_name_size();
+	char *conf_name = mem_alloc(len+1, NULL);
+
+	if (!conf_get_str(conf, "jack_client_name",
+			conf_name, len)) {
+		st->client = jack_client_open(conf_name, options,
+						&status, server_name);
+	}
+	else {
+		st->client = jack_client_open(client_name, options,
+							&status, server_name);
+	}
+	mem_deref(conf_name);
 
-	st->client = jack_client_open(client_name, options,
-				      &status, server_name);
 	if (st->client == NULL) {
 		warning("jack: jack_client_open() failed, "
 			"status = 0x%2.0x\n", status);
@@ -115,10 +119,8 @@ static int start_jack(struct ausrc_st *st)
 	if (status & JackServerStarted) {
 		info("jack: JACK server started\n");
 	}
-	if (status & JackNameNotUnique) {
-		client_name = jack_get_client_name(st->client);
-		info("jack: unique name `%s' assigned\n", client_name);
-	}
+	client_name = jack_get_client_name(st->client);
+	info("jack: destination unique name `%s' assigned\n", client_name);
 
 	jack_set_process_callback(st->client, process_handler, st);
 
@@ -136,7 +138,7 @@ static int start_jack(struct ausrc_st *st)
 	}
 
 	st->sampc = st->nframes * st->prm.ch;
-	st->sampv = mem_alloc(st->sampc * sizeof(int16_t), NULL);
+	st->sampv = mem_alloc(st->sampc * sizeof(float), NULL);
 	if (!st->sampv)
 		return ENOMEM;
 
@@ -163,22 +165,24 @@ static int start_jack(struct ausrc_st *st)
 		return ENODEV;
 	}
 
-	ports = jack_get_ports (st->client, NULL, NULL,
-				JackPortIsOutput);
-	if (ports == NULL) {
-		warning("jack: no physical playback ports\n");
-		return ENODEV;
-	}
-
-	for (ch=0; ch<st->prm.ch; ch++) {
-
-		if (jack_connect(st->client, ports[ch],
-				 jack_port_name(st->portv[ch]))) {
-			warning("jack: cannot connect output ports\n");
+	if (jack_connect_ports) {
+		info("jack: connecting default output ports\n");
+		ports = jack_get_ports (st->client, NULL, NULL,
+					JackPortIsOutput | JackPortIsPhysical);
+		if (ports == NULL) {
+			warning("jack: no physical playback ports\n");
+			return ENODEV;
 		}
-	}
 
-	jack_free(ports);
+		for (ch=0; ch<st->prm.ch; ch++) {
+			if (jack_connect(st->client, ports[ch],
+					 jack_port_name(st->portv[ch]))) {
+				warning("jack: cannot connect output ports\n");
+			}
+		}
+
+		jack_free(ports);
+	}
 
 	return 0;
 }
@@ -199,7 +203,7 @@ int jack_src_alloc(struct ausrc_st **stp, const struct ausrc *as,
 	if (!stp || !as || !prm || !rh)
 		return EINVAL;
 
-	if (prm->fmt != AUFMT_S16LE) {
+	if (prm->fmt != AUFMT_FLOAT) {
 		warning("jack: source: unsupported sample format (%s)\n",
 			aufmt_name(prm->fmt));
 		return ENOTSUP;
