@@ -30,7 +30,6 @@ static struct {
 	struct tmr tmr_alert;         /**< Incoming call alert timer      */
 	struct tmr tmr_stat;          /**< Call status timer              */
 	struct play *play;            /**< Current audio player state     */
-	struct message_lsnr *message; /**< Message listener               */
 	struct mbuf *dialbuf;         /**< Buffer for dialled number      */
 	struct le *le_cur;            /**< Current User-Agent (struct ua) */
 	bool bell;                    /**< ANSI Bell alert enabled        */
@@ -41,6 +40,8 @@ static struct {
 	uint32_t current_attempts;    /**< Current number of re-dials     */
 	uint64_t start_ticks;         /**< Ticks when app started         */
 	enum statmode statmode;       /**< Status mode                    */
+	bool clean_number;            /**< Remove -/() from diald numbers */
+	char redial_aor[128];
 } menu;
 
 
@@ -49,9 +50,7 @@ static void update_callstatus(void);
 static void alert_stop(void);
 static int switch_audio_source(struct re_printf *pf, void *arg);
 static int switch_audio_player(struct re_printf *pf, void *arg);
-#ifdef USE_VIDEO
 static int switch_video_source(struct re_printf *pf, void *arg);
-#endif
 
 
 static void redial_reset(void)
@@ -102,17 +101,6 @@ static void check_registrations(void)
 }
 
 
-/**
- * Return the current User-Agent in focus
- *
- * @return Current User-Agent
- */
-static struct ua *uag_cur(void)
-{
-	return uag_current();
-}
-
-
 /* Return TRUE if there are any active calls for any UAs */
 static bool have_active_calls(void)
 {
@@ -145,13 +133,13 @@ static int ua_print_reg_status(struct re_printf *pf, void *unused)
 
 	(void)unused;
 
-	err = re_hprintf(pf, "\n--- Useragents: %u ---\n",
+	err = re_hprintf(pf, "\n--- User Agents (%u) ---\n",
 			 list_count(uag_list()));
 
 	for (le = list_head(uag_list()); le && !err; le = le->next) {
 		const struct ua *ua = le->data;
 
-		err  = re_hprintf(pf, "%s ", ua == uag_cur() ? ">" : " ");
+		err  = re_hprintf(pf, "%s ", ua == uag_current() ? ">" : " ");
 		err |= ua_print_status(pf, ua);
 	}
 
@@ -161,6 +149,36 @@ static int ua_print_reg_status(struct re_printf *pf, void *unused)
 }
 
 
+static int cmd_set_answermode(struct re_printf *pf, void *arg)
+{
+	enum answermode mode;
+	const struct cmd_arg *carg = arg;
+	int err;
+
+	if (0 == str_cmp(carg->prm, "manual")) {
+		mode = ANSWERMODE_MANUAL;
+	}
+	else if (0 == str_cmp(carg->prm, "early")) {
+		mode = ANSWERMODE_EARLY;
+	}
+	else if (0 == str_cmp(carg->prm, "auto")) {
+		mode = ANSWERMODE_AUTO;
+	}
+	else {
+		(void)re_hprintf(pf, "Invalid answer mode: %s\n", carg->prm);
+		return EINVAL;
+	}
+
+	err = account_set_answermode(ua_account(uag_current()), mode);
+	if (err)
+		return err;
+
+	(void)re_hprintf(pf, "Answer mode changed to: %s\n", carg->prm);
+
+	return 0;
+}
+
+
 /**
  * Print the current SIP Call status for the current User-Agent
  *
@@ -176,7 +194,7 @@ static int ua_print_call_status(struct re_printf *pf, void *unused)
 
 	(void)unused;
 
-	call = ua_call(uag_cur());
+	call = ua_call(uag_current());
 	if (call) {
 		err  = re_hprintf(pf, "\n%H\n", call_debug, call);
 	}
@@ -188,6 +206,49 @@ static int ua_print_call_status(struct re_printf *pf, void *unused)
 }
 
 
+static void clean_number(char *str)
+{
+	int i = 0, k = 0;
+
+	/* only clean numeric numbers
+	 * In other cases trust the user input
+	 */
+	int err = re_regex(str, sizeof(str), "[A-Za-z]");
+	if (err == 0)
+		return;
+
+	/* remove (0) which is in some mal-formated numbers
+	 * but only if trailed by another character
+	 */
+	if (str[0] == '+' || (str[0] == '0' && str[1] == '0'))
+		while (str[i]) {
+			if (str[i] == '('
+			 && str[i+1] == '0'
+			 && str[i+2] == ')'
+			 && (str[i+3] == ' '
+				 || (str[i+3] >= '0' && str[i+3] <= '9')
+			    )
+			) {
+				str[i+1] = ' ';
+				break;
+			}
+			++i;
+		}
+	i = 0;
+	while (str[i]) {
+		if (str[i] == ' '
+		 || str[i] == '.'
+		 || str[i] == '-'
+		 || str[i] == '/'
+		 || str[i] == '('
+		 || str[i] == ')')
+			++i;
+		else
+			str[k++] = str[i++];
+	}
+	str[k] = '\0';
+}
+
 static int dial_handler(struct re_printf *pf, void *arg)
 {
 	const struct cmd_arg *carg = arg;
@@ -199,8 +260,10 @@ static int dial_handler(struct re_printf *pf, void *arg)
 
 		mbuf_rewind(menu.dialbuf);
 		(void)mbuf_write_str(menu.dialbuf, carg->prm);
+		if (menu.clean_number)
+			clean_number(carg->prm);
 
-		err = ua_connect(uag_cur(), NULL, NULL,
+		err = ua_connect(uag_current(), NULL, NULL,
 				 carg->prm, VIDMODE_ON);
 	}
 	else if (menu.dialbuf->end > 0) {
@@ -211,8 +274,10 @@ static int dial_handler(struct re_printf *pf, void *arg)
 		err = mbuf_strdup(menu.dialbuf, &uri, menu.dialbuf->end);
 		if (err)
 			return err;
+		if (menu.clean_number)
+			clean_number(uri);
 
-		err = ua_connect(uag_cur(), NULL, NULL, uri, VIDMODE_ON);
+		err = ua_connect(uag_current(), NULL, NULL, uri, VIDMODE_ON);
 
 		mem_deref(uri);
 	}
@@ -241,12 +306,12 @@ static void options_resp_handler(int err, const struct sip_msg *msg, void *arg)
 
 		mbuf_set_pos(msg->mb, 0);
 		info("----- OPTIONS of %r -----\n%b",
-		     &(msg->to.auri), mbuf_buf(msg->mb),
+		     &msg->to.auri, mbuf_buf(msg->mb),
 		     mbuf_get_left(msg->mb));
 		return;
 	}
 
-	info("%r: OPTIONS failed: %u %r\n", &(msg->to.auri),
+	info("%r: OPTIONS failed: %u %r\n", &msg->to.auri,
 	     msg->scode, &msg->reason);
 }
 
@@ -258,29 +323,8 @@ static int options_command(struct re_printf *pf, void *arg)
 
 	(void)pf;
 
-	if (str_isset(carg->prm)) {
-
-		mbuf_rewind(menu.dialbuf);
-		(void)mbuf_write_str(menu.dialbuf, carg->prm);
-
-		err = ua_options_send(uag_cur(), carg->prm,
-				      options_resp_handler, NULL);
-	}
-	else if (menu.dialbuf->end > 0) {
-
-		char *uri;
-
-		menu.dialbuf->pos = 0;
-		err = mbuf_strdup(menu.dialbuf, &uri, menu.dialbuf->end);
-		if (err)
-			return err;
-
-		err = ua_options_send(uag_cur(), uri,
-				      options_resp_handler, NULL);
-
-		mem_deref(uri);
-	}
-
+	err = ua_options_send(uag_current(), carg->prm,
+			      options_resp_handler, NULL);
 	if (err) {
 		warning("menu: ua_options failed: %m\n", err);
 	}
@@ -291,7 +335,7 @@ static int options_command(struct re_printf *pf, void *arg)
 
 static int cmd_answer(struct re_printf *pf, void *unused)
 {
-	struct ua *ua = uag_cur();
+	struct ua *ua = uag_current();
 	int err;
 	(void)unused;
 
@@ -300,7 +344,7 @@ static int cmd_answer(struct re_printf *pf, void *unused)
 	/* Stop any ongoing ring-tones */
 	menu.play = mem_deref(menu.play);
 
-	ua_hold_answer(ua, NULL);
+	ua_hold_answer(ua, NULL, VIDMODE_ON);
 
 	return err;
 }
@@ -315,7 +359,7 @@ static int cmd_hangup(struct re_printf *pf, void *unused)
 	menu.play = mem_deref(menu.play);
 	alert_stop();
 
-	ua_hangup(uag_cur(), NULL, 0, NULL);
+	ua_hangup(uag_current(), NULL, 0, NULL);
 
 	/* note: must be called after ua_hangup() */
 	menu_set_incall(have_active_calls());
@@ -328,7 +372,6 @@ static int create_ua(struct re_printf *pf, void *arg)
 {
 	const struct cmd_arg *carg = arg;
 	struct ua *ua = NULL;
-	struct le *le;
 	int err = 0;
 
 	if (str_isset(carg->prm)) {
@@ -343,14 +386,7 @@ static int create_ua(struct re_printf *pf, void *arg)
 		(void)ua_register(ua);
 	}
 
-	for (le = list_head(uag_list()); le && !err; le = le->next) {
-		ua = le->data;
-
-		err  = re_hprintf(pf, "%s ", ua == uag_cur() ? ">" : " ");
-		err |= ua_print_status(pf, ua);
-	}
-
-	err |= re_hprintf(pf, "\n");
+	err = ua_print_reg_status(pf, NULL);
 
  out:
 	if (err) {
@@ -410,6 +446,31 @@ static int cmd_ua_find(struct re_printf *pf, void *arg)
 }
 
 
+static int cmd_ua_delete(struct re_printf *pf, void *arg)
+{
+	const struct cmd_arg *carg = arg;
+	struct ua *ua = NULL;
+
+	if (str_isset(carg->prm)) {
+		ua = uag_find_aor(carg->prm);
+	}
+
+	if (!ua) {
+		return ENOENT;
+	}
+
+	if (ua == uag_current()) {
+		(void)cmd_ua_next(pf, NULL);
+	}
+
+	(void)re_hprintf(pf, "deleting ua: %s\n", carg->prm);
+	mem_deref(ua);
+	(void)ua_print_reg_status(pf, NULL);
+
+	return 0;
+}
+
+
 static int print_commands(struct re_printf *pf, void *unused)
 {
 	(void)unused;
@@ -420,7 +481,7 @@ static int print_commands(struct re_printf *pf, void *unused)
 static int cmd_print_calls(struct re_printf *pf, void *unused)
 {
 	(void)unused;
-	return ua_print_calls(pf, uag_cur());
+	return ua_print_calls(pf, uag_current());
 }
 
 
@@ -436,7 +497,7 @@ static const char about_fmt[] =
 	"| with audio and video support                               |\n"
 	"|                                                            |\n"
 	"| License:   BSD                                             |\n"
-	"| Homepage:  https://github.com/alfredh/baresip              |\n"
+	"| Homepage:  https://github.com/baresip/baresip              |\n"
 	"|                                                            |\n"
 	"'------------------------------------------------------------'\n"
 	;
@@ -460,16 +521,16 @@ static const struct cmd cmdv[] = {
 {"listcalls", 'l',        0, "List active calls",       cmd_print_calls      },
 {"options",   'o',  CMD_PRM, "Options",                 options_command      },
 {"reginfo",   'r',        0, "Registration info",       ua_print_reg_status  },
+{"answermode",0,    CMD_PRM, "Set answer mode",         cmd_set_answermode   },
 {NULL,        KEYCODE_ESC,0, "Hangup call",             cmd_hangup           },
 {"uanext",    'T',        0, "Toggle UAs",              cmd_ua_next          },
 {"uanew",     0,    CMD_PRM, "Create User-Agent",       create_ua            },
+{"uadel",     0,    CMD_PRM, "Delete User-Agent",       cmd_ua_delete        },
 {"uafind",    0,    CMD_PRM, "Find User-Agent <aor>",   cmd_ua_find          },
 {"ausrc",     0,    CMD_PRM, "Switch audio source",     switch_audio_source  },
 {"auplay",    0,    CMD_PRM, "Switch audio player",     switch_audio_player  },
 {"about",     0,          0, "About box",               about_box            },
-#ifdef USE_VIDEO
 {"vidsrc",    0,    CMD_PRM, "Switch video source",     switch_video_source  },
-#endif
 
 };
 
@@ -493,16 +554,7 @@ static const struct cmd dialcmdv[] = {
 static int call_audio_debug(struct re_printf *pf, void *unused)
 {
 	(void)unused;
-	return audio_debug(pf, call_audio(ua_call(uag_cur())));
-}
-
-
-static int call_audioenc_cycle(struct re_printf *pf, void *unused)
-{
-	(void)pf;
-	(void)unused;
-	audio_encoder_cycle(call_audio(ua_call(uag_cur())));
-	return 0;
+	return audio_debug(pf, call_audio(ua_call(uag_current())));
 }
 
 
@@ -510,13 +562,13 @@ static int call_reinvite(struct re_printf *pf, void *unused)
 {
 	(void)pf;
 	(void)unused;
-	return call_modify(ua_call(uag_cur()));
+	return call_modify(ua_call(uag_current()));
 }
 
 
 static int call_mute(struct re_printf *pf, void *unused)
 {
-	struct audio *audio = call_audio(ua_call(uag_cur()));
+	struct audio *audio = call_audio(ua_call(uag_current()));
 	bool muted = !audio_ismuted(audio);
 	(void)unused;
 
@@ -532,7 +584,7 @@ static int call_xfer(struct re_printf *pf, void *arg)
 	const struct cmd_arg *carg = arg;
 	(void)pf;
 
-	return call_transfer(ua_call(uag_cur()), carg->prm);
+	return call_transfer(ua_call(uag_current()), carg->prm);
 }
 
 
@@ -541,7 +593,7 @@ static int cmd_call_hold(struct re_printf *pf, void *arg)
 	(void)pf;
 	(void)arg;
 
-	return call_hold(ua_call(uag_cur()), true);
+	return call_hold(ua_call(uag_current()), true);
 }
 
 
@@ -550,7 +602,7 @@ static int cmd_call_resume(struct re_printf *pf, void *arg)
 	(void)pf;
 	(void)arg;
 
-	return call_hold(ua_call(uag_cur()), false);
+	return call_hold(ua_call(uag_current()), false);
 }
 
 
@@ -559,7 +611,7 @@ static int hold_prev_call(struct re_printf *pf, void *arg)
 	const struct cmd_arg *carg = arg;
 	(void)pf;
 
-	return call_hold(ua_prev_call(uag_cur()), 'H' == carg->key);
+	return call_hold(ua_prev_call(uag_current()), 'H' == carg->key);
 }
 
 
@@ -619,7 +671,7 @@ static int switch_audio_player(struct re_printf *pf, void *arg)
 	str_ncpy(aucfg->alert_mod, driver, sizeof(aucfg->alert_mod));
 	str_ncpy(aucfg->alert_dev, device, sizeof(aucfg->alert_dev));
 
-	for (le = list_tail(ua_calls(uag_cur())); le; le = le->prev) {
+	for (le = list_tail(ua_calls(uag_current())); le; le = le->prev) {
 
 		struct call *call = le->data;
 
@@ -690,7 +742,7 @@ static int switch_audio_source(struct re_printf *pf, void *arg)
 	str_ncpy(aucfg->src_mod, driver, sizeof(aucfg->src_mod));
 	str_ncpy(aucfg->src_dev, device, sizeof(aucfg->src_dev));
 
-	for (le = list_tail(ua_calls(uag_cur())); le; le = le->prev) {
+	for (le = list_tail(ua_calls(uag_current())); le; le = le->prev) {
 
 		struct call *call = le->data;
 
@@ -707,7 +759,7 @@ static int switch_audio_source(struct re_printf *pf, void *arg)
 	return 0;
 }
 
-#ifdef USE_VIDEO
+
 static int switch_video_source(struct re_printf *pf, void *arg)
 {
 	const struct cmd_arg *carg = arg;
@@ -761,7 +813,7 @@ static int switch_video_source(struct re_printf *pf, void *arg)
 	str_ncpy(vidcfg->src_mod, driver, sizeof(vidcfg->src_mod));
 	str_ncpy(vidcfg->src_dev, device, sizeof(vidcfg->src_dev));
 
-	for (le = list_tail(ua_calls(uag_cur())); le; le = le->prev) {
+	for (le = list_tail(ua_calls(uag_current())); le; le = le->prev) {
 
 		struct call *call = le->data;
 
@@ -777,25 +829,41 @@ static int switch_video_source(struct re_printf *pf, void *arg)
 
 	return 0;
 }
-#endif
-
-
-#ifdef USE_VIDEO
-static int call_videoenc_cycle(struct re_printf *pf, void *unused)
-{
-	(void)pf;
-	(void)unused;
-	video_encoder_cycle(call_video(ua_call(uag_cur())));
-	return 0;
-}
 
 
 static int call_video_debug(struct re_printf *pf, void *unused)
 {
 	(void)unused;
-	return video_debug(pf, call_video(ua_call(uag_cur())));
+	return video_debug(pf, call_video(ua_call(uag_current())));
+}
+
+
+static int set_video_dir(struct re_printf *pf, void *arg)
+{
+	const struct cmd_arg *carg = arg;
+	int err = 0;
+
+	if (0 == str_cmp(carg->prm, sdp_dir_name(SDP_INACTIVE))) {
+		err = call_set_video_dir(ua_call(uag_current()), SDP_INACTIVE);
+	}
+	else if (0 == str_cmp(carg->prm, sdp_dir_name(SDP_SENDONLY))) {
+		err = call_set_video_dir(ua_call(uag_current()), SDP_SENDONLY);
+	}
+	else if (0 == str_cmp(carg->prm, sdp_dir_name(SDP_RECVONLY))) {
+		err = call_set_video_dir(ua_call(uag_current()), SDP_RECVONLY);
+	}
+	else if (0 == str_cmp(carg->prm, sdp_dir_name(SDP_SENDRECV))) {
+		err = call_set_video_dir(ua_call(uag_current()), SDP_SENDRECV);
+	}
+	else {
+		(void)re_hprintf(pf, "Invalid video direction %s"
+			" (inactive, sendonly, recvonly, sendrecv)\n",
+			carg->prm);
+		return EINVAL;
+	}
+
+	return err;
 }
-#endif
 
 
 static int digit_handler(struct re_printf *pf, void *arg)
@@ -806,7 +874,7 @@ static int digit_handler(struct re_printf *pf, void *arg)
 
 	(void)pf;
 
-	call = ua_call(uag_cur());
+	call = ua_call(uag_current());
 	if (call)
 		err = call_send_digit(call, carg->key);
 
@@ -822,11 +890,14 @@ static int send_code(struct re_printf *pf, void *arg)
 	int err = 0;
 	(void)pf;
 
-	call = ua_call(uag_cur());
+	call = ua_call(uag_current());
 	if (call) {
 		for (i = 0; i < str_len(carg->prm) && !err; i++) {
 			err = call_send_digit(call, carg->prm[i]);
 		}
+		if (!err) {
+			err = call_send_digit(call, KEYCODE_REL);
+		}
 	}
 
 	return err;
@@ -854,11 +925,11 @@ static int set_current_call(struct re_printf *pf, void *arg)
 	uint32_t linenum = atoi(carg->prm);
 	int err;
 
-	call = call_find_linenum(ua_calls(uag_cur()), linenum);
+	call = call_find_linenum(ua_calls(uag_current()), linenum);
 	if (call) {
 		err = re_hprintf(pf, "setting current call: line %u\n",
 				 linenum);
-		call_set_current(ua_calls(uag_cur()), call);
+		call_set_current(ua_calls(uag_current()), call);
 	}
 	else {
 		err = re_hprintf(pf, "call not found\n");
@@ -872,10 +943,10 @@ static int set_audio_bitrate(struct re_printf *pf, void *arg)
 {
 	struct cmd_arg *carg = arg;
 	struct call *call;
-	uint32_t bitrate = atoi(carg->prm);
+	uint32_t bitrate = str_isset(carg->prm) ? atoi(carg->prm) : 0;
 	int err;
 
-	call = ua_call(uag_cur());
+	call = ua_call(uag_current());
 	if (call) {
 		err = re_hprintf(pf, "setting audio bitrate: %u bps\n",
 				 bitrate);
@@ -889,23 +960,44 @@ static int set_audio_bitrate(struct re_printf *pf, void *arg)
 }
 
 
-static const struct cmd callcmdv[] = {
-{"reinvite",  'I',        0, "Send re-INVITE",      call_reinvite         },
-{"resume",    'X',        0, "Call resume",         cmd_call_resume       },
-{"audio_debug",'A',       0, "Audio stream",        call_audio_debug      },
-{"audio_cycle",'e',       0, "Cycle audio encoder", call_audioenc_cycle   },
-{"mute",      'm',        0, "Call mute/un-mute",   call_mute             },
-{"transfer",  't',  CMD_PRM, "Transfer call",       call_xfer             },
-{"hold",      'x',        0, "Call hold",           cmd_call_hold         },
-{"",          'H',        0, "Hold previous call",  hold_prev_call        },
-{"",          'L',        0, "Resume previous call",hold_prev_call        },
-{"aubitrate",   0,  CMD_PRM, "Set audio bitrate",   set_audio_bitrate     },
-{"sndcode",   0,    CMD_PRM, "Send Code",           send_code             },
+static int cmd_find_call(struct re_printf *pf, void *arg)
+{
+	struct cmd_arg *carg = arg;
+	const char *id = carg->prm;
+	struct list *calls = ua_calls(uag_current());
+	struct call *call;
+	int err;
 
-#ifdef USE_VIDEO
-{"video_cycle", 'E',      0, "Cycle video encoder", call_videoenc_cycle   },
-{"video_debug", 'V',      0, "Video stream",        call_video_debug      },
-#endif
+	call = call_find_id(calls, id);
+	if (call) {
+		err = re_hprintf(pf, "setting current call: %s\n", id);
+		call_set_current(calls, call);
+	}
+	else {
+		err = re_hprintf(pf, "call not found (id=%s)\n", id);
+	}
+
+	return err;
+}
+
+
+static const struct cmd callcmdv[] = {
+
+{"aubitrate",    0,  CMD_PRM, "Set audio bitrate",    set_audio_bitrate    },
+{"audio_debug", 'A',       0, "Audio stream",         call_audio_debug     },
+{"callfind",     0,  CMD_PRM, "Find call <callid>",   cmd_find_call        },
+{"hold",        'x',       0, "Call hold",            cmd_call_hold        },
+{"line",        '@', CMD_PRM, "Set current call <line>", set_current_call  },
+{"mute",        'm',       0, "Call mute/un-mute",    call_mute            },
+{"prevhold",    'H',       0, "Hold previous call",   hold_prev_call       },
+{"prevresume",  'L',       0, "Resume previous call", hold_prev_call       },
+{"reinvite",    'I',       0, "Send re-INVITE",       call_reinvite        },
+{"resume",      'X',       0, "Call resume",          cmd_call_resume      },
+{"sndcode",      0,  CMD_PRM, "Send Code",            send_code            },
+{"statmode",    'S',       0, "Statusmode toggle",    toggle_statmode      },
+{"transfer",    't', CMD_PRM, "Transfer call",        call_xfer            },
+{"video_debug", 'V',       0, "Video stream",         call_video_debug     },
+{"video_dir",     0, CMD_PRM, "Set video direction",  set_video_dir        },
 
 /* Numeric keypad for DTMF events: */
 {NULL, '#',         0, NULL,                  digit_handler         },
@@ -922,8 +1014,6 @@ static const struct cmd callcmdv[] = {
 {NULL, '9',         0, NULL,                  digit_handler         },
 {NULL, KEYCODE_REL, 0, NULL,                  digit_handler         },
 
-{NULL,  'S',        0, "Statusmode toggle",       toggle_statmode   },
-{"line",'@',  CMD_PRM, "Set current call <line>", set_current_call  },
 };
 
 
@@ -963,7 +1053,7 @@ static void tmrstat_handler(void *arg)
 	(void)arg;
 
 	/* the UI will only show the current active call */
-	call = ua_call(uag_cur());
+	call = ua_call(uag_current());
 	if (!call)
 		return;
 
@@ -1039,13 +1129,13 @@ static void redial_handler(void *arg)
 	if (err)
 		return;
 
-	err = ua_connect(uag_cur(), NULL, NULL, uri, VIDMODE_ON);
+	err = ua_connect(uag_find_aor(menu.redial_aor), NULL, NULL,
+			 uri, VIDMODE_ON);
 	if (err) {
 		warning("menu: redial: ua_connect failed (%m)\n", err);
 	}
 
 	mem_deref(uri);
-
 }
 
 
@@ -1054,9 +1144,8 @@ static void ua_event_handler(struct ua *ua, enum ua_event ev,
 {
 	struct player *player = baresip_player();
 	struct call *call2 = NULL;
+	struct config *cfg;
 	int err;
-
-	(void)call;
 	(void)prm;
 	(void)arg;
 
@@ -1065,6 +1154,8 @@ static void ua_event_handler(struct ua *ua, enum ua_event ev,
 	      ua_aor(ua), call_id(call), uag_event_str(ev), prm);
 #endif
 
+	cfg = conf_config();
+
 	switch (ev) {
 
 	case UA_EVENT_CALL_INCOMING:
@@ -1086,12 +1177,16 @@ static void ua_event_handler(struct ua *ua, enum ua_event ev,
 
 			if (list_count(ua_calls(ua)) > 1) {
 				(void)play_file(&menu.play, player,
-						"callwaiting.wav", 3);
+						"callwaiting.wav", 3,
+						cfg->audio.play_mod,
+						cfg->audio.play_dev);
 			}
 			else {
 				/* Alert user */
 				(void)play_file(&menu.play, player,
-						"ring.wav", -1);
+						"ring.wav", -1,
+						cfg->audio.alert_mod,
+						cfg->audio.alert_dev);
 			}
 
 			if (menu.bell)
@@ -1108,7 +1203,9 @@ static void ua_event_handler(struct ua *ua, enum ua_event ev,
 		}
 		else {
 			(void)play_file(&menu.play, player,
-					"ringback.wav",-1);
+					"ringback.wav", -1,
+					cfg->audio.play_mod,
+					cfg->audio.play_dev);
 		}
 		break;
 
@@ -1132,7 +1229,9 @@ static void ua_event_handler(struct ua *ua, enum ua_event ev,
 			tone = translate_errorcode(call_scode(call));
 			if (tone) {
 				(void)play_file(&menu.play, player,
-						tone, 1);
+						tone, 1,
+						cfg->audio.play_mod,
+						cfg->audio.play_dev);
 			}
 		}
 
@@ -1157,6 +1256,9 @@ static void ua_event_handler(struct ua *ua, enum ua_event ev,
 
 				++menu.current_attempts;
 
+				str_ncpy(menu.redial_aor, ua_aor(ua),
+					 sizeof(menu.redial_aor));
+
 				tmr_start(&menu.tmr_redial,
 					  menu.redial_delay*1000,
 					  redial_handler, NULL);
@@ -1165,7 +1267,6 @@ static void ua_event_handler(struct ua *ua, enum ua_event ev,
 				info("menu: call closed -- not redialing\n");
 			}
 		}
-
 		break;
 
 	case UA_EVENT_CALL_TRANSFER:
@@ -1201,7 +1302,6 @@ static void ua_event_handler(struct ua *ua, enum ua_event ev,
 
 	case UA_EVENT_CALL_TRANSFER_FAILED:
 		info("menu: transfer failure: %s\n", prm);
-		mem_deref(call);
 		break;
 
 	case UA_EVENT_REGISTER_OK:
@@ -1218,7 +1318,6 @@ static void ua_event_handler(struct ua *ua, enum ua_event ev,
 
 	case UA_EVENT_AUDIO_ERROR:
 		info("menu: audio error (%s)\n", prm);
-		mem_deref(call);
 		break;
 
 	default:
@@ -1234,14 +1333,18 @@ static void message_handler(struct ua *ua, const struct pl *peer,
 			    const struct pl *ctype,
 			    struct mbuf *body, void *arg)
 {
+	struct config *cfg;
 	(void)ua;
 	(void)ctype;
 	(void)arg;
 
+	cfg = conf_config();
+
 	ui_output(baresip_uis(), "\r%r: \"%b\"\n",
 		  peer, mbuf_buf(body), mbuf_get_left(body));
 
-	(void)play_file(NULL, baresip_player(), "message.wav", 0);
+	(void)play_file(NULL, baresip_player(), "message.wav", 0,
+	                cfg->audio.alert_mod, cfg->audio.alert_dev);
 }
 
 
@@ -1250,12 +1353,20 @@ static int module_init(void)
 	struct pl val;
 	int err;
 
+	menu.bell = true;
+	menu.redial_attempts = 0;
+	menu.redial_delay = 5;
+	menu.ringback_disabled = false;
+	menu.statmode = STATMODE_CALL;
+	menu.clean_number = false;
+
 	/*
 	 * Read the config values
 	 */
 	conf_get_bool(conf_cur(), "menu_bell", &menu.bell);
 	conf_get_bool(conf_cur(), "ringback_disabled",
 		      &menu.ringback_disabled);
+	conf_get_bool(conf_cur(), "menu_clean_number", &menu.clean_number);
 
 	if (0 == conf_get(conf_cur(), "redial_attempts", &val) &&
 	    0 == pl_strcasecmp(&val, "inf")) {
@@ -1299,7 +1410,7 @@ static int module_init(void)
 	if (err)
 		return err;
 
-	err = message_listen(&menu.message, baresip_message(),
+	err = message_listen(baresip_message(),
 			     message_handler, NULL);
 	if (err)
 		return err;
@@ -1313,7 +1424,8 @@ static int module_close(void)
 	debug("menu: close (redial current_attempts=%d)\n",
 	      menu.current_attempts);
 
-	menu.message = mem_deref(menu.message);
+	message_unlisten(baresip_message(), message_handler);
+
 	uag_event_unregister(ua_event_handler);
 	cmd_unregister(baresip_commands(), cmdv);
 	cmd_unregister(baresip_commands(), dialcmdv);
