@@ -15,15 +15,15 @@ struct mpa_param {
 	unsigned samplerate;
 	unsigned bitrate;
 	unsigned layer;
-	enum { AUTO=0, STEREO, JOINT_STEREO, SINGLE_CHANNEL, DUAL_CHANNEL }
-		mode;
+	int mode;  /* MPEG_mode */
 };
 
 
 /* Encode */
 int mpa_encode_update(struct auenc_state **aesp, const struct aucodec *ac,
 		       struct auenc_param *prm, const char *fmtp);
-int mpa_encode_frm(struct auenc_state *aes, uint8_t *buf, size_t *len,
+int mpa_encode_frm(struct auenc_state *aes,
+		   bool *marker, uint8_t *buf, size_t *len,
 		   int fmt, const void *sampv, size_t sampc);
 
 
@@ -32,7 +32,10 @@ int mpa_decode_update(struct audec_state **adsp, const struct aucodec *ac,
 		       const char *fmtp);
 int mpa_decode_frm(struct audec_state *ads,
 		   int fmt, void *sampv, size_t *sampc,
-		   const uint8_t *buf, size_t len);
+		   bool marker, const uint8_t *buf, size_t len);
 
 /* SDP */
 void mpa_decode_fmtp(struct mpa_param *prm, const char *fmtp);
+
+
+void mpa_mirror_params(const char *fmtp);
