@@ -30,6 +30,7 @@ static struct mqtt s_mqtt;
 static void fd_handler(int flags, void *arg)
 {
 	struct mqtt *mqtt = arg;
+	(void)flags;
 
 	mosquitto_loop_read(mqtt->mosq, 1);
 
@@ -59,7 +60,7 @@ static void connect_callback(struct mosquitto *mosq, void *obj, int result)
 {
 	struct mqtt *mqtt = obj;
 	int err;
-	(void)mqtt;
+	(void)mosq;
 
 	if (result != MOSQ_ERR_SUCCESS) {
 		warning("mqtt: could not connect to broker (%s) \n",
@@ -102,12 +103,14 @@ static int module_init(void)
 	info("mqtt: connecting to broker at %s:%d as %s topic %s\n",
 		broker_host, broker_port, mqttclientid, mqttbasetopic);
 
-	snprintf(mqttsubscribetopic, sizeof(mqttsubscribetopic),
-		"/%s/command/+", mqttbasetopic);
-	snprintf(mqttpublishtopic, sizeof(mqttpublishtopic), "/%s/event",
-		mqttbasetopic);
+	re_snprintf(mqttsubscribetopic, sizeof(mqttsubscribetopic),
+		    "/%s/command/+", mqttbasetopic);
+	re_snprintf(mqttpublishtopic, sizeof(mqttpublishtopic), "/%s/event",
+		    mqttbasetopic);
+
 	info("mqtt: Publishing on %s, subscribing to %s\n",
 		mqttpublishtopic, mqttsubscribetopic);
+
 	s_mqtt.basetopic = mqttbasetopic;
 	s_mqtt.subtopic = mqttsubscribetopic;
 	s_mqtt.pubtopic = mqttpublishtopic;
@@ -129,6 +132,8 @@ static int module_init(void)
 	if (*mqttusername != '\0') {
 		ret = mosquitto_username_pw_set(s_mqtt.mosq, mqttusername,
 			mqttpassword);
+		if (ret != MOSQ_ERR_SUCCESS)
+			return ret == MOSQ_ERR_ERRNO ? errno : EIO;
 	}
 
 	ret = mosquitto_connect(s_mqtt.mosq, broker_host, broker_port,
