@@ -9,6 +9,7 @@
 #include <baresip.h>
 #include "mqtt.h"
 
+
 static int print_handler(const char *p, size_t size, void *arg)
 {
 	struct mbuf *mb = arg;
@@ -21,13 +22,12 @@ static void handle_command(struct mqtt *mqtt, const struct pl *msg)
 {
 	struct mbuf *resp = mbuf_alloc(1024);
 	struct re_printf pf = {print_handler, resp};
-	struct odict *od = NULL;
+	struct odict *od = NULL, *od_resp = NULL;
 	const struct odict_entry *oe_cmd, *oe_prm, *oe_tok;
 	char buf[256], resp_topic[256];
+	const char *aor, *callid;
 	int err;
 
-	/* XXX: add transaction ID ? */
-
 	err = json_decode_odict(&od, 32, msg->p, msg->l, 16);
 	if (err) {
 		warning("mqtt: failed to decode JSON with %zu bytes (%m)\n",
@@ -43,8 +43,33 @@ static void handle_command(struct mqtt *mqtt, const struct pl *msg)
 		goto out;
 	}
 
+	aor    = odict_string(od, "accountaor");
+	callid = odict_string(od, "callid");
+
+	if (aor) {
+		struct ua *ua = uag_find_aor(aor);
+		if (!ua) {
+			warning("mqtt: ua not found (%s)\n", aor);
+			goto out;
+		}
+
+		uag_current_set(ua);
+
+		if (callid) {
+			struct call *call;
+
+			call = call_find_id(ua_calls(ua), callid);
+			if (!call) {
+				warning("mqtt: call not found (%s)\n", callid);
+				goto out;
+			}
+
+			call_set_current(ua_calls(ua), call);
+		}
+	}
+
 	debug("mqtt: handle_command:  cmd='%s', token='%s'\n",
-	      oe_cmd ? oe_cmd->u.str : "",
+	      oe_cmd->u.str,
 	      oe_tok ? oe_tok->u.str : "");
 
 	re_snprintf(buf, sizeof(buf), "%s%s%s",
@@ -61,6 +86,10 @@ static void handle_command(struct mqtt *mqtt, const struct pl *msg)
 		warning("mqtt: error processing command (%m)\n", err);
 	}
 
+	err = mbuf_write_u8(resp, '\0');
+	if (err)
+		goto out;
+
 	/* NOTE: the command will now write the response
 	   to the resp mbuf, send it back to broker */
 
@@ -68,9 +97,23 @@ static void handle_command(struct mqtt *mqtt, const struct pl *msg)
 		    "/%s/command_resp/%s", mqtt->basetopic,
 		    oe_tok ? oe_tok->u.str : "nil");
 
+	err = odict_alloc(&od_resp, 8);
+	if (err)
+		goto out;
+
+	err  = odict_entry_add(od_resp, "response", ODICT_BOOL, true);
+	err |= odict_entry_add(od_resp, "ok", ODICT_BOOL, (bool)err==0);
+	err |= odict_entry_add(od_resp, "data", ODICT_STRING, resp->buf);
+	if (oe_tok) {
+		err |= odict_entry_add(od_resp, "token",
+				       ODICT_STRING, oe_tok->u.str);
+	}
+	if (err)
+		goto out;
+
 	err = mqtt_publish_message(mqtt, resp_topic,
-				   "%b",
-				   resp->buf, resp->end);
+				   "%H",
+				   json_encode_odict, od_resp);
 	if (err) {
 		warning("mqtt: failed to publish message (%m)\n", err);
 		goto out;
@@ -78,6 +121,7 @@ static void handle_command(struct mqtt *mqtt, const struct pl *msg)
 
  out:
 	mem_deref(resp);
+	mem_deref(od_resp);
 	mem_deref(od);
 }
 
@@ -91,6 +135,7 @@ static void message_callback(struct mosquitto *mosq, void *obj,
 	struct mqtt *mqtt = obj;
 	struct pl msg;
 	bool match = false;
+	(void)mosq;
 
 	info("mqtt: got message '%b' for topic '%s'\n",
 	     (char*) message->payload, (size_t)message->payloadlen,
