@@ -55,10 +55,15 @@ static void ausrc_destructor(void *arg)
 static void bqRecorderCallback(SLAndroidSimpleBufferQueueItf bq, void *context)
 {
 	struct ausrc_st *st = context;
-
+	struct auframe af = {
+		.fmt   = AUFMT_S16LE,
+		.sampv = st->sampv[st->bufferId],
+		.sampc = st->sampc,
+		.timestamp = tmr_jiffies_usec()
+	};
 	(void)bq;
 
-	st->rh(st->sampv[st->bufferId], st->sampc, st->arg);
+	st->rh(&af, st->arg);
 
 	st->bufferId = ( st->bufferId + 1 ) % N_REC_QUEUE_BUFFERS;
 
