@@ -146,6 +146,10 @@ int opus_encode_update(struct auenc_state **aesp, const struct aucodec *ac,
 	(void)opus_encoder_ctl(aes->enc, OPUS_SET_INBAND_FEC(prm.inband_fec));
 	(void)opus_encoder_ctl(aes->enc, OPUS_SET_DTX(prm.dtx));
 
+	if (opus_packet_loss) {
+		opus_encoder_ctl(aes->enc,
+				 OPUS_SET_PACKET_LOSS_PERC(opus_packet_loss));
+	}
 
 #if 0
 	{
@@ -170,10 +174,12 @@ int opus_encode_update(struct auenc_state **aesp, const struct aucodec *ac,
 }
 
 
-int opus_encode_frm(struct auenc_state *aes, uint8_t *buf, size_t *len,
+int opus_encode_frm(struct auenc_state *aes,
+		    bool *marker, uint8_t *buf, size_t *len,
 		    int fmt, const void *sampv, size_t sampc)
 {
 	opus_int32 n;
+	(void)marker;
 
 	if (!aes || !buf || !len || !sampv)
 		return EINVAL;
