@@ -28,6 +28,7 @@
   opus_dtx        {yes,no}   # Enable Discontinuous Transmission (DTX)
   opus_complexity {0-10}     # Encoder's computational complexity (10 max)
   opus_application {audio, voip} # Encoder's intended application
+  opus_packet_loss {0-100}   # Expected packet loss for FEC
  \endverbatim
  *
  * References:
@@ -45,6 +46,8 @@ static char fmtp_mirror[256];
 
 uint32_t opus_complexity = 10;
 opus_int32 opus_application = OPUS_APPLICATION_AUDIO;
+opus_int32 opus_packet_loss = 0;
+
 
 static int opus_fmtp_enc(struct mbuf *mb, const struct sdp_format *fmt,
 			 bool offer, void *arg)
@@ -102,6 +105,9 @@ static int module_init(void)
 	conf_get_bool(conf, "opus_stereo", &stereo);
 	conf_get_bool(conf, "opus_sprop_stereo", &sprop_stereo);
 
+	if (!stereo || !sprop_stereo)
+		opus.ch = 1;
+
 	/* always set stereo parameter first */
 	n = re_snprintf(p, sizeof(fmtp) - str_len(p),
 			"stereo=%d;sprop-stereo=%d", stereo, sprop_stereo);
@@ -119,6 +125,15 @@ static int module_init(void)
 
 		p += n;
 	}
+	if (0 == conf_get_u32(conf, "opus_samplerate", &value)) {
+
+		if ((value != 8000) && (value != 12000) && (value != 16000) &&
+		    (value != 24000) && (value != 48000)) {
+			warning("opus: invalid samplerate: %d\n", value);
+			return EINVAL;
+		}
+		opus.srate = value;
+	}
 
 	if (0 == conf_get_bool(conf, "opus_cbr", &b)) {
 
@@ -174,6 +189,14 @@ static int module_init(void)
 		}
 	}
 
+	if (0 == conf_get_u32(conf, "opus_packet_loss", &value)) {
+
+		if (value > 100)
+			opus_packet_loss = 100;
+		else
+			opus_packet_loss = value;
+	}
+
 	debug("opus: fmtp=\"%s\"\n", fmtp);
 
 	aucodec_register(baresip_aucodecl(), &opus);
