@@ -51,7 +51,6 @@ static int update(struct aufilt_dec_st **stp, void **ctx,
 	if (*stp)
 		return 0;
 
-	/* XXX: add support for stereo PLC */
 	if (prm->ch != 1) {
 		warning("plc: only mono supported (ch=%u)\n", prm->ch);
 		return ENOSYS;
@@ -72,8 +71,6 @@ static int update(struct aufilt_dec_st **stp, void **ctx,
 		goto out;
 	}
 
-	st->sampc = prm->srate * prm->ch * prm->ptime / 1000;
-
  out:
 	if (err)
 		mem_deref(st);
@@ -89,27 +86,35 @@ static int update(struct aufilt_dec_st **stp, void **ctx,
  *
  * NOTE: sampc == 0 , means Packet loss
  */
-static int decode(struct aufilt_dec_st *st, void *sampv, size_t *sampc)
+static int decode(struct aufilt_dec_st *st, struct auframe *af)
 {
 	struct plc_st *plc = (struct plc_st *)st;
 
-	if (*sampc)
-		plc_rx(&plc->plc, sampv, (int)*sampc);
-	else
-		*sampc = plc_fillin(&plc->plc, sampv, (int)plc->sampc);
+	if (!st || !af)
+		return EINVAL;
+
+	if (af->sampc) {
+		plc_rx(&plc->plc, af->sampv, (int)af->sampc);
+		plc->sampc = af->sampc;
+	}
+	else if (plc->sampc)
+		af->sampc = plc_fillin(&plc->plc, af->sampv, (int)plc->sampc);
 
 	return 0;
 }
 
 
 static struct aufilt plc = {
-	LE_INIT, "plc", NULL, NULL, update, decode
+	.name    = "plc",
+	.decupdh = update,
+	.dech    = decode
 };
 
 
 static int module_init(void)
 {
 	aufilt_register(baresip_aufiltl(), &plc);
+
 	return 0;
 }
 
@@ -117,6 +122,7 @@ static int module_init(void)
 static int module_close(void)
 {
 	aufilt_unregister(&plc);
+
 	return 0;
 }
 
