@@ -47,8 +47,8 @@ static int cmd_offline(struct re_printf *pf, void *arg)
 
 
 static const struct cmd cmdv[] = {
-	{'[', 0, "Set presence online",   cmd_online  },
-	{']', 0, "Set presence offline",  cmd_offline },
+	{"presence_online",  '[', 0, "Set presence online",   cmd_online  },
+	{"presence_offline", ']', 0, "Set presence offline",  cmd_offline },
 };
 
 
@@ -87,7 +87,7 @@ static int module_init(void)
 	if (err)
 		return err;
 
-	err = cmd_register(cmdv, ARRAY_SIZE(cmdv));
+	err = cmd_register(baresip_commands(), cmdv, ARRAY_SIZE(cmdv));
 	if (err)
 		return err;
 
@@ -103,7 +103,7 @@ static int module_close(void)
 {
 	uag_event_unregister(event_handler);
 
-	cmd_unregister(cmdv);
+	cmd_unregister(baresip_commands(), cmdv);
 
 	publisher_close();
 
