@@ -102,10 +102,13 @@ static int selfview_alloc(struct selfview **selfviewp, void **ctx)
 
 
 static int encode_update(struct vidfilt_enc_st **stp, void **ctx,
-			 const struct vidfilt *vf)
+			 const struct vidfilt *vf, struct vidfilt_prm *prm,
+			 const struct video *vid)
 {
 	struct selfview_enc *st;
 	int err;
+	(void)prm;
+	(void)vid;
 
 	if (!stp || !ctx || !vf)
 		return EINVAL;
@@ -129,10 +132,13 @@ static int encode_update(struct vidfilt_enc_st **stp, void **ctx,
 
 
 static int decode_update(struct vidfilt_dec_st **stp, void **ctx,
-			 const struct vidfilt *vf)
+			 const struct vidfilt *vf, struct vidfilt_prm *prm,
+			 const struct video *vid)
 {
 	struct selfview_dec *st;
 	int err;
+	(void)prm;
+	(void)vid;
 
 	if (!stp || !ctx || !vf)
 		return EINVAL;
@@ -244,11 +250,16 @@ static int decode_pip(struct vidfilt_dec_st *st, struct vidframe *frame,
 
 
 static struct vidfilt selfview_win = {
-	LE_INIT, "selfview_window", encode_update, encode_win, NULL, NULL
+	.name    = "selfview_window",
+	.encupdh = encode_update,
+	.ench    = encode_win,
 };
 static struct vidfilt selfview_pip = {
-	LE_INIT, "selfview_pip",
-	encode_update, encode_pip, decode_update, decode_pip
+	.name    = "selfview_pip",
+	.encupdh = encode_update,
+	.ench    = encode_pip,
+	.decupdh = decode_update,
+	.dech    = decode_pip
 };
 
 
