@@ -7,7 +7,6 @@
 #define _DEFAULT_SOURCE 1
 #define _BSD_SOURCE 1
 #include <string.h>
-#include <time.h>
 #include <png.h>
 #include <re.h>
 #include <rem.h>
@@ -15,8 +14,6 @@
 #include "png_vf.h"
 
 
-static char *png_filename(const struct tm *tmx, const char *name,
-			  char *buf, unsigned int length);
 static void png_save_free(png_structp png_ptr, png_byte **png_row_pointers,
 			  int png_height);
 
@@ -34,15 +31,9 @@ int png_save_vidframe(const struct vidframe *vf, const char *path)
 	unsigned int width = vf->size.w & ~1;
 	unsigned int height = vf->size.h & ~1;
 	unsigned int bytes_per_pixel = 3; /* RGB format */
-	time_t tnow;
-	struct tm *tmx;
-	char filename_buf[64];
 	struct vidframe *f2 = NULL;
 	int err = 0;
 
-	tnow = time(NULL);
-	tmx = localtime(&tnow);
-
 	if (vf->fmt != VID_FMT_RGB32) {
 
 		err = vidframe_alloc(&f2, VID_FMT_RGB32, &vf->size);
@@ -118,8 +109,7 @@ int png_save_vidframe(const struct vidframe *vf, const char *path)
 	}
 
 	/* Write the image data. */
-	fp = fopen(png_filename(tmx, path,
-				filename_buf, sizeof(filename_buf)), "wb");
+	fp = fopen(path, "wb");
 	if (fp == NULL) {
 		err = errno;
 		goto out;
@@ -129,7 +119,7 @@ int png_save_vidframe(const struct vidframe *vf, const char *path)
 	png_set_rows(png_ptr, info_ptr, png_row_pointers);
 	png_write_png(png_ptr, info_ptr, PNG_TRANSFORM_IDENTITY, NULL);
 
-	info("png: wrote %s\n", filename_buf);
+	info("png: wrote %s\n", path);
 
  out:
 	/* Finish writing. */
@@ -159,31 +149,3 @@ static void png_save_free(png_structp png_ptr, png_byte **png_row_pointers,
 }
 
 
-static char *png_filename(const struct tm *tmx, const char *name,
-			  char *buf, unsigned int length)
-{
-	/*
-	 * -2013-03-03-15-22-56.png - 24 chars
-	 */
-	if (strlen(name) + 24 >= length) {
-		buf[0] = '\0';
-		return buf;
-	}
-
-	sprintf(buf, (tmx->tm_mon < 9 ? "%s-%d-0%d" : "%s-%d-%d"), name,
-		1900 + tmx->tm_year, tmx->tm_mon + 1);
-
-	sprintf(buf + strlen(buf), (tmx->tm_mday < 10 ? "-0%d" : "-%d"),
-		tmx->tm_mday);
-
-	sprintf(buf + strlen(buf), (tmx->tm_hour < 10 ? "-0%d" : "-%d"),
-		tmx->tm_hour);
-
-	sprintf(buf + strlen(buf), (tmx->tm_min < 10 ? "-0%d" : "-%d"),
-		tmx->tm_min);
-
-	sprintf(buf + strlen(buf), (tmx->tm_sec < 10 ? "-0%d.png" : "-%d.png"),
-		tmx->tm_sec);
-
-	return buf;
-}
