@@ -25,10 +25,18 @@
  */
 
 
+struct menc_sess {
+	menc_event_h *eventh;
+	void *arg;
+};
+
+
 struct menc_st {
 	/* one SRTP session per media line */
-	uint8_t key_tx[32];
-	uint8_t key_rx[32];
+	const struct menc_sess *sess;
+	uint8_t key_tx[32+12];
+	/* base64_decoding worst case encoded 32+12 key */
+	uint8_t key_rx[46];
 	struct srtp *srtp_tx, *srtp_rx;
 	bool use_srtp;
 	bool got_sdp;
@@ -39,12 +47,14 @@ struct menc_st {
 	struct udp_helper *uh_rtp;   /**< UDP helper for RTP encryption    */
 	struct udp_helper *uh_rtcp;  /**< UDP helper for RTCP encryption   */
 	struct sdp_media *sdpm;
+	const struct stream *strm;   /**< pointer to parent */
 };
 
 
 static const char aes_cm_128_hmac_sha1_32[] = "AES_CM_128_HMAC_SHA1_32";
 static const char aes_cm_128_hmac_sha1_80[] = "AES_CM_128_HMAC_SHA1_80";
 static const char aes_128_gcm[]             = "AEAD_AES_128_GCM";
+static const char aes_256_gcm[]             = "AEAD_AES_256_GCM";
 
 static const char *preferred_suite = aes_cm_128_hmac_sha1_80;
 
@@ -72,6 +82,7 @@ static bool cryptosuite_issupported(const struct pl *suite)
 	if (0 == pl_strcasecmp(suite, aes_cm_128_hmac_sha1_32)) return true;
 	if (0 == pl_strcasecmp(suite, aes_cm_128_hmac_sha1_80)) return true;
 	if (0 == pl_strcasecmp(suite, aes_128_gcm))             return true;
+	if (0 == pl_strcasecmp(suite, aes_256_gcm))             return true;
 
 	return false;
 }
@@ -123,6 +134,8 @@ static enum srtp_suite resolve_suite(const char *suite)
 		return SRTP_AES_CM_128_HMAC_SHA1_80;
 	if (0 == str_casecmp(suite, aes_128_gcm))
 		return SRTP_AES_128_GCM;
+	if (0 == str_casecmp(suite, aes_256_gcm))
+		return SRTP_AES_256_GCM;
 
 	return -1;
 }
@@ -135,6 +148,7 @@ static size_t get_master_keylen(enum srtp_suite suite)
 	case SRTP_AES_CM_128_HMAC_SHA1_32: return 16+14;
 	case SRTP_AES_CM_128_HMAC_SHA1_80: return 16+14;
 	case SRTP_AES_128_GCM:             return 16+12;
+	case SRTP_AES_256_GCM:             return 32+12;
 	default: return 0;
 	}
 }
@@ -258,6 +272,7 @@ static int sdp_enc(struct menc_st *st, struct sdp_media *m,
 static int start_crypto(struct menc_st *st, const struct pl *key_info)
 {
 	size_t olen, len;
+	char buf[64] = "";
 	int err;
 
 	len = get_master_keylen(resolve_suite(st->crypto_suite));
@@ -281,6 +296,18 @@ static int start_crypto(struct menc_st *st, const struct pl *key_info)
 	info("srtp: %s: SRTP is Enabled (cryptosuite=%s)\n",
 	     sdp_media_name(st->sdpm), st->crypto_suite);
 
+	if (st->sess->eventh) {
+		if (re_snprintf(buf, sizeof(buf), "%s,%s",
+				sdp_media_name(st->sdpm),
+				st->crypto_suite))
+			st->sess->eventh(MENC_EVENT_SECURE, buf,
+					 (struct stream *)st->strm,
+					 st->sess->arg);
+		else
+			warning("srtp: failed to print secure"
+				" event arguments\n");
+	}
+
 	return 0;
 }
 
@@ -312,10 +339,38 @@ static bool sdp_attr_handler(const char *name, const char *value, void *arg)
 }
 
 
-static int alloc(struct menc_media **stp, struct menc_sess *sess,
+static int session_alloc(struct menc_sess **sessp,
+			 struct sdp_session *sdp, bool offerer,
+			 menc_event_h *eventh, menc_error_h *errorh,
+			 void *arg)
+{
+	struct menc_sess *sess;
+	(void)sdp;
+	(void)offerer;
+	(void)errorh;
+
+	if (!sessp)
+		return EINVAL;
+
+	sess = mem_zalloc(sizeof(*sess), NULL);
+	if (!sess)
+		return ENOMEM;
+
+	sess->eventh  = eventh;
+	sess->arg     = arg;
+
+	*sessp = sess;
+
+	return 0;
+}
+
+
+static int media_alloc(struct menc_media **stp, struct menc_sess *sess,
 		 struct rtp_sock *rtp,
-		 int proto, void *rtpsock, void *rtcpsock,
-		 struct sdp_media *sdpm)
+		 struct udp_sock *rtpsock, struct udp_sock *rtcpsock,
+	         const struct sa *raddr_rtp,
+	         const struct sa *raddr_rtcp,
+		 struct sdp_media *sdpm, const struct stream *strm)
 {
 	struct menc_st *st;
 	const char *rattr = NULL;
@@ -324,11 +379,11 @@ static int alloc(struct menc_media **stp, struct menc_sess *sess,
 	bool mux = (rtpsock == rtcpsock);
 	(void)sess;
 	(void)rtp;
+	(void)raddr_rtp;
+	(void)raddr_rtcp;
 
-	if (!stp || !sdpm)
+	if (!stp || !sdpm || !sess)
 		return EINVAL;
-	if (proto != IPPROTO_UDP)
-		return EPROTONOSUPPORT;
 
 	st = (struct menc_st *)*stp;
 	if (!st) {
@@ -337,7 +392,9 @@ static int alloc(struct menc_media **stp, struct menc_sess *sess,
 		if (!st)
 			return ENOMEM;
 
+		st->sess = sess;
 		st->sdpm = mem_ref(sdpm);
+		st->strm = strm;
 
 		if (0 == str_cmp(sdp_media_proto(sdpm), "RTP/AVP")) {
 			err = sdp_media_set_alt_protos(st->sdpm, 4,
@@ -401,15 +458,24 @@ static int alloc(struct menc_media **stp, struct menc_sess *sess,
 
 
 static struct menc menc_srtp_opt = {
-	LE_INIT, "srtp", "RTP/AVP", NULL, alloc
+	.id        = "srtp",
+	.sdp_proto = "RTP/AVP",
+	.sessh     = session_alloc,
+	.mediah    = media_alloc
 };
 
 static struct menc menc_srtp_mand = {
-	LE_INIT, "srtp-mand", "RTP/SAVP", NULL, alloc
+	.id        = "srtp-mand",
+	.sdp_proto = "RTP/SAVP",
+	.sessh     = session_alloc,
+	.mediah    = media_alloc
 };
 
 static struct menc menc_srtp_mandf = {
-	LE_INIT, "srtp-mandf", "RTP/SAVPF", NULL, alloc
+	.id        = "srtp-mandf",
+	.sdp_proto = "RTP/SAVPF",
+	.sessh     = session_alloc,
+	.mediah    = media_alloc
 };
 
 
