@@ -32,12 +32,6 @@
  * for devices that supports compressed formats such as H.264.
  * The module implements both the vidsrc API and the vidcodec API.
  *
- *
- * TODO:
- *
- * - timestamp syncronization
- * - how to configure the wanted bitrate and framerate
- * - how to handle Key-frame requests
  */
 
 
@@ -64,7 +58,7 @@ struct videnc_state {
 };
 
 
-/* TODO: global data, move to per vidsrc instance */
+/* NOTE: global data, move to per vidsrc instance */
 static struct {
 	/* List of encoder-states (struct videnc_state) */
 	struct list encoderl;
@@ -424,7 +418,6 @@ static int encode_update(struct videnc_state **vesp, const struct vidcodec *vc,
 			 videnc_packet_h *pkth, void *arg)
 {
 	struct videnc_state *st;
-	int err = 0;
 	(void)fmtp;
 
 	if (!vesp || !vc || !prm || !pkth)
@@ -446,12 +439,9 @@ static int encode_update(struct videnc_state **vesp, const struct vidcodec *vc,
 	info("v4l2_codec: video encoder %s: %.2f fps, %d bit/s, pktsize=%u\n",
 	      vc->name, prm->fps, prm->bitrate, prm->pktsize);
 
-	if (err)
-		mem_deref(st);
-	else
-		*vesp = st;
+	*vesp = st;
 
-	return err;
+	return 0;
 }
 
 
@@ -465,7 +455,7 @@ static int encode_packet(struct videnc_state *st, bool update,
 	(void)timestamp;
 
 	/*
-	 * XXX: add support for KEY frame requests
+	 * NOTE: add support for KEY frame requests
 	 */
 	if (update) {
 		info("v4l2_codec: peer requested a KEY frame (ignored)\n");
@@ -589,17 +579,12 @@ out:
 
 
 static struct vidcodec h264 = {
-	LE_INIT,
-	NULL,
-	"H264",
-	"packetization-mode=0",
-	NULL,
-	encode_update,
-	encode_packet,
-	NULL,
-	NULL,
-	h264_fmtp_enc,
-	h264_fmtp_cmp,
+	.name      = "H264",
+	.variant   = "packetization-mode=0",
+	.encupdh   = encode_update,
+	.ench      = encode_packet,
+	.fmtp_ench = h264_fmtp_enc,
+	.fmtp_cmph = h264_fmtp_cmp,
 };
 
 
