@@ -10,7 +10,6 @@ struct vidsrc_st {
 
 	struct le le;
 	struct vidisp_st *vidisp;
-	uint64_t timestamp;
 	double fps;
 	char *device;
 	vidsrc_frame_h *frameh;
@@ -35,7 +34,7 @@ int vidbridge_disp_alloc(struct vidisp_st **stp, const struct vidisp *vd,
 			 struct vidisp_prm *prm, const char *dev,
 			 vidisp_resize_h *resizeh, void *arg);
 int vidbridge_disp_display(struct vidisp_st *st, const char *title,
-			   const struct vidframe *frame);
+			   const struct vidframe *frame, uint64_t timestamp);
 struct vidisp_st *vidbridge_disp_find(const char *device);
 
 
@@ -46,4 +45,4 @@ int vidbridge_src_alloc(struct vidsrc_st **stp, const struct vidsrc *vs,
 			vidsrc_error_h *errorh, void *arg);
 struct vidsrc_st *vidbridge_src_find(const char *device);
 void vidbridge_src_input(struct vidsrc_st *st,
-			 const struct vidframe *frame);
+			 const struct vidframe *frame, uint64_t timestamp);
