@@ -27,6 +27,7 @@ struct vumeter_enc {
 	const struct audio *au;
 	double avg_rec;
 	volatile bool started;
+	enum aufmt fmt;
 };
 
 struct vumeter_dec {
@@ -35,26 +36,16 @@ struct vumeter_dec {
 	const struct audio *au;
 	double avg_play;
 	volatile bool started;
+	enum aufmt fmt;
 };
 
+
 static bool vumeter_stderr;
 
+
 static void send_event(const struct audio *au, enum ua_event ev, double value)
 {
-	struct stream *strm;
-	struct call *call;
-	struct ua *ua;
-
-	/* get the stream from the audio object */
-	strm = audio_strm(au);
-
-	/* get the call from the stream object */
-	call = stream_call(strm);
-
-	/* get the useragent from the call object */
-	ua = call_get_ua(call);
-
-	ua_event(ua, ev, call, "%.2f", value);
+	audio_level_put(au, ev == UA_EVENT_VU_TX, value);
 }
 
 
@@ -143,23 +134,18 @@ static int encode_update(struct aufilt_enc_st **stp, void **ctx,
 	(void)ctx;
 	(void)prm;
 
-	if (!stp || !af)
+	if (!stp || !af || !prm)
 		return EINVAL;
 
 	if (*stp)
 		return 0;
 
-	if (prm->fmt != AUFMT_S16LE) {
-		warning("vumeter: unsupported sample format (%s)\n",
-			aufmt_name(prm->fmt));
-		return ENOTSUP;
-	}
-
 	st = mem_zalloc(sizeof(*st), enc_destructor);
 	if (!st)
 		return ENOMEM;
 
 	st->au = au;
+	st->fmt = prm->fmt;
 	tmr_start(&st->tmr, 100, enc_tmr_handler, st);
 
 	*stp = (struct aufilt_enc_st *)st;
@@ -176,23 +162,18 @@ static int decode_update(struct aufilt_dec_st **stp, void **ctx,
 	(void)ctx;
 	(void)prm;
 
-	if (!stp || !af)
+	if (!stp || !af || !prm)
 		return EINVAL;
 
 	if (*stp)
 		return 0;
 
-	if (prm->fmt != AUFMT_S16LE) {
-		warning("vumeter: unsupported sample format (%s)\n",
-			aufmt_name(prm->fmt));
-		return ENOTSUP;
-	}
-
 	st = mem_zalloc(sizeof(*st), dec_destructor);
 	if (!st)
 		return ENOMEM;
 
 	st->au = au;
+	st->fmt = prm->fmt;
 	tmr_start(&st->tmr, 100, dec_tmr_handler, st);
 
 	*stp = (struct aufilt_dec_st *)st;
@@ -201,28 +182,28 @@ static int decode_update(struct aufilt_dec_st **stp, void **ctx,
 }
 
 
-static int encode(struct aufilt_enc_st *st, void *sampv, size_t *sampc)
+static int encode(struct aufilt_enc_st *st, struct auframe *af)
 {
 	struct vumeter_enc *vu = (void *)st;
 
-	if (!st || !sampv || !sampc)
+	if (!st || !af)
 		return EINVAL;
 
-	vu->avg_rec = aulevel_calc_dbov(sampv, *sampc);
+	vu->avg_rec = aulevel_calc_dbov(vu->fmt, af->sampv, af->sampc);
 	vu->started = true;
 
 	return 0;
 }
 
 
-static int decode(struct aufilt_dec_st *st, void *sampv, size_t *sampc)
+static int decode(struct aufilt_dec_st *st, struct auframe *af)
 {
 	struct vumeter_dec *vu = (void *)st;
 
-	if (!st || !sampv || !sampc)
+	if (!st || !af)
 		return EINVAL;
 
-	vu->avg_play = aulevel_calc_dbov(sampv, *sampc);
+	vu->avg_play = aulevel_calc_dbov(vu->fmt, af->sampv, af->sampc);
 	vu->started = true;
 
 	return 0;
@@ -230,7 +211,11 @@ static int decode(struct aufilt_dec_st *st, void *sampv, size_t *sampc)
 
 
 static struct aufilt vumeter = {
-	LE_INIT, "vumeter", encode_update, encode, decode_update, decode
+	.name    = "vumeter",
+	.encupdh = encode_update,
+	.ench    = encode,
+	.decupdh = decode_update,
+	.dech    = decode
 };
 
 
