@@ -5,10 +5,10 @@
  */
 
 
+#include <pthread.h>
 #include "modules/audio_processing/aec/echo_cancellation.h"
 
 
-#define MAX_SAMPLE_RATE  16000
 #define MAX_CHANNELS         1
 
 
@@ -21,6 +21,7 @@ struct aec {
 	pthread_mutex_t mutex;
 	uint32_t srate;
 	uint32_t subframe_len;
+	int num_bands;
 };
 
 
@@ -29,7 +30,7 @@ struct aec {
 int webrtc_aec_encode_update(struct aufilt_enc_st **stp, void **ctx,
 			     const struct aufilt *af, struct aufilt_prm *prm,
 			     const struct audio *au);
-int webrtc_aec_encode(struct aufilt_enc_st *st, void *sampv, size_t *sampc);
+int webrtc_aec_encode(struct aufilt_enc_st *st, struct auframe *af);
 
 
 /* Decoder */
@@ -37,7 +38,7 @@ int webrtc_aec_encode(struct aufilt_enc_st *st, void *sampv, size_t *sampc);
 int webrtc_aec_decode_update(struct aufilt_dec_st **stp, void **ctx,
 			     const struct aufilt *af, struct aufilt_prm *prm,
 			     const struct audio *au);
-int webrtc_aec_decode(struct aufilt_dec_st *st, void *sampv, size_t *sampc);
+int webrtc_aec_decode(struct aufilt_dec_st *st, struct auframe *af);
 
 
 /* Common */
