@@ -57,6 +57,7 @@ struct menc_media {
 	void *rtpsock;
 	void *rtcpsock;
 	zrtp_stream_t *zrtp_stream;
+	const struct stream *strm;   /**< pointer to parent */
 };
 
 
@@ -347,15 +348,20 @@ static int session_alloc(struct menc_sess **sessp, struct sdp_session *sdp,
 
 static int media_alloc(struct menc_media **stp, struct menc_sess *sess,
 		       struct rtp_sock *rtp,
-		       int proto, void *rtpsock, void *rtcpsock,
-		       struct sdp_media *sdpm)
+		       struct udp_sock *rtpsock, struct udp_sock *rtcpsock,
+		       const struct sa *raddr_rtp,
+		       const struct sa *raddr_rtcp,
+		       struct sdp_media *sdpm,
+		       const struct stream *strm)
 {
 	struct menc_media *st;
 	zrtp_status_t s;
 	int layer = 10; /* above zero */
 	int err = 0;
+	(void)raddr_rtp;
+	(void)raddr_rtcp;
 
-	if (!stp || !sess || proto != IPPROTO_UDP)
+	if (!stp || !sess)
 		return EINVAL;
 
 	st = *stp;
@@ -367,6 +373,7 @@ static int media_alloc(struct menc_media **stp, struct menc_sess *sess,
 		return ENOMEM;
 
 	st->sess = sess;
+	st->strm = strm;
 	if (rtpsock) {
 		st->rtpsock = mem_ref(rtpsock);
 		err |= udp_register_helper(&st->uh_rtp, rtpsock, layer,
@@ -478,7 +485,9 @@ static void on_zrtp_secure(zrtp_stream_t *stream)
 					sess_info.peer_zid.buffer,
 					(size_t)sess_info.peer_zid.length))
 				(sess->eventh)(MENC_EVENT_VERIFY_REQUEST,
-					       buf, sess->arg);
+					       buf,
+					       (struct stream *)st->strm,
+					       sess->arg);
 			else
 				warning("zrtp: failed to print verify "
 					" arguments\n");
@@ -493,7 +502,9 @@ static void on_zrtp_secure(zrtp_stream_t *stream)
 					sess_info.peer_zid.buffer,
 					(size_t)sess_info.peer_zid.length))
 				(sess->eventh)(MENC_EVENT_PEER_VERIFIED,
-					       buf, sess->arg);
+					       buf,
+					       (struct stream *)st->strm,
+					       sess->arg);
 			else
 				warning("zrtp: failed to print verified "
 					" argument\n");
@@ -522,7 +533,10 @@ static void on_zrtp_security_event(zrtp_stream_t *stream,
 
 
 static struct menc menc_zrtp = {
-	LE_INIT, "zrtp", "RTP/AVP", session_alloc, media_alloc
+	.id        = "zrtp",
+	.sdp_proto = "RTP/AVP",
+	.sessh     = session_alloc,
+	.mediah    = media_alloc
 };
 
 
