@@ -35,6 +35,11 @@ static void destructor(void *arg)
 	mem_deref(acc->aor);
 	mem_deref(acc->dispname);
 	mem_deref(acc->buf);
+	mem_deref(acc->ausrc_mod);
+	mem_deref(acc->ausrc_dev);
+	mem_deref(acc->auplay_mod);
+	mem_deref(acc->auplay_dev);
+	mem_deref(acc->extra);
 }
 
 
@@ -74,27 +79,19 @@ static int param_u32(uint32_t *v, const struct pl *params, const char *name)
 static int stunsrv_decode(struct account *acc, const struct sip_addr *aor)
 {
 	struct pl srv, tmp;
-	struct uri uri;
 	int err;
 
 	if (!acc || !aor)
 		return EINVAL;
 
-	memset(&uri, 0, sizeof(uri));
-
 	if (0 == msg_param_decode(&aor->params, "stunserver", &srv)) {
 
 		info("using stunserver: '%r'\n", &srv);
 
-		err = uri_decode(&uri, &srv);
+		err = stunuri_decode(&acc->stun_host, &srv);
 		if (err) {
-			warning("account: %r: decode failed: %m\n", &srv, err);
-			memset(&uri, 0, sizeof(uri));
-		}
-
-		if (0 != pl_strcasecmp(&uri.scheme, "stun")) {
-			warning("account: unknown scheme: %r\n", &uri.scheme);
-			return EINVAL;
+			warning("account: decode '%r' failed: %m\n",
+				&srv, err);
 		}
 	}
 
@@ -102,25 +99,14 @@ static int stunsrv_decode(struct account *acc, const struct sip_addr *aor)
 
 	if (0 == msg_param_exists(&aor->params, "stunuser", &tmp))
 		err |= param_dstr(&acc->stun_user, &aor->params, "stunuser");
-	else if (pl_isset(&uri.user))
-		err |= pl_strdup(&acc->stun_user, &uri.user);
 	else
 		err |= pl_strdup(&acc->stun_user, &aor->uri.user);
 
 	if (0 == msg_param_exists(&aor->params, "stunpass", &tmp))
 		err |= param_dstr(&acc->stun_pass, &aor->params, "stunpass");
-	else if (pl_isset(&uri.password))
-		err |= pl_strdup(&acc->stun_pass, &uri.password);
 	else if (acc->auth_pass)
 		err |= str_dup(&acc->stun_pass, acc->auth_pass);
 
-	if (pl_isset(&uri.host))
-		err |= pl_strdup(&acc->stun_host, &uri.host);
-	else
-		err |= pl_strdup(&acc->stun_host, &aor->uri.host);
-
-	acc->stun_port = uri.port;
-
 	return err;
 }
 
@@ -141,11 +127,50 @@ static int media_decode(struct account *acc, const struct pl *prm)
 }
 
 
+/* Decode extra parameter */
+static int extra_decode(struct account *acc, const struct pl *prm)
+{
+	int err = 0;
+
+	if (!acc || !prm)
+		return EINVAL;
+
+	err |= param_dstr(&acc->extra, prm, "extra");
+
+	return err;
+}
+
+
+static int decode_pair(char **val1, char **val2,
+		       const struct pl *params, const char *name)
+{
+	struct pl val, pl1, pl2;
+	int err = 0;
+
+	if (0 == msg_param_decode(params, name, &val)) {
+
+		/* note: second value may be quoted */
+		err = re_regex(val.p, val.l, "[^,]+,[~]*", &pl1, &pl2);
+		if (err)
+			return err;
+
+		err  = pl_strdup(val1, &pl1);
+		err |= pl_strdup(val2, &pl2);
+		if (err)
+			return err;
+	}
+
+	return 0;
+}
+
+
 /* Decode answermode parameter */
 static void answermode_decode(struct account *prm, const struct pl *pl)
 {
 	struct pl amode;
 
+	prm->answermode = ANSWERMODE_MANUAL;
+
 	if (0 == msg_param_decode(pl, "answermode", &amode)) {
 
 		if (0 == pl_strcasecmp(&amode, "manual")) {
@@ -159,7 +184,6 @@ static void answermode_decode(struct account *prm, const struct pl *pl)
 		}
 		else {
 			warning("account: answermode unknown (%r)\n", &amode);
-			prm->answermode = ANSWERMODE_MANUAL;
 		}
 	}
 }
@@ -240,7 +264,6 @@ static int audio_codecs_decode(struct account *acc, const struct pl *prm)
 }
 
 
-#ifdef USE_VIDEO
 static int video_codecs_decode(struct account *acc, const struct pl *prm)
 {
 	struct list *vidcodecl = baresip_vidcodecl();
@@ -280,7 +303,6 @@ static int video_codecs_decode(struct account *acc, const struct pl *prm)
 
 	return 0;
 }
-#endif
 
 
 static int sip_params_decode(struct account *acc, const struct sip_addr *aor)
@@ -294,6 +316,12 @@ static int sip_params_decode(struct account *acc, const struct sip_addr *aor)
 
 	acc->regint = REG_INTERVAL + (rand_u32()&0xff);
 	err |= param_u32(&acc->regint, &aor->params, "regint");
+	err |= param_u32(&acc->rwait, &aor->params, "rwait");
+	if (acc->rwait > 95)
+		acc->rwait = 95;
+
+	if (acc->rwait && acc->rwait < 5)
+		acc->rwait = 5;
 
 	acc->pubint = 0;
 	err |= param_u32(&acc->pubint, &aor->params, "pubint");
@@ -392,23 +420,22 @@ int account_alloc(struct account **accp, const char *sipaddr)
 	err |= sip_params_decode(acc, &acc->laddr);
 	       answermode_decode(acc, &acc->laddr.params);
 	err |= audio_codecs_decode(acc, &acc->laddr.params);
-#ifdef USE_VIDEO
 	err |= video_codecs_decode(acc, &acc->laddr.params);
-#endif
 	err |= media_decode(acc, &acc->laddr.params);
 	if (err)
 		goto out;
 
-	/* optional password prompt */
-	if (pl_isset(&acc->laddr.uri.password)) {
-
-		warning("account: username:password is now disabled"
-			" please use ;auth_pass=xxx instead\n");
-
-		err = EINVAL;
+	err  = decode_pair(&acc->ausrc_mod, &acc->ausrc_dev,
+			   &acc->laddr.params, "audio_source");
+	err |= decode_pair(&acc->auplay_mod, &acc->auplay_dev,
+			   &acc->laddr.params, "audio_player");
+	if (err) {
+		warning("account: audio_source/player parse error\n");
 		goto out;
 	}
-	else if (0 == msg_param_decode(&acc->laddr.params, "auth_pass", &pl)) {
+
+	/* optional password prompt */
+	if (0 == msg_param_decode(&acc->laddr.params, "auth_pass", &pl)) {
 		err = pl_strdup(&acc->auth_pass, &pl);
 		if (err)
 			goto out;
@@ -421,7 +448,7 @@ int account_alloc(struct account **accp, const char *sipaddr)
 	if (acc->mnatid) {
 		acc->mnat = mnat_find(baresip_mnatl(), acc->mnatid);
 		if (!acc->mnat) {
-			warning("account: medianat not found: `%s'\n",
+			warning("account: medianat not found: '%s'\n",
 				acc->mnatid);
 		}
 	}
@@ -429,11 +456,13 @@ int account_alloc(struct account **accp, const char *sipaddr)
 	if (acc->mencid) {
 		acc->menc = menc_find(baresip_mencl(), acc->mencid);
 		if (!acc->menc) {
-			warning("account: mediaenc not found: `%s'\n",
+			warning("account: mediaenc not found: '%s'\n",
 				acc->mencid);
 		}
 	}
 
+	err |= extra_decode(acc, &acc->laddr.params);
+
  out:
 	if (err)
 		mem_deref(acc);
@@ -552,6 +581,37 @@ int account_set_regint(struct account *acc, uint32_t regint)
 }
 
 
+/**
+ * Set the STUN server URI for a SIP account
+ *
+ * @param acc   User-Agent account
+ * @param uri   STUN server URI (NULL to reset)
+ *
+ * @return 0 if success, otherwise errorcode
+ */
+int account_set_stun_uri(struct account *acc, const char *uri)
+{
+	struct pl pl;
+	int err;
+
+	if (!acc)
+		return EINVAL;
+
+	acc->stun_host = mem_deref(acc->stun_host);
+
+	if (!uri)
+		return 0;
+
+	pl_set_str(&pl, uri);
+	err = stunuri_decode(&acc->stun_host, &pl);
+	if (err)
+		warning("account: decode '%r' failed: %m\n",
+			&pl, err);
+
+	return err;
+}
+
+
 /**
  * Set the stun host for a SIP account
  *
@@ -565,10 +625,8 @@ int account_set_stun_host(struct account *acc, const char *host)
 	if (!acc)
 		return EINVAL;
 
-	acc->stun_host = mem_deref(acc->stun_host);
-
-	if (host)
-		return str_dup(&acc->stun_host, host);
+	if (acc->stun_host)
+		return stunuri_set_host(acc->stun_host, host);
 
 	return 0;
 }
@@ -587,7 +645,52 @@ int account_set_stun_port(struct account *acc, uint16_t port)
 	if (!acc)
 		return EINVAL;
 
-	acc->stun_port = port;
+	if (acc->stun_host)
+		return stunuri_set_port(acc->stun_host, port);
+
+	return 0;
+}
+
+
+/**
+ * Set the STUN user for a SIP account
+ *
+ * @param acc   User-Agent account
+ * @param user  STUN username (NULL to reset)
+ *
+ * @return 0 if success, otherwise errorcode
+ */
+int account_set_stun_user(struct account *acc, const char *user)
+{
+	if (!acc)
+		return EINVAL;
+
+	acc->stun_user = mem_deref(acc->stun_user);
+
+	if (user)
+		return str_dup(&acc->stun_user, user);
+
+	return 0;
+}
+
+
+/**
+ * Set the STUN password for a SIP account
+ *
+ * @param acc   User-Agent account
+ * @param pass  STUN password (NULL to reset)
+ *
+ * @return 0 if success, otherwise errorcode
+ */
+int account_set_stun_pass(struct account *acc, const char *pass)
+{
+	if (!acc)
+		return EINVAL;
+
+	acc->stun_pass = mem_deref(acc->stun_pass);
+
+	if (pass)
+		return str_dup(&acc->stun_pass, pass);
 
 	return 0;
 }
@@ -668,7 +771,7 @@ int account_set_medianat(struct account *acc, const char *mnatid)
  * Sets audio codecs
  *
  * @param acc      User-Agent account
- * @param codecs   Comma separed list of audio codecs (NULL to disable)
+ * @param codecs   Comma separated list of audio codecs (NULL to disable)
  *
  * @return 0 if success, otherwise errorcode
  */
@@ -683,7 +786,7 @@ int account_set_audio_codecs(struct account *acc, const char *codecs)
 	list_clear(&acc->aucodecl);
 
 	if (codecs) {
-		re_snprintf(buf, sizeof buf, ";audio_codecs=%s", codecs);
+		re_snprintf(buf, sizeof(buf), ";audio_codecs=%s", codecs);
 		pl_set_str(&pl, buf);
 		return audio_codecs_decode(acc, &pl);
 	}
@@ -692,6 +795,34 @@ int account_set_audio_codecs(struct account *acc, const char *codecs)
 }
 
 
+/**
+ * Sets video codecs
+ *
+ * @param acc      User-Agent account
+ * @param codecs   Comma separated list of video codecs (NULL to disable)
+ *
+ * @return 0 if success, otherwise errorcode
+ */
+int account_set_video_codecs(struct account *acc, const char *codecs)
+{
+	char buf[256];
+	struct pl pl;
+
+	if (!acc)
+		return EINVAL;
+
+	list_clear(&acc->vidcodecl);
+
+	if (codecs) {
+		re_snprintf(buf, sizeof(buf), ";video_codecs=%s", codecs);
+		pl_set_str(&pl, buf);
+		return video_codecs_decode(acc, &pl);
+	}
+
+	return 0;
+}
+
+
 /**
  * Sets the displayed name. Pass null in dname to disable display name
  *
@@ -815,7 +946,6 @@ struct list *account_aucodecl(const struct account *acc)
 }
 
 
-#ifdef USE_VIDEO
 /**
  * Get the video codecs of an account
  *
@@ -828,7 +958,6 @@ struct list *account_vidcodecl(const struct account *acc)
 	return (acc && !list_isempty(&acc->vidcodecl))
 		? (struct list *)&acc->vidcodecl : baresip_vidcodecl();
 }
-#endif
 
 
 /**
@@ -844,6 +973,19 @@ struct sip_addr *account_laddr(const struct account *acc)
 }
 
 
+/**
+ * Get the decoded AOR URI of an account
+ *
+ * @param acc User-Agent account
+ *
+ * @return Decoded URI
+ */
+struct uri *account_luri(const struct account *acc)
+{
+	return acc ? (struct uri *)&acc->luri : NULL;
+}
+
+
 /**
  * Get the Registration interval of an account
  *
@@ -1029,6 +1171,22 @@ const char *account_stun_pass(const struct account *acc)
 }
 
 
+/**
+ * Get the STUN server URI of an account
+ *
+ * @param acc User-Agent account
+ *
+ * @return STUN server URI
+ */
+const struct stun_uri *account_stun_uri(const struct account *acc)
+{
+	if (!acc)
+		return NULL;
+
+	return acc->stun_host ? acc->stun_host : NULL;
+}
+
+
 /**
  * Get the STUN hostname of an account
  *
@@ -1038,7 +1196,10 @@ const char *account_stun_pass(const struct account *acc)
  */
 const char *account_stun_host(const struct account *acc)
 {
-	return acc ? acc->stun_host : NULL;
+	if (!acc)
+		return NULL;
+
+	return acc->stun_host ? acc->stun_host->host : NULL;
 }
 
 
@@ -1051,7 +1212,10 @@ const char *account_stun_host(const struct account *acc)
  */
 uint16_t account_stun_port(const struct account *acc)
 {
-	return acc ? acc->stun_port : 0;
+	if (!acc)
+		return 0;
+
+	return acc->stun_host ? acc->stun_host->port : 0;
 }
 
 
@@ -1125,6 +1289,19 @@ const char *account_call_transfer(const struct account *acc)
 }
 
 
+/**
+ * Get extra parameter value of an account
+ *
+ * @param acc User-Agent account
+ *
+ * @return extra parameter value
+ */
+const char *account_extra(const struct account *acc)
+{
+	return acc ? acc->extra : NULL;
+}
+
+
 /**
  * Print the account debug information
  *
@@ -1177,8 +1354,10 @@ int account_debug(struct re_printf *pf, const struct account *acc)
 	err |= re_hprintf(pf, " pubint:       %u\n", acc->pubint);
 	err |= re_hprintf(pf, " regq:         %s\n", acc->regq);
 	err |= re_hprintf(pf, " sipnat:       %s\n", acc->sipnat);
-	err |= re_hprintf(pf, " stunserver:   stun:%s@%s:%u\n",
-			  acc->stun_user, acc->stun_host, acc->stun_port);
+	err |= re_hprintf(pf, " stunuser:     %s\n", acc->stun_user);
+	err |= re_hprintf(pf, " stunserver:   %H\n",
+			  stunuri_print, acc->stun_host);
+
 	if (!list_isempty(&acc->vidcodecl)) {
 		err |= re_hprintf(pf, " video_codecs:");
 		for (le = list_head(&acc->vidcodecl); le; le = le->next) {
@@ -1187,8 +1366,72 @@ int account_debug(struct re_printf *pf, const struct account *acc)
 		}
 		err |= re_hprintf(pf, "\n");
 	}
-	err |= re_hprintf(pf, " call_transfer:         %s\n",
+	err |= re_hprintf(pf, " call_transfer:%s\n",
 			  account_call_transfer(acc));
+	err |= re_hprintf(pf, " extra:        %s\n",
+			  acc->extra ? acc->extra : "none");
 
 	return err;
 }
+
+
+/**
+ * Print the account information in JSON
+ *
+ * @param od  Account dict
+ * @param odcfg  Configuration dict
+ * @param acc User-Agent account
+ *
+ * @return 0 if success, otherwise errorcode
+ */
+int account_json_api(struct odict *od, struct odict *odcfg,
+		const struct account *acc)
+{
+	int err = 0;
+	struct odict *obn = NULL;
+	const char *stunhost = "";
+
+	if (!acc)
+		return 0;
+
+	/* account */
+	err |= odict_entry_add(od, "aor", ODICT_STRING, acc->aor);
+	if (acc->dispname) {
+		err |= odict_entry_add(od, "display_name", ODICT_STRING,
+				acc->dispname);
+	}
+
+	/* config */
+	if (acc->sipnat) {
+		err |= odict_entry_add(odcfg, "sip_nat", ODICT_STRING,
+				acc->sipnat);
+	}
+
+	err |= odict_alloc(&obn, 8);
+	for (size_t i=0; i<ARRAY_SIZE(acc->outboundv); i++) {
+		if (acc->outboundv[i]) {
+			err |= odict_entry_add(obn, "outbound", ODICT_STRING,
+					acc->outboundv[i]);
+		}
+	}
+	err |= odict_entry_add(odcfg, "sip_nat_outbound", ODICT_ARRAY, obn);
+
+	stunhost = account_stun_host(acc) ? account_stun_host(acc) : "";
+	err |= odict_entry_add(odcfg, "stun_host", ODICT_STRING, stunhost);
+	err |= odict_entry_add(odcfg, "stun_port", ODICT_INT,
+			account_stun_port(acc));
+	if (acc->stun_user) {
+		err |= odict_entry_add(odcfg, "stun_user", ODICT_STRING,
+				acc->stun_user);
+	}
+
+	err |= odict_entry_add(odcfg, "answer_mode", ODICT_STRING,
+			answermode_str(acc->answermode));
+	err |= odict_entry_add(odcfg, "call_transfer", ODICT_BOOL, acc->refer);
+
+	err |= odict_entry_add(odcfg, "packet_time", ODICT_INT,
+			account_ptime(acc));
+
+	mem_deref(obn);
+	return err;
+}
