@@ -89,7 +89,9 @@ struct autx {
 	struct auresamp resamp;       /**< Optional resampler for DSP      */
 	struct list filtl;            /**< Audio filters in encoding order */
 	struct mbuf *mb;              /**< Buffer for outgoing RTP packets */
-	char device[64];              /**< Audio source device name        */
+	struct media_ctx **ctx;       /**< Shared A/V source media context */
+	char *module;                 /**< Audio source module name        */
+	char *device;                 /**< Audio source device name        */
 	void *sampv;                  /**< Sample buffer                   */
 	int16_t *sampv_rs;            /**< Sample buffer for resampler     */
 	uint32_t ptime;               /**< Packet time for sending         */
@@ -145,7 +147,8 @@ struct aurx {
 	volatile bool aubuf_started;  /**< Aubuf was started flag          */
 	struct auresamp resamp;       /**< Optional resampler for DSP      */
 	struct list filtl;            /**< Audio filters in decoding order */
-	char device[64];              /**< Audio player device name        */
+	char *module;                 /**< Audio player module name        */
+	char *device;                 /**< Audio player device name        */
 	void *sampv;                  /**< Sample buffer                   */
 	int16_t *sampv_rs;            /**< Sample buffer for resampler     */
 	uint32_t ptime;               /**< Packet time for receiving       */
@@ -156,6 +159,7 @@ struct aurx {
 	enum aufmt dec_fmt;           /**< Sample format for decoder       */
 	bool need_conv;               /**< Sample format conversion needed */
 	struct timestamp_recv ts_recv;/**< Receive timestamp state         */
+	size_t last_sampc;
 
 	struct {
 		uint64_t aubuf_overrun;
@@ -178,6 +182,7 @@ struct audio {
 	bool hold;                    /**< Local hold flag                 */
 	unsigned extmap_aulevel;      /**< ID Range 1-14 inclusive         */
 	audio_event_h *eventh;        /**< Event handler                   */
+	audio_level_h *levelh;        /**< Audio level handler             */
 	audio_err_h *errh;            /**< Audio error handler             */
 	void *arg;                    /**< Handler argument                */
 };
@@ -187,14 +192,38 @@ struct audio {
 static const char *uri_aulevel = "urn:ietf:params:rtp-hdrext:ssrc-audio-level";
 
 
-static double audio_calc_seconds(uint64_t rtp_ts, uint32_t clock_rate)
+/**
+ * Get the current audio receive buffer length in milliseconds
+ *
+ * @param au Audio object
+ *
+ * @return Audio buffer length in [ms]
+ */
+uint64_t audio_jb_current_value(const struct audio *au)
 {
-	double timestamp;
+	const struct aurx *rx;
 
-	/* convert from RTP clockrate to seconds */
-	timestamp = (double)rtp_ts / (double)clock_rate;
+	if (!au)
+		return 0;
 
-	return timestamp;
+	rx = &au->rx;
+
+	if (rx->aubuf) {
+		uint64_t b_p_ms;  /* bytes per ms */
+
+		b_p_ms = aufmt_sample_size(rx->play_fmt) *
+			rx->auplay_prm.srate * rx->auplay_prm.ch / 1000;
+
+		if (b_p_ms) {
+			uint64_t val;
+
+			val = aubuf_cur_size(rx->aubuf) / b_p_ms;
+
+			return val;
+		}
+	}
+
+	return 0;
 }
 
 
@@ -207,7 +236,7 @@ static double autx_calc_seconds(const struct autx *autx)
 
 	dur = autx->ts_ext - autx->ts_base;
 
-	return audio_calc_seconds(dur, autx->ac->crate);
+	return timestamp_calc_seconds(dur, autx->ac->crate);
 }
 
 
@@ -220,7 +249,7 @@ static double aurx_calc_seconds(const struct aurx *aurx)
 
 	dur = timestamp_duration(&aurx->ts_recv);
 
-	return audio_calc_seconds(dur, aurx->ac->crate);
+	return timestamp_calc_seconds(dur, aurx->ac->crate);
 }
 
 
@@ -282,6 +311,10 @@ static void audio_destructor(void *arg)
 	mem_deref(a->rx.aubuf);
 	mem_deref(a->tx.sampv_rs);
 	mem_deref(a->rx.sampv_rs);
+	mem_deref(a->tx.module);
+	mem_deref(a->tx.device);
+	mem_deref(a->rx.module);
+	mem_deref(a->rx.device);
 
 	list_flush(&a->tx.filtl);
 	list_flush(&a->rx.filtl);
@@ -317,46 +350,12 @@ static inline double calc_ptime(size_t nsamp, uint32_t srate, uint8_t channels)
 }
 
 
-/*
- * Get the DSP samplerate for an audio-codec
- */
-static inline uint32_t get_srate(const struct aucodec *ac)
-{
-	if (!ac)
-		return 0;
-
-	return ac->srate;
-}
-
-
-/*
- * Get the DSP channels for an audio-codec
- */
-static inline uint32_t get_ch(const struct aucodec *ac)
-{
-	if (!ac)
-		return 0;
-
-	return ac->ch;
-}
-
-
-static inline uint32_t get_framesize(const struct aucodec *ac,
-				     uint32_t ptime)
-{
-	if (!ac)
-		return 0;
-
-	return calc_nsamp(get_srate(ac), get_ch(ac), ptime);
-}
-
-
 static bool aucodec_equal(const struct aucodec *a, const struct aucodec *b)
 {
 	if (!a || !b)
 		return false;
 
-	return get_srate(a) == get_srate(b) && get_ch(a) == get_ch(b);
+	return a->srate == b->srate && a->ch == b->ch;
 }
 
 
@@ -380,7 +379,7 @@ static int add_audio_codec(struct sdp_media *m, struct aucodec *ac)
 
 
 static int append_rtpext(struct audio *au, struct mbuf *mb,
-			 int16_t *sampv, size_t sampc)
+			 enum aufmt fmt, const void *sampv, size_t sampc)
 {
 	uint8_t data[1];
 	double level;
@@ -388,7 +387,7 @@ static int append_rtpext(struct audio *au, struct mbuf *mb,
 
 	/* audio level must be calculated from the audio samples that
 	 * are actually sent on the network. */
-	level = aulevel_calc_dbov(sampv, sampc);
+	level = aulevel_calc_dbov(fmt, sampv, sampc);
 
 	data[0] = (int)-level & 0x7f;
 
@@ -402,15 +401,10 @@ static int append_rtpext(struct audio *au, struct mbuf *mb,
 }
 
 
-/**
- * Encoder audio and send via stream
+/*
+ * Encode audio and send via stream
  *
  * @note This function has REAL-TIME properties
- *
- * @param a     Audio object
- * @param tx    Audio transmit object
- * @param sampv Audio samples
- * @param sampc Number of audio samples
  */
 static void encode_rtp_send(struct audio *a, struct autx *tx,
 			    int16_t *sampv, size_t sampc)
@@ -420,6 +414,7 @@ static void encode_rtp_send(struct audio *a, struct autx *tx,
 	size_t len;
 	size_t ext_len = 0;
 	uint32_t ts_delta = 0;
+	bool marker = tx->marker;
 	int err;
 
 	if (!tx->ac || !tx->ac->ench)
@@ -432,7 +427,7 @@ static void encode_rtp_send(struct audio *a, struct autx *tx,
 		/* skip the extension header */
 		tx->mb->pos += RTPEXT_HDR_SIZE;
 
-		err = append_rtpext(a, tx->mb, sampv, sampc);
+		err = append_rtpext(a, tx->mb, tx->enc_fmt, sampv, sampc);
 		if (err)
 			return;
 
@@ -451,7 +446,7 @@ static void encode_rtp_send(struct audio *a, struct autx *tx,
 
 	len = mbuf_get_space(tx->mb);
 
-	err = tx->ac->ench(tx->enc, mbuf_buf(tx->mb), &len,
+	err = tx->ac->ench(tx->enc, &marker, mbuf_buf(tx->mb), &len,
 			   tx->enc_fmt, sampv, sampc);
 
 	if ((err & 0xffff0000) == 0x00010000) {
@@ -460,8 +455,6 @@ static void encode_rtp_send(struct audio *a, struct autx *tx,
 
 		ts_delta = err & 0xffff;
 		sampc = 0;
-
-		err = 0;
 	}
 	else if (err) {
 		warning("audio: %s encode error: %d samples (%m)\n",
@@ -477,7 +470,7 @@ static void encode_rtp_send(struct audio *a, struct autx *tx,
 		uint32_t rtp_ts = tx->ts_ext & 0xffffffff;
 
 		if (len) {
-			err = stream_send(a->strm, ext_len!=0, tx->marker, -1,
+			err = stream_send(a->strm, ext_len!=0, marker, -1,
 					  rtp_ts, tx->mb);
 			if (err)
 				goto out;
@@ -497,7 +490,7 @@ static void encode_rtp_send(struct audio *a, struct autx *tx,
 	 * However, MPA support variable packet durations. Thus, MPA
 	 * should update the ts according to its current internal state.
 	 */
-	frame_size = sampc_rtp / tx->ac->pch;
+	frame_size = sampc_rtp / tx->ac->ch;
 
 	tx->ts_ext += (uint32_t)frame_size;
 
@@ -512,6 +505,7 @@ static void encode_rtp_send(struct audio *a, struct autx *tx,
 static void poll_aubuf_tx(struct audio *a)
 {
 	struct autx *tx = &a->tx;
+	struct auframe af;
 	int16_t *sampv = tx->sampv;
 	size_t sampc;
 	size_t sz;
@@ -582,50 +576,60 @@ static void poll_aubuf_tx(struct audio *a)
 		sampc = sampc_rs;
 	}
 
+	auframe_init(&af, tx->enc_fmt, sampv, sampc);
+
 	/* Process exactly one audio-frame in list order */
 	for (le = tx->filtl.head; le; le = le->next) {
 		struct aufilt_enc_st *st = le->data;
 
 		if (st->af && st->af->ench)
-			err |= st->af->ench(st, sampv, &sampc);
+			err |= st->af->ench(st, &af);
 	}
 	if (err) {
 		warning("audio: aufilter encode: %m\n", err);
 	}
 
 	/* Encode and send */
-	encode_rtp_send(a, tx, sampv, sampc);
+	encode_rtp_send(a, tx, af.sampv, af.sampc);
 }
 
 
 static void check_telev(struct audio *a, struct autx *tx)
 {
 	const struct sdp_format *fmt;
+	struct mbuf *mb;
 	bool marker = false;
 	int err;
 
-	tx->mb->pos = tx->mb->end = STREAM_PRESZ;
-
-	err = telev_poll(a->telev, &marker, tx->mb);
-	if (err)
+	mb = mbuf_alloc(STREAM_PRESZ + 64);
+	if (!mb)
 		return;
 
+	mb->pos = mb->end = STREAM_PRESZ;
+
+	err = telev_poll(a->telev, &marker, mb);
+	if (err)
+		goto out;
+
 	if (marker)
 		tx->ts_tel = (uint32_t)tx->ts_ext;
 
 	fmt = sdp_media_rformat(stream_sdpmedia(audio_strm(a)), telev_rtpfmt);
 	if (!fmt)
-		return;
+		goto out;
 
-	tx->mb->pos = STREAM_PRESZ;
-	err = stream_send(a->strm, false, marker, fmt->pt, tx->ts_tel, tx->mb);
+	mb->pos = STREAM_PRESZ;
+	err = stream_send(a->strm, false, marker, fmt->pt, tx->ts_tel, mb);
 	if (err) {
 		warning("audio: telev: stream_send %m\n", err);
 	}
+
+ out:
+	mem_deref(mb);
 }
 
 
-/**
+/*
  * Write samples to Audio Player.
  *
  * @note This function has REAL-TIME properties
@@ -636,10 +640,6 @@ static void check_telev(struct audio *a, struct autx *tx)
  * @note This function may be called from any thread
  *
  * @note The sample format is set in rx->play_fmt
- *
- * @param sampv  Buffer to fill with audio samples
- * @param sampc  Number of samples in buffer
- * @param arg    Handler argument
  */
 static void auplay_write_handler(void *sampv, size_t sampc, void *arg)
 {
@@ -652,7 +652,7 @@ static void auplay_write_handler(void *sampv, size_t sampc, void *arg)
 
 #if 0
 		debug("audio: rx aubuf underrun (total %llu)\n",
-		      rx->stats.aubuf_underrun);
+			rx->stats.aubuf_underrun);
 #endif
 	}
 
@@ -660,25 +660,29 @@ static void auplay_write_handler(void *sampv, size_t sampc, void *arg)
 }
 
 
-/**
+/*
  * Read samples from Audio Source
  *
  * @note This function has REAL-TIME properties
  *
  * @note This function may be called from any thread
- *
- * @param sampv  Buffer with audio samples
- * @param sampc  Number of samples in buffer
- * @param arg    Handler argument
  */
-static void ausrc_read_handler(const void *sampv, size_t sampc, void *arg)
+static void ausrc_read_handler(struct auframe *af, void *arg)
 {
 	struct audio *a = arg;
 	struct autx *tx = &a->tx;
-	size_t num_bytes = sampc * aufmt_sample_size(tx->src_fmt);
+	size_t num_bytes = auframe_size(af);
+
+	if ((int)tx->src_fmt != af->fmt) {
+		warning("audio: ausrc format mismatch:"
+			" expected=%d(%s), actual=%d(%s)\n",
+			tx->src_fmt, aufmt_name(tx->src_fmt),
+			af->fmt, aufmt_name(af->fmt));
+		return;
+	}
 
 	if (tx->muted)
-		memset((void *)sampv, 0, num_bytes);
+		auframe_mute(af);
 
 	if (aubuf_cur_size(tx->aubuf) >= tx->aubuf_maxsz) {
 
@@ -688,7 +692,7 @@ static void ausrc_read_handler(const void *sampv, size_t sampc, void *arg)
 		      tx->stats.aubuf_overrun);
 	}
 
-	(void)aubuf_write(tx->aubuf, sampv, num_bytes);
+	(void)aubuf_write(tx->aubuf, af->sampv, num_bytes);
 
 	tx->aubuf_started = true;
 
@@ -752,8 +756,10 @@ static void handle_telev(struct audio *a, struct mbuf *mb)
 }
 
 
-static int aurx_stream_decode(struct aurx *rx, struct mbuf *mb)
+static int aurx_stream_decode(struct aurx *rx, bool marker,
+			      struct mbuf *mb, unsigned lostc)
 {
+	struct auframe af;
 	size_t sampc = AUDIO_SAMPSZ;
 	void *sampv;
 	struct le *le;
@@ -763,41 +769,50 @@ static int aurx_stream_decode(struct aurx *rx, struct mbuf *mb)
 	if (!rx->ac)
 		return 0;
 
-	if (mbuf_get_left(mb)) {
+	if (lostc && rx->ac->plch) {
+
+		err = rx->ac->plch(rx->dec,
+				   rx->dec_fmt, rx->sampv, &sampc,
+				   mbuf_buf(mb), mbuf_get_left(mb));
+		if (err) {
+			warning("audio: %s codec decode %u bytes: %m\n",
+				rx->ac->name, mbuf_get_left(mb), err);
+			goto out;
+		}
+	}
+	else if (mbuf_get_left(mb)) {
 
 		err = rx->ac->dech(rx->dec,
 				   rx->dec_fmt, rx->sampv, &sampc,
-				   mbuf_buf(mb), mbuf_get_left(mb));
+				   marker, mbuf_buf(mb), mbuf_get_left(mb));
+		if (err) {
+			warning("audio: %s codec decode %u bytes: %m\n",
+				rx->ac->name, mbuf_get_left(mb), err);
+			goto out;
+		}
 
-	}
-	else if (rx->ac->plch && rx->dec_fmt == AUFMT_S16LE) {
-		sampc = rx->ac->srate * rx->ac->ch * rx->ptime / 1000;
-
-		err = rx->ac->plch(rx->dec, rx->dec_fmt, rx->sampv, &sampc);
+		rx->last_sampc = sampc;
 	}
 	else {
 		/* no PLC in the codec, might be done in filters below */
 		sampc = 0;
 	}
 
-	if (err) {
-		warning("audio: %s codec decode %u bytes: %m\n",
-			rx->ac->name, mbuf_get_left(mb), err);
-		goto out;
-	}
+	auframe_init(&af, rx->dec_fmt, rx->sampv, sampc);
 
 	/* Process exactly one audio-frame in reverse list order */
 	for (le = rx->filtl.tail; le; le = le->prev) {
 		struct aufilt_dec_st *st = le->data;
 
 		if (st->af && st->af->dech)
-			err |= st->af->dech(st, rx->sampv, &sampc);
+			err |= st->af->dech(st, &af);
 	}
 
 	if (!rx->aubuf)
 		goto out;
 
-	sampv = rx->sampv;
+	sampv = af.sampv;
+	sampc = af.sampc;
 
 	/* optional resampler */
 	if (rx->resamp.resample) {
@@ -881,7 +896,7 @@ static int aurx_stream_decode(struct aurx *rx, struct mbuf *mb)
 /* Handle incoming stream data from the network */
 static void stream_recv_handler(const struct rtp_header *hdr,
 				struct rtpext *extv, size_t extc,
-				struct mbuf *mb, void *arg)
+				struct mbuf *mb, unsigned lostc, void *arg)
 {
 	struct audio *a = arg;
 	struct aurx *rx = &a->rx;
@@ -953,7 +968,7 @@ static void stream_recv_handler(const struct rtp_header *hdr,
 			warning("audio: [time=%.3f]"
 				" discard old frame (%.3f seconds old)\n",
 				aurx_calc_seconds(rx),
-				audio_calc_seconds(delta, rx->ac->crate));
+				timestamp_calc_seconds(delta, rx->ac->crate));
 
 			discard = true;
 		}
@@ -997,7 +1012,10 @@ static void stream_recv_handler(const struct rtp_header *hdr,
 	}
 
  out:
-	(void)aurx_stream_decode(&a->rx, mb);
+	if (lostc)
+		aurx_stream_decode(&a->rx, hdr->m, mb, lostc);
+
+	(void)aurx_stream_decode(&a->rx, hdr->m, mb, 0);
 }
 
 
@@ -1019,116 +1037,14 @@ static int add_telev_codec(struct audio *a)
 }
 
 
-/*
- * EBU ACIP (Audio Contribution over IP) Profile
- *
- * Ref: https://tech.ebu.ch/docs/tech/tech3368.pdf
- */
-static int set_ebuacip_params(struct audio *au, uint32_t ptime)
-{
-	struct sdp_media *sdp = stream_sdpmedia(au->strm);
-	const struct config_avt *avt = &au->strm->cfg;
-	char str[64];
-	int jbvalue = 0;
-	int jb_id = 0;
-	int err = 0;
-
-	/* set ebuacip version fixed value 0 for now. */
-	err |= sdp_media_set_lattr(sdp, false, "ebuacip", "version %i", 0);
-
-	/* set jb option, only one in our case */
-	err |= sdp_media_set_lattr(sdp, false, "ebuacip", "jb %i", jb_id);
-
-	/* define jb value in option */
-	if (0 == conf_get_str(conf_cur(), "ebuacip_jb_type",str,sizeof(str))) {
-
-		if (0 == str_cmp(str, "auto")) {
-
-			err |= sdp_media_set_lattr(sdp, false,
-						   "ebuacip",
-						   "jbdef %i auto %d-%d",
-						   jb_id,
-						   avt->jbuf_del.min * ptime,
-						   avt->jbuf_del.max * ptime);
-		}
-		else if (0 == str_cmp(str, "fixed")) {
-
-			/* define jb value in option */
-			jbvalue = avt->jbuf_del.max * ptime;
-
-			err |= sdp_media_set_lattr(sdp, false,
-						   "ebuacip",
-						   "jbdef %i fixed %d",
-						   jb_id, jbvalue);
-		}
-	}
-
-	/* set QOS recomendation use tos / 4 to set DSCP value */
-	err |= sdp_media_set_lattr(sdp, false, "ebuacip", "qosrec %u",
-				   avt->rtp_tos / 4);
-
-	/* EBU ACIP FEC:: NOT SET IN BARESIP */
-
-	return err;
-}
-
-
-static bool ebuacip_handler(const char *name, const char *value, void *arg)
-{
-	struct sdp_media *sdp;
-	struct audio *au = arg;
-	struct aurx *rx = &au->rx;
-	struct pl type, val;
-	uint32_t frames;
-	(void)name;
-
-	if (0 == re_regex(value, str_len(value),
-		"jbdef [0-9]+ [^ ]+ [0-9]+",
-		NULL, &type, &val)) {
-
-		const uint32_t ptime = rx->ptime ? rx->ptime : 20;
-
-		frames = pl_u32(&val) / ptime;
-
-		if (0 == pl_strcasecmp(&type,"fixed")) {
-
-			uint32_t frames_min;
-
-			/*
-			fixed jb, set to frames -1 as min and frames as max.
-			*/
-
-			if (frames > 1)
-				frames_min = frames - 1;
-			else
-				frames_min = 1;
-
-			stream_jbuf_reset(au->strm, frames_min, frames);
-		}
-		else if (0 == pl_strcasecmp(&type, "auto")) {
-			/*
-			at the moment only min value is known,
-			therefor max value is here set to 2 times min value
-			This needs to be addressed later
-			*/
-			stream_jbuf_reset(au->strm, frames, frames*2);
-		}
-
-		sdp = stream_sdpmedia(au->strm);
-		sdp_media_del_lattr(sdp, "ebuacip");
-	}
-
-	return false;
-}
-
-
 /**
  * Allocate an audio stream
  *
  * @param ap         Pointer to allocated audio stream object
+ * @param streaml    List of streams
  * @param stream_prm Stream parameters
  * @param cfg        Global configuration
- * @param call       Call object
+ * @param acc        User-Agent account
  * @param sdp_sess   SDP Session
  * @param label      SDP label
  * @param mnat       Media NAT (optional)
@@ -1139,28 +1055,38 @@ static bool ebuacip_handler(const char *name, const char *value, void *arg)
  * @param aucodecl   List of audio codecs
  * @param offerer    True if SDP offerer, false if SDP answerer
  * @param eventh     Event handler
+ * @param levelh     Audio level handler
  * @param errh       Error handler
  * @param arg        Handler argument
  *
  * @return 0 if success, otherwise errorcode
  */
-int audio_alloc(struct audio **ap, const struct stream_param *stream_prm,
+int audio_alloc(struct audio **ap, struct list *streaml,
+		const struct stream_param *stream_prm,
 		const struct config *cfg,
-		struct call *call, struct sdp_session *sdp_sess, int label,
+		struct account *acc, struct sdp_session *sdp_sess, int label,
 		const struct mnat *mnat, struct mnat_sess *mnat_sess,
 		const struct menc *menc, struct menc_sess *menc_sess,
 		uint32_t ptime, const struct list *aucodecl, bool offerer,
-		audio_event_h *eventh, audio_err_h *errh, void *arg)
+		audio_event_h *eventh, audio_level_h *levelh,
+		audio_err_h *errh, void *arg)
 {
 	struct audio *a;
 	struct autx *tx;
 	struct aurx *rx;
 	struct le *le;
+	uint32_t minptime = ptime;
 	int err;
 
 	if (!ap || !cfg)
 		return EINVAL;
 
+	if (ptime < 1 || ptime > MAX_PTIME) {
+		warning("audio: ptime %ums out of range (%ums - %ums)\n",
+			ptime, 1, MAX_PTIME);
+		return ENOTSUP;
+	}
+
 	a = mem_zalloc(sizeof(*a), audio_destructor);
 	if (!a)
 		return ENOMEM;
@@ -1177,9 +1103,10 @@ int audio_alloc(struct audio **ap, const struct stream_param *stream_prm,
 	tx->enc_fmt = cfg->audio.enc_fmt;
 	rx->dec_fmt = cfg->audio.dec_fmt;
 
-	err = stream_alloc(&a->strm, stream_prm, &cfg->avt, call, sdp_sess,
-			   "audio", label,
-			   mnat, mnat_sess, menc, menc_sess,
+	err = stream_alloc(&a->strm, streaml,
+			   stream_prm, &cfg->avt, sdp_sess,
+			   MEDIA_AUDIO, label,
+			   mnat, mnat_sess, menc, menc_sess, offerer,
 			   stream_recv_handler, NULL, a);
 	if (err)
 		goto out;
@@ -1188,8 +1115,23 @@ int audio_alloc(struct audio **ap, const struct stream_param *stream_prm,
 		stream_set_bw(a->strm, AUDIO_BANDWIDTH);
 	}
 
-	err = sdp_media_set_lattr(stream_sdpmedia(a->strm), true,
-				  "ptime", "%u", ptime);
+	/* Audio codecs */
+	for (le = list_head(aucodecl); le; le = le->next) {
+
+		struct aucodec *ac = le->data;
+
+		if (ac->ptime)
+			minptime = min(minptime, ac->ptime);
+
+		err = add_audio_codec(stream_sdpmedia(a->strm), ac);
+		if (err)
+			goto out;
+	}
+
+	err  = sdp_media_set_lattr(stream_sdpmedia(a->strm), true,
+				   "minptime", "%u", minptime);
+	err |= sdp_media_set_lattr(stream_sdpmedia(a->strm), true,
+				   "ptime", "%u", ptime);
 	if (err)
 		goto out;
 
@@ -1205,20 +1147,6 @@ int audio_alloc(struct audio **ap, const struct stream_param *stream_prm,
 			goto out;
 	}
 
-	if (cfg->sdp.ebuacip) {
-
-		err = set_ebuacip_params(a, ptime);
-		if (err)
-			goto out;
-	}
-
-	/* Audio codecs */
-	for (le = list_head(aucodecl); le; le = le->next) {
-		err = add_audio_codec(stream_sdpmedia(a->strm), le->data);
-		if (err)
-			goto out;
-	}
-
 	tx->mb = mbuf_alloc(STREAM_PRESZ + 4096);
 	tx->sampv = mem_zalloc(AUDIO_SAMPSZ * aufmt_sample_size(tx->enc_fmt),
 			       NULL);
@@ -1239,17 +1167,44 @@ int audio_alloc(struct audio **ap, const struct stream_param *stream_prm,
 		goto out;
 
 	auresamp_init(&tx->resamp);
-	str_ncpy(tx->device, a->cfg.src_dev, sizeof(tx->device));
+
+	if (acc && acc->ausrc_mod) {
+
+		tx->module = mem_ref(acc->ausrc_mod);
+		tx->device = mem_ref(acc->ausrc_dev);
+
+		info("audio: using account specific source: (%s,%s)\n",
+		     tx->module, tx->device);
+	}
+	else {
+		err  = str_dup(&tx->module, a->cfg.src_mod);
+		err |= str_dup(&tx->device, a->cfg.src_dev);
+	}
+
 	tx->ptime  = ptime;
 	tx->ts_ext = tx->ts_base = rand_u16();
 	tx->marker = true;
 
 	auresamp_init(&rx->resamp);
-	str_ncpy(rx->device, a->cfg.play_dev, sizeof(rx->device));
+
+	if (acc && acc->auplay_mod) {
+
+		rx->module = mem_ref(acc->auplay_mod);
+		rx->device = mem_ref(acc->auplay_dev);
+
+		info("audio: using account specific player: (%s,%s)\n",
+		     rx->module, rx->device);
+	}
+	else {
+		err  = str_dup(&rx->module, a->cfg.play_mod);
+		err |= str_dup(&rx->device, a->cfg.play_dev);
+	}
+
 	rx->pt     = -1;
 	rx->ptime  = ptime;
 
 	a->eventh  = eventh;
+	a->levelh  = levelh;
 	a->errh    = errh;
 	a->arg     = arg;
 
@@ -1311,17 +1266,10 @@ static void *tx_thread(void *arg)
 
 
 static void aufilt_param_set(struct aufilt_prm *prm,
-			     const struct aucodec *ac, uint32_t ptime,
-			     enum aufmt fmt)
+			     const struct aucodec *ac, enum aufmt fmt)
 {
-	if (!ac) {
-		memset(prm, 0, sizeof(*prm));
-		return;
-	}
-
-	prm->srate      = get_srate(ac);
-	prm->ch         = get_ch(ac);
-	prm->ptime      = ptime;
+	prm->srate      = ac->srate;
+	prm->ch         = ac->ch;
 	prm->fmt        = fmt;
 }
 
@@ -1335,7 +1283,7 @@ static int autx_print_pipeline(struct re_printf *pf, const struct autx *autx)
 		return 0;
 
 	err = re_hprintf(pf, "audio tx pipeline:  %10s",
-			 autx->ausrc ? autx->ausrc->as->name : "src");
+			 autx->ausrc ? autx->ausrc->as->name : "(src)");
 
 	for (le = list_head(&autx->filtl); le; le = le->next) {
 		struct aufilt_enc_st *st = le->data;
@@ -1345,7 +1293,7 @@ static int autx_print_pipeline(struct re_printf *pf, const struct autx *autx)
 	}
 
 	err |= re_hprintf(pf, " ---> %s\n",
-			  autx->ac ? autx->ac->name : "encoder");
+			  autx->ac ? autx->ac->name : "(encoder)");
 
 	return err;
 }
@@ -1360,7 +1308,7 @@ static int aurx_print_pipeline(struct re_printf *pf, const struct aurx *aurx)
 		return 0;
 
 	err = re_hprintf(pf, "audio rx pipeline:  %10s",
-			 aurx->auplay ? aurx->auplay->ap->name : "play");
+			 aurx->auplay ? aurx->auplay->ap->name : "(play)");
 
 	for (le = list_head(&aurx->filtl); le; le = le->next) {
 		struct aufilt_dec_st *st = le->data;
@@ -1370,7 +1318,7 @@ static int aurx_print_pipeline(struct re_printf *pf, const struct aurx *aurx)
 	}
 
 	err |= re_hprintf(pf, " <--- %s\n",
-			  aurx->ac ? aurx->ac->name : "decoder");
+			  aurx->ac ? aurx->ac->name : "(decoder)");
 
 	return err;
 }
@@ -1385,7 +1333,7 @@ static int aurx_print_pipeline(struct re_printf *pf, const struct aurx *aurx)
  *
  * @return 0 if success, otherwise errorcode
  */
-static int aufilt_setup(struct audio *a)
+static int aufilt_setup(struct audio *a, struct list *aufiltl)
 {
 	struct aufilt_prm encprm, decprm;
 	struct autx *tx = &a->tx;
@@ -1400,11 +1348,11 @@ static int aufilt_setup(struct audio *a)
 	if (!list_isempty(&tx->filtl) || !list_isempty(&rx->filtl))
 		return 0;
 
-	aufilt_param_set(&encprm, tx->ac, tx->ptime, tx->enc_fmt);
-	aufilt_param_set(&decprm, rx->ac, rx->ptime, rx->dec_fmt);
+	aufilt_param_set(&encprm, tx->ac, tx->enc_fmt);
+	aufilt_param_set(&decprm, rx->ac, rx->dec_fmt);
 
 	/* Audio filters */
-	for (le = list_head(baresip_aufiltl()); le; le = le->next) {
+	for (le = list_head(aufiltl); le; le = le->next) {
 		struct aufilt *af = le->data;
 		struct aufilt_enc_st *encst = NULL;
 		struct aufilt_dec_st *decst = NULL;
@@ -1445,10 +1393,11 @@ static int aufilt_setup(struct audio *a)
 }
 
 
-static int start_player(struct aurx *rx, struct audio *a)
+static int start_player(struct aurx *rx, struct audio *a,
+			struct list *auplayl)
 {
 	const struct aucodec *ac = rx->ac;
-	uint32_t srate_dsp = get_srate(ac);
+	uint32_t srate_dsp;
 	uint32_t channels_dsp;
 	bool resamp = false;
 	int err;
@@ -1456,7 +1405,8 @@ static int start_player(struct aurx *rx, struct audio *a)
 	if (!ac)
 		return 0;
 
-	channels_dsp = get_ch(ac);
+	srate_dsp    = ac->srate;
+	channels_dsp = ac->ch;
 
 	if (a->cfg.srate_play && a->cfg.srate_play != srate_dsp) {
 		resamp = true;
@@ -1472,7 +1422,7 @@ static int start_player(struct aurx *rx, struct audio *a)
 
 		info("audio: enable auplay resampler:"
 		     " %uHz/%uch --> %uHz/%uch\n",
-		     get_srate(ac), get_ch(ac), srate_dsp, channels_dsp);
+		     ac->srate, ac->ch, srate_dsp, channels_dsp);
 
 		rx->sampv_rs = mem_zalloc(AUDIO_SAMPSZ * sizeof(int16_t),
 					  NULL);
@@ -1480,7 +1430,7 @@ static int start_player(struct aurx *rx, struct audio *a)
 			return ENOMEM;
 
 		err = auresamp_setup(&rx->resamp,
-				     get_srate(ac), get_ch(ac),
+				     ac->srate, ac->ch,
 				     srate_dsp, channels_dsp);
 		if (err) {
 			warning("audio: could not setup auplay resampler"
@@ -1490,7 +1440,7 @@ static int start_player(struct aurx *rx, struct audio *a)
 	}
 
 	/* Start Audio Player */
-	if (!rx->auplay && auplay_find(baresip_auplayl(), NULL)) {
+	if (!rx->auplay && auplay_find(auplayl, NULL)) {
 
 		struct auplay_prm prm;
 
@@ -1500,26 +1450,40 @@ static int start_player(struct aurx *rx, struct audio *a)
 		prm.fmt        = rx->play_fmt;
 
 		if (!rx->aubuf) {
-			size_t psize;
-			size_t sz = aufmt_sample_size(rx->play_fmt);
+			const size_t sz = aufmt_sample_size(rx->play_fmt);
+			const size_t ptime_min = a->cfg.buffer.min;
+			const size_t ptime_max = a->cfg.buffer.max;
+			size_t min_sz;
+			size_t max_sz;
 
-			psize = sz * calc_nsamp(prm.srate, prm.ch, prm.ptime);
+			if (!ptime_min || !ptime_max)
+				return EINVAL;
 
-			rx->aubuf_maxsz = psize * 8;
+			min_sz = sz*calc_nsamp(prm.srate, prm.ch, ptime_min);
+			max_sz = sz*calc_nsamp(prm.srate, prm.ch, ptime_max);
 
-			err = aubuf_alloc(&rx->aubuf, psize * 1,
-					  rx->aubuf_maxsz);
-			if (err)
+			debug("audio: create recv buffer"
+			      " [%zu - %zu ms]"
+			      " [%zu - %zu bytes]\n",
+			      ptime_min, ptime_max, min_sz, max_sz);
+
+			err = aubuf_alloc(&rx->aubuf, min_sz, max_sz);
+			if (err) {
+				warning("audio: aubuf alloc error (%m)\n",
+					err);
 				return err;
+			}
+
+			rx->aubuf_maxsz = max_sz;
 		}
 
-		err = auplay_alloc(&rx->auplay, baresip_auplayl(),
-				   a->cfg.play_mod,
+		err = auplay_alloc(&rx->auplay, auplayl,
+				   rx->module,
 				   &prm, rx->device,
 				   auplay_write_handler, rx);
 		if (err) {
 			warning("audio: start_player failed (%s.%s): %m\n",
-				a->cfg.play_mod, rx->device, err);
+				rx->module, rx->device, err);
 			return err;
 		}
 
@@ -1533,10 +1497,10 @@ static int start_player(struct aurx *rx, struct audio *a)
 }
 
 
-static int start_source(struct autx *tx, struct audio *a)
+static int start_source(struct autx *tx, struct audio *a, struct list *ausrcl)
 {
 	const struct aucodec *ac = tx->ac;
-	uint32_t srate_dsp = get_srate(ac);
+	uint32_t srate_dsp;
 	uint32_t channels_dsp;
 	bool resamp = false;
 	int err;
@@ -1544,7 +1508,8 @@ static int start_source(struct autx *tx, struct audio *a)
 	if (!ac)
 		return 0;
 
-	channels_dsp = get_ch(ac);
+	srate_dsp    = ac->srate;
+	channels_dsp = ac->ch;
 
 	if (a->cfg.srate_src && a->cfg.srate_src != srate_dsp) {
 		resamp = true;
@@ -1560,7 +1525,7 @@ static int start_source(struct autx *tx, struct audio *a)
 
 		info("audio: enable ausrc resampler:"
 		     " %uHz/%uch <-- %uHz/%uch\n",
-		     get_srate(ac), get_ch(ac), srate_dsp, channels_dsp);
+		     ac->srate, ac->ch, srate_dsp, channels_dsp);
 
 		tx->sampv_rs = mem_zalloc(AUDIO_SAMPSZ * sizeof(int16_t),
 					  NULL);
@@ -1569,7 +1534,7 @@ static int start_source(struct autx *tx, struct audio *a)
 
 		err = auresamp_setup(&tx->resamp,
 				     srate_dsp, channels_dsp,
-				     get_srate(ac), get_ch(ac));
+				     ac->srate, ac->ch);
 		if (err) {
 			warning("audio: could not setup ausrc resampler"
 				" (%m)\n", err);
@@ -1578,7 +1543,7 @@ static int start_source(struct autx *tx, struct audio *a)
 	}
 
 	/* Start Audio Source */
-	if (!tx->ausrc && ausrc_find(baresip_ausrcl(), NULL) && !a->hold) {
+	if (!tx->ausrc && ausrc_find(ausrcl, NULL) && !a->hold) {
 
 		struct ausrc_prm prm;
 		size_t sz;
@@ -1601,13 +1566,13 @@ static int start_source(struct autx *tx, struct audio *a)
 				return err;
 		}
 
-		err = ausrc_alloc(&tx->ausrc, baresip_ausrcl(),
-				  NULL, a->cfg.src_mod,
+		err = ausrc_alloc(&tx->ausrc, ausrcl,
+				  tx->ctx, tx->module,
 				  &prm, tx->device,
 				  ausrc_read_handler, ausrc_error_handler, a);
 		if (err) {
 			warning("audio: start_source failed (%s.%s): %m\n",
-				a->cfg.src_mod, tx->device, err);
+				tx->module, tx->device, err);
 			return err;
 		}
 
@@ -1623,7 +1588,7 @@ static int start_source(struct autx *tx, struct audio *a)
 				err = pthread_create(&tx->u.thr.tid, NULL,
 						     tx_thread, a);
 				if (err) {
-					tx->u.thr.tid = false;
+					tx->u.thr.run = false;
 					return err;
 				}
 			}
@@ -1655,6 +1620,7 @@ static int start_source(struct autx *tx, struct audio *a)
  */
 int audio_start(struct audio *a)
 {
+	struct list *aufiltl = baresip_aufiltl();
 	int err;
 
 	if (!a)
@@ -1663,21 +1629,15 @@ int audio_start(struct audio *a)
 	debug("audio: start\n");
 
 	/* Audio filter */
-	if (!list_isempty(baresip_aufiltl())) {
-		err = aufilt_setup(a);
+	if (!list_isempty(aufiltl)) {
+
+		err = aufilt_setup(a, aufiltl);
 		if (err)
 			return err;
 	}
 
-	/* configurable order of play/src start */
-	if (a->cfg.src_first) {
-		err  = start_source(&a->tx, a);
-		err |= start_player(&a->rx, a);
-	}
-	else {
-		err  = start_player(&a->rx, a);
-		err |= start_source(&a->tx, a);
-	}
+	err  = start_player(&a->rx, a, baresip_auplayl());
+	err |= start_source(&a->tx, a, baresip_ausrcl());
 	if (err)
 		return err;
 
@@ -1696,6 +1656,47 @@ int audio_start(struct audio *a)
 }
 
 
+/**
+ * Start the audio source
+ *
+ * @param a       Audio object
+ * @param ausrcl  List of audio sources
+ * @param aufiltl List of audio filters
+ *
+ * @return 0 if success, otherwise errorcode
+ */
+int audio_start_source(struct audio *a, struct list *ausrcl,
+		       struct list *aufiltl)
+{
+	int err;
+
+	if (!a)
+		return EINVAL;
+
+	/* NOTE: audio encoder must be set first */
+	if (!a->tx.ac) {
+		warning("audio: start_source: no encoder set\n");
+		return ENOENT;
+	}
+
+	/* Audio filter */
+	if (!list_isempty(aufiltl)) {
+
+		err = aufilt_setup(a, aufiltl);
+		if (err)
+			return err;
+	}
+
+	err = start_source(&a->tx, a, ausrcl);
+	if (err)
+		return err;
+
+	a->started = true;
+
+	return 0;
+}
+
+
 /**
  * Stop the audio playback and recording
  *
@@ -1753,7 +1754,7 @@ int audio_encoder_set(struct audio *a, const struct aucodec *ac,
 
 	if (ac != tx->ac) {
 		info("audio: Set audio encoder: %s %uHz %dch\n",
-		     ac->name, get_srate(ac), get_ch(ac));
+		     ac->name, ac->srate, ac->ch);
 
 		/* Audio source must be stopped first */
 		if (reset) {
@@ -1778,11 +1779,19 @@ int audio_encoder_set(struct audio *a, const struct aucodec *ac,
 		}
 	}
 
-	stream_set_srate(a->strm, ac->crate, ac->crate);
+	stream_set_srate(a->strm, ac->crate, 0);
 	stream_update_encoder(a->strm, pt_tx);
 
 	telev_set_srate(a->telev, ac->crate);
 
+	/* use a codec-specific ptime */
+	if (ac->ptime) {
+		const size_t sz = aufmt_sample_size(tx->src_fmt);
+
+		tx->ptime = ac->ptime;
+		tx->psize = sz * calc_nsamp(ac->srate, ac->ch, ac->ptime);
+	}
+
 	if (!tx->ausrc) {
 		err |= audio_start(a);
 	}
@@ -1805,6 +1814,7 @@ int audio_decoder_set(struct audio *a, const struct aucodec *ac,
 		      int pt_rx, const char *params)
 {
 	struct aurx *rx;
+	struct sdp_media *m;
 	bool reset = false;
 	int err = 0;
 
@@ -1814,11 +1824,13 @@ int audio_decoder_set(struct audio *a, const struct aucodec *ac,
 	rx = &a->rx;
 
 	reset = !aucodec_equal(ac, rx->ac);
+	m = stream_sdpmedia(audio_strm(a));
+	reset |= sdp_media_dir(m)!=SDP_SENDRECV;
 
 	if (ac != rx->ac) {
 
 		info("audio: Set audio decoder: %s %uHz %dch\n",
-		     ac->name, get_srate(ac), get_ch(ac));
+		     ac->name, ac->srate, ac->ch);
 
 		rx->pt = pt_rx;
 		rx->ac = ac;
@@ -1833,9 +1845,10 @@ int audio_decoder_set(struct audio *a, const struct aucodec *ac,
 		}
 	}
 
-	stream_set_srate(a->strm, ac->crate, ac->crate);
+	stream_set_srate(a->strm, 0, ac->crate);
 
 	if (reset) {
+		stream_reset(audio_strm(a));
 
 		rx->auplay = mem_deref(rx->auplay);
 		aubuf_flush(rx->aubuf);
@@ -1850,28 +1863,6 @@ int audio_decoder_set(struct audio *a, const struct aucodec *ac,
 }
 
 
-/**
- * Use the next audio encoder in the local list of negotiated codecs
- *
- * @param audio  Audio object
- */
-void audio_encoder_cycle(struct audio *audio)
-{
-	const struct sdp_format *rc = NULL;
-
-	if (!audio)
-		return;
-
-	rc = sdp_media_format_cycle(stream_sdpmedia(audio_strm(audio)));
-	if (!rc) {
-		info("audio: encoder cycle: no remote codec found\n");
-		return;
-	}
-
-	(void)audio_encoder_set(audio, rc->data, rc->pt, rc->params);
-}
-
-
 /**
  * Get the RTP Stream object from an Audio object
  *
@@ -2002,7 +1993,8 @@ void audio_sdp_attr_decode(struct audio *a)
 		struct autx *tx = &a->tx;
 		uint32_t ptime_tx = atoi(attr);
 
-		if (ptime_tx && ptime_tx != a->tx.ptime) {
+		if (ptime_tx && ptime_tx != a->tx.ptime
+		    && ptime_tx <= MAX_PTIME) {
 
 			info("audio: peer changed ptime_tx %ums -> %ums\n",
 			     a->tx.ptime, ptime_tx);
@@ -2014,19 +2006,16 @@ void audio_sdp_attr_decode(struct audio *a)
 
 				sz = aufmt_sample_size(tx->src_fmt);
 
-				tx->psize = sz * get_framesize(tx->ac,
-							       ptime_tx);
+				tx->psize = sz * calc_nsamp(tx->ac->srate,
+							    tx->ac->ch,
+							    ptime_tx);
 			}
+
+			sdp_media_set_lattr(stream_sdpmedia(a->strm), true,
+					    "ptime", "%u", ptime_tx);
 		}
 	}
 
-	/*
-	 * EBUACIP handler
-	 * EBU TECH 3368 profile provisioning on incoming invite.
-	 */
-	sdp_media_rattr_apply(stream_sdpmedia(a->strm), "ebuacip",
-			      ebuacip_handler, a);
-
 	/* Client-to-Mixer Audio Level Indication */
 	if (a->cfg.level) {
 		sdp_media_rattr_apply(stream_sdpmedia(a->strm),
@@ -2036,6 +2025,23 @@ void audio_sdp_attr_decode(struct audio *a)
 }
 
 
+/**
+ * Put an audio level value, call the level handler
+ *
+ * @param au  Audio object
+ * @param tx  Direction; true for transmit, false for receive
+ * @param lvl Audio level value
+ */
+void audio_level_put(const struct audio *au, bool tx, double lvl)
+{
+	if (!au)
+		return;
+
+	if (au->levelh)
+		au->levelh(tx, lvl, au->arg);
+}
+
+
 /**
  * Get the last value of the audio level from incoming RTP packets
  *
@@ -2068,7 +2074,7 @@ static int aucodec_print(struct re_printf *pf, const struct aucodec *ac)
 		return 0;
 
 	return re_hprintf(pf, "%s %uHz/%dch",
-			  ac->name, get_srate(ac), get_ch(ac));
+			  ac->name, ac->srate, ac->ch);
 }
 
 
@@ -2121,10 +2127,8 @@ int audio_debug(struct re_printf *pf, const struct audio *a)
 			  autx_calc_seconds(tx));
 
 	err |= re_hprintf(pf,
-			  " rx:   decode: %H %s\n"
-			  "       ptime=%ums pt=%d\n",
-			  aucodec_print, rx->ac, aufmt_name(rx->dec_fmt),
-			  rx->ptime, rx->pt);
+			  " rx:   decode: %H %s\n",
+			  aucodec_print, rx->ac, aufmt_name(rx->dec_fmt));
 	err |= re_hprintf(pf, "       aubuf: %H"
 			  " (cur %.2fms, max %.2fms, or %llu, ur %llu)\n",
 			  aubuf_debug, rx->aubuf,
@@ -2152,7 +2156,7 @@ int audio_debug(struct re_printf *pf, const struct audio *a)
 				  aurx_calc_seconds(rx));
 	}
 	else {
-		err |= re_hprintf(pf, "     time = (not started)\n");
+		err |= re_hprintf(pf, "       time = (not started)\n");
 	}
 
 	err |= re_hprintf(pf,
@@ -2174,14 +2178,23 @@ int audio_debug(struct re_printf *pf, const struct audio *a)
  * @param a     Audio object
  * @param src   Audio source device name
  * @param play  Audio player device name
+ *
+ * @return 0 if success, otherwise errorcode
  */
-void audio_set_devicename(struct audio *a, const char *src, const char *play)
+int audio_set_devicename(struct audio *a, const char *src, const char *play)
 {
-	if (!a)
-		return;
+	int err;
 
-	str_ncpy(a->tx.device, src, sizeof(a->tx.device));
-	str_ncpy(a->rx.device, play, sizeof(a->rx.device));
+	if (!a)
+		return EINVAL;
+
+	a->tx.device = mem_deref(a->tx.device);
+	a->rx.device = mem_deref(a->rx.device);
+
+	err  = str_dup(&a->tx.device, src);
+	err |= str_dup(&a->rx.device, play);
+
+	return err;
 }
 
 
@@ -2263,72 +2276,6 @@ int audio_set_player(struct audio *au, const char *mod, const char *device)
 }
 
 
-/*
- * Reference:
- *
- * https://www.avm.de/de/Extern/files/x-rtp/xrtpv32.pdf
- */
-int audio_print_rtpstat(struct re_printf *pf, const struct audio *a)
-{
-	const struct stream *s;
-	const struct rtcp_stats *rtcp;
-	int srate_tx = 8000;
-	int srate_rx = 8000;
-	int err;
-
-	if (!a)
-		return 1;
-
-	s = a->strm;
-	rtcp = &s->rtcp_stats;
-
-	if (!rtcp->tx.sent)
-		return 1;
-
-	if (a->tx.ac)
-		srate_tx = get_srate(a->tx.ac);
-	if (a->rx.ac)
-		srate_rx = get_srate(a->rx.ac);
-
-	err = re_hprintf(pf,
-			 "EX=BareSip;"   /* Reporter Identifier	             */
-			 "CS=%d;"        /* Call Setup in milliseconds       */
-			 "CD=%d;"        /* Call Duration in seconds	     */
-			 "PR=%u;PS=%u;"  /* Packets RX, TX                   */
-			 "PL=%d,%d;"     /* Packets Lost RX, TX              */
-			 "PD=%d,%d;"     /* Packets Discarded, RX, TX        */
-			 "JI=%.1f,%.1f;" /* Jitter RX, TX in timestamp units */
-			 "IP=%J,%J"      /* Local, Remote IPs                */
-			 ,
-			 call_setup_duration(s->call) * 1000,
-			 call_duration(s->call),
-
-			 s->metric_rx.n_packets,
-			 s->metric_tx.n_packets,
-
-			 rtcp->rx.lost, rtcp->tx.lost,
-
-			 s->metric_rx.n_err, s->metric_tx.n_err,
-
-			 /* timestamp units (ie: 8 ts units = 1 ms @ 8KHZ) */
-			 1.0 * rtcp->rx.jit/1000 * (srate_rx/1000),
-			 1.0 * rtcp->tx.jit/1000 * (srate_tx/1000),
-
-			 sdp_media_laddr(s->sdp),
-			 sdp_media_raddr(s->sdp)
-			 );
-
-	if (a->tx.ac) {
-		err |= re_hprintf(pf, ";EN=%s/%d", a->tx.ac->name, srate_tx );
-	}
-	if (a->rx.ac) {
-		err |= re_hprintf(pf, ";DE=%s/%d", a->rx.ac->name, srate_rx );
-	}
-
-	return err;
-}
-
-
 /**
  * Set the bitrate for the audio encoder
  *
@@ -2428,3 +2375,31 @@ const struct aucodec *audio_codec(const struct audio *au, bool tx)
 
 	return tx ? au->tx.ac : au->rx.ac;
 }
+
+
+/**
+ * Accessor function to audio configuration
+ *
+ * @param au Audio object
+ *
+ * @return Audio configuration
+ */
+struct config_audio *audio_config(struct audio *au)
+{
+	return au ? &au->cfg : NULL;
+}
+
+
+/**
+ * Set the media context
+ *
+ * @param au  Audio object
+ * @param ctx Media context
+ */
+void audio_set_media_context(struct audio *au, struct media_ctx **ctx)
+{
+	if (!au)
+		return;
+
+	au->tx.ctx = ctx;
+}
