@@ -21,22 +21,6 @@
 #define FOREACH_STREAM						\
 	for (le = call->streaml.head; le; le = le->next)
 
-/** Call constants */
-enum {
-	PTIME           = 20,    /**< Packet time for audio               */
-};
-
-
-/** Call States */
-enum state {
-	STATE_IDLE = 0,
-	STATE_INCOMING,
-	STATE_OUTGOING,
-	STATE_RINGING,
-	STATE_EARLY,
-	STATE_ESTABLISHED,
-	STATE_TERMINATED
-};
 
 /** SIP Call Control object */
 struct call {
@@ -50,11 +34,9 @@ struct call {
 	struct sipnot *not;       /**< REFER/NOTIFY client                  */
 	struct list streaml;      /**< List of mediastreams (struct stream) */
 	struct audio *audio;      /**< Audio stream                         */
-#ifdef USE_VIDEO
 	struct video *video;      /**< Video stream                         */
-	struct bfcp *bfcp;        /**< BFCP Client                          */
-#endif
-	enum state state;         /**< Call state                           */
+	struct media_ctx *ctx;    /**< Shared A/V source media context      */
+	enum call_state state;    /**< Call state                           */
 	char *local_uri;          /**< Local SIP uri                        */
 	char *local_name;         /**< Local display name                   */
 	char *peer_uri;           /**< Peer SIP Address                     */
@@ -89,72 +71,101 @@ struct call {
 static int send_invite(struct call *call);
 
 
-static const char *state_name(enum state st)
+static const char *state_name(enum call_state st)
 {
 	switch (st) {
 
-	case STATE_IDLE:        return "IDLE";
-	case STATE_INCOMING:    return "INCOMING";
-	case STATE_OUTGOING:    return "OUTGOING";
-	case STATE_RINGING:     return "RINGING";
-	case STATE_EARLY:       return "EARLY";
-	case STATE_ESTABLISHED: return "ESTABLISHED";
-	case STATE_TERMINATED:  return "TERMINATED";
+	case CALL_STATE_IDLE:        return "IDLE";
+	case CALL_STATE_INCOMING:    return "INCOMING";
+	case CALL_STATE_OUTGOING:    return "OUTGOING";
+	case CALL_STATE_RINGING:     return "RINGING";
+	case CALL_STATE_EARLY:       return "EARLY";
+	case CALL_STATE_ESTABLISHED: return "ESTABLISHED";
+	case CALL_STATE_TERMINATED:  return "TERMINATED";
+	case CALL_STATE_UNKNOWN:     return "UNKNOWN";
 	default:                return "???";
 	}
 }
 
 
-static void set_state(struct call *call, enum state st)
+static void set_state(struct call *call, enum call_state st)
 {
 	call->state = st;
 }
 
 
-static void call_stream_start(struct call *call, bool active)
+static const struct sdp_format *sdp_media_rcodec(const struct sdp_media *m)
+{
+	const struct list *lst;
+	struct le *le;
+
+	if (!m || !sdp_media_rport(m))
+		return NULL;
+
+	lst = sdp_media_format_lst(m, false);
+
+	for (le=list_head(lst); le; le=le->next) {
+
+		const struct sdp_format *fmt = le->data;
+
+		if (!fmt->sup)
+			continue;
+
+		if (!fmt->data)
+			continue;
+
+		return fmt;
+	}
+
+	return NULL;
+}
+
+
+static int start_audio(struct call *call)
 {
 	const struct sdp_format *sc;
-	int err;
-
-	debug("call: stream start (active=%d)\n", active);
+	int err = 0;
 
 	/* Audio Stream */
-	sc = sdp_media_rformat(stream_sdpmedia(audio_strm(call->audio)), NULL);
+	sc = sdp_media_rcodec(stream_sdpmedia(audio_strm(call->audio)));
 	if (sc) {
 		struct aucodec *ac = sc->data;
 
-		if (ac) {
-			err  = audio_encoder_set(call->audio, sc->data,
-						 sc->pt, sc->params);
-			if (err) {
-				warning("call: start:"
-					" audio_encoder_set error: %m\n", err);
-			}
-			err |= audio_decoder_set(call->audio, sc->data,
-						 sc->pt, sc->params);
-			if (err) {
-				warning("call: start:"
-					" audio_decoder_set error: %m\n", err);
-			}
-
-			if (!err) {
-				err = audio_start(call->audio);
-				if (err) {
-					warning("call: start:"
-						" audio_start error: %m\n",
-						err);
-				}
-			}
+		err  = audio_encoder_set(call->audio, ac,
+					 sc->pt, sc->params);
+		if (err) {
+			warning("call: start:"
+				" audio_encoder_set error: %m\n", err);
 		}
-		else {
-			info("call: no common audio-codecs..\n");
+		err |= audio_decoder_set(call->audio, ac,
+					 sc->pt, sc->params);
+		if (err) {
+			warning("call: start:"
+				" audio_decoder_set error: %m\n", err);
+		}
+
+		if (!err) {
+			err = audio_start(call->audio);
+			if (err) {
+				warning("call: start:"
+					" audio_start error: %m\n",
+					err);
+			}
 		}
 	}
 	else {
 		info("call: audio stream is disabled..\n");
 	}
 
-#ifdef USE_VIDEO
+	return err;
+}
+
+
+static int start_video(struct call *call)
+{
+	const struct sdp_format *sc;
+	int err = 0;
+
 	/* Video Stream */
 	sc = sdp_media_rformat(stream_sdpmedia(video_strm(call->video)), NULL);
 	if (sc) {
@@ -162,9 +173,15 @@ static void call_stream_start(struct call *call, bool active)
 					 sc->params);
 		err |= video_decoder_set(call->video, sc->data, sc->pt,
 					 sc->rparams);
-		if (!err && !video_is_started(call->video)) {
-			err = video_start(call->video, call->peer_uri);
+		if (err)
+			return err;
+
+		if (!video_is_started(call->video)) {
+			err  = video_start_display(call->video,
+						   call->peer_uri);
+			err |= video_start_source(call->video, &call->ctx);
 		}
+
 		if (err) {
 			warning("call: video stream error: %m\n", err);
 		}
@@ -173,13 +190,29 @@ static void call_stream_start(struct call *call, bool active)
 		info("call: video stream is disabled..\n");
 	}
 
-	if (call->bfcp) {
-		err = bfcp_start(call->bfcp);
+	return err;
+}
+
+
+static void call_stream_start(struct call *call, bool active)
+{
+	int err;
+
+	debug("call: stream start (active=%d)\n", active);
+
+	if (stream_is_ready(audio_strm(call->audio))) {
+		err = start_audio(call);
 		if (err) {
-			warning("call: could not start BFCP: %m\n", err);
+			warning("call: could not start audio: %m\n", err);
+		}
+	}
+
+	if (stream_is_ready(video_strm(call->video))) {
+		err = start_video(call);
+		if (err) {
+			warning("call: could not start video: %m\n", err);
 		}
 	}
-#endif
 
 	if (active) {
 		struct le *le;
@@ -205,9 +238,7 @@ static void call_stream_stop(struct call *call)
 	audio_stop(call->audio);
 
 	/* Video */
-#ifdef USE_VIDEO
 	video_stop(call->video);
-#endif
 
 	tmr_cancel(&call->tmr_inv);
 }
@@ -263,7 +294,8 @@ static void mnat_handler(int err, uint16_t scode, const char *reason,
 		return;
 	}
 
-	info("call: media-nat '%s' established\n", call->acc->mnatid);
+	info("call: media-nat '%s' established/gathered\n",
+	     call->acc->mnatid);
 
 	/* Re-INVITE */
 	if (!call->mnat_wait) {
@@ -276,11 +308,11 @@ static void mnat_handler(int err, uint16_t scode, const char *reason,
 
 	switch (call->state) {
 
-	case STATE_OUTGOING:
+	case CALL_STATE_OUTGOING:
 		(void)send_invite(call);
 		break;
 
-	case STATE_INCOMING:
+	case CALL_STATE_INCOMING:
 		call_event_handler(call, CALL_EVENT_INCOMING, call->peer_uri);
 		break;
 
@@ -290,49 +322,42 @@ static void mnat_handler(int err, uint16_t scode, const char *reason,
 }
 
 
-static int update_media(struct call *call)
+static int update_audio(struct call *call)
 {
 	const struct sdp_format *sc;
-	struct le *le;
 	int err = 0;
 
-	debug("call: update media\n");
-
-	/* media attributes */
-	audio_sdp_attr_decode(call->audio);
-
-#ifdef USE_VIDEO
-	if (call->video)
-		video_sdp_attr_decode(call->video);
-#endif
-
-	/* Update each stream */
-	FOREACH_STREAM {
-		stream_update(le->data);
-	}
-
-	if (call->acc->mnat && call->acc->mnat->updateh && call->mnats)
-		err = call->acc->mnat->updateh(call->mnats);
-
-	sc = sdp_media_rformat(stream_sdpmedia(audio_strm(call->audio)), NULL);
+	sc = sdp_media_rcodec(stream_sdpmedia(audio_strm(call->audio)));
 	if (sc) {
 		struct aucodec *ac = sc->data;
-		if (ac) {
-			err  = audio_decoder_set(call->audio, sc->data,
-						 sc->pt, sc->params);
-			err |= audio_encoder_set(call->audio, sc->data,
-						 sc->pt, sc->params);
-		}
-		else {
-			info("no common audio-codecs..\n");
+
+		err  = audio_decoder_set(call->audio, ac,
+					 sc->pt, sc->params);
+		if (err) {
+			warning("call: update:"
+				" audio_decoder_set error: %m\n", err);
 		}
+		err |= audio_encoder_set(call->audio, ac,
+					 sc->pt, sc->params);
 	}
 	else {
 		info("audio stream is disabled..\n");
 	}
 
-#ifdef USE_VIDEO
-	sc = sdp_media_rformat(stream_sdpmedia(video_strm(call->video)), NULL);
+	return err;
+}
+
+
+static int update_video(struct call *call)
+{
+	const struct sdp_format *sc = NULL;
+	int err = 0;
+
+	struct sdp_media *m = stream_sdpmedia(video_strm(call->video));
+
+	if (!sdp_media_disabled(m))
+		sc = sdp_media_rformat(m, NULL);
+
 	if (sc) {
 		err = video_encoder_set(call->video, sc->data,
 					sc->pt, sc->params);
@@ -342,7 +367,9 @@ static int update_media(struct call *call)
 		}
 
 		if (!video_is_started(call->video)) {
-			err = video_start(call->video, call->peer_uri);
+			err  = video_start_display(call->video,
+						   call->peer_uri);
+			err |= video_start_source(call->video, &call->ctx);
 			if (err) {
 				warning("call: update: failed to"
 					" start video (%m)\n", err);
@@ -352,8 +379,44 @@ static int update_media(struct call *call)
 	else if (call->video) {
 		info("video stream is disabled..\n");
 		video_stop(call->video);
+		video_stop_display(call->video);
+	}
+
+	return err;
+}
+
+
+static int update_media(struct call *call)
+{
+	struct le *le;
+	int err = 0;
+
+	debug("call: update media\n");
+
+	ua_event(call->ua, UA_EVENT_CALL_REMOTE_SDP, call,
+		 call->got_offer ? "offer" : "answer");
+
+	/* media attributes */
+	audio_sdp_attr_decode(call->audio);
+
+	if (call->video)
+		video_sdp_attr_decode(call->video);
+
+	/* Update each stream */
+	FOREACH_STREAM {
+		stream_update(le->data);
+	}
+
+	if (call->acc->mnat && call->acc->mnat->updateh && call->mnats)
+		err = call->acc->mnat->updateh(call->mnats);
+
+	if (stream_is_ready(audio_strm(call->audio))) {
+		err |= update_audio(call);
+	}
+
+	if (stream_is_ready(video_strm(call->video))) {
+		err |= update_video(call);
 	}
-#endif
 
 	return err;
 }
@@ -374,7 +437,7 @@ static void call_destructor(void *arg)
 {
 	struct call *call = arg;
 
-	if (call->state != STATE_IDLE)
+	if (call->state != CALL_STATE_IDLE)
 		print_summary(call);
 
 	call_stream_stop(call);
@@ -388,10 +451,7 @@ static void call_destructor(void *arg)
 	mem_deref(call->peer_uri);
 	mem_deref(call->peer_name);
 	mem_deref(call->audio);
-#ifdef USE_VIDEO
 	mem_deref(call->video);
-	mem_deref(call->bfcp);
-#endif
 	mem_deref(call->sdp);
 	mem_deref(call->mnats);
 	mem_deref(call->mencs);
@@ -415,6 +475,16 @@ static void audio_event_handler(int key, bool end, void *arg)
 }
 
 
+static void audio_level_handler(bool tx, double lvl, void *arg)
+{
+	struct call *call = arg;
+	MAGIC_CHECK(call);
+
+	ua_event(call->ua, tx ? UA_EVENT_VU_TX : UA_EVENT_VU_RX,
+		 call, "%.2f", lvl);
+}
+
+
 static void audio_error_handler(int err, const char *str, void *arg)
 {
 	struct call *call = arg;
@@ -425,10 +495,11 @@ static void audio_error_handler(int err, const char *str, void *arg)
 	}
 
 	ua_event(call->ua, UA_EVENT_AUDIO_ERROR, call, "%d,%s", err, str);
+	call_stream_stop(call);
+	call_event_handler(call, CALL_EVENT_CLOSED, str);
 }
 
 
-#ifdef USE_VIDEO
 static void video_error_handler(int err, const char *str, void *arg)
 {
 	struct call *call = arg;
@@ -439,7 +510,55 @@ static void video_error_handler(int err, const char *str, void *arg)
 	call_stream_stop(call);
 	call_event_handler(call, CALL_EVENT_CLOSED, str);
 }
-#endif
+
+
+static void menc_event_handler(enum menc_event event,
+			       const char *prm, struct stream *strm, void *arg)
+{
+	struct call *call = arg;
+	int err;
+	(void)strm;
+	MAGIC_CHECK(call);
+
+	debug("call: mediaenc event '%s' (%s)\n", menc_event_name(event), prm);
+
+	switch (event) {
+
+	case MENC_EVENT_SECURE:
+		if (strstr(prm, "audio")) {
+			stream_set_secure(audio_strm(call->audio), true);
+			stream_start(audio_strm(call->audio));
+			err = start_audio(call);
+			if (err) {
+				warning("call: secure: could not"
+					" start audio: %m\n", err);
+			}
+		}
+		else if (strstr(prm, "video")) {
+			stream_set_secure(video_strm(call->video), true);
+			stream_start(video_strm(call->video));
+			err = start_video(call);
+			if (err) {
+				warning("call: secure: could not"
+					" start video: %m\n", err);
+			}
+		}
+		else {
+			info("call: mediaenc: no match for stream (%s)\n",
+			     prm);
+		}
+		break;
+
+	default:
+		break;
+	}
+
+	if (str_isset(prm))
+		call_event_handler(call, CALL_EVENT_MENC, "%u,%s", event,
+				   prm);
+	else
+		call_event_handler(call, CALL_EVENT_MENC, "%u", event);
+}
 
 
 static void menc_error_handler(int err, void *arg)
@@ -454,17 +573,75 @@ static void menc_error_handler(int err, void *arg)
 }
 
 
-static void menc_event_handler(const enum menc_event event,
-			       const char *prm, void *arg)
+static void stream_mnatconn_handler(struct stream *strm, void *arg)
+{
+	struct call *call = arg;
+	int err;
+	MAGIC_CHECK(call);
+
+	if (strm->mencs) {
+		err = stream_start_mediaenc(strm);
+		if (err) {
+			call_event_handler(call, CALL_EVENT_CLOSED,
+					   "mediaenc failed %m", err);
+		}
+	}
+	else if (stream_is_ready(strm)) {
+
+		switch (strm->type) {
+
+		case MEDIA_AUDIO:
+			err = start_audio(call);
+			if (err) {
+				warning("call: mnatconn: could not"
+					" start audio: %m\n", err);
+			}
+			break;
+
+		case MEDIA_VIDEO:
+			err = start_video(call);
+			if (err) {
+				warning("call: mnatconn: could not"
+					" start video: %m\n", err);
+			}
+			break;
+		}
+	}
+}
+
+
+static void stream_rtpestab_handler(struct stream *strm, void *arg)
 {
 	struct call *call = arg;
 	MAGIC_CHECK(call);
 
-	if (str_isset(prm))
-		call_event_handler(call, CALL_EVENT_MENC, "%u,%s", event,
-				   prm);
-	else
-		call_event_handler(call, CALL_EVENT_MENC, "%u", event);
+	ua_event(call->ua, UA_EVENT_CALL_RTPESTAB, call,
+		 "%s", sdp_media_name(stream_sdpmedia(strm)));
+}
+
+
+static void stream_rtcp_handler(struct stream *strm,
+				struct rtcp_msg *msg, void *arg)
+{
+	struct call *call = arg;
+
+	MAGIC_CHECK(call);
+
+	switch (msg->hdr.pt) {
+
+	case RTCP_SR:
+		if (strm->cfg.rtp_stats)
+			call_set_xrtpstat(call);
+
+		ua_event(call->ua, UA_EVENT_CALL_RTCP, call,
+			 "%s", sdp_media_name(stream_sdpmedia(strm)));
+		break;
+
+	case RTCP_APP:
+		ua_event(call->ua, UA_EVENT_CALL_RTCP, call,
+			 "%s", sdp_media_name(stream_sdpmedia(strm)));
+		break;
+	}
 }
 
 
@@ -477,7 +654,7 @@ static void stream_error_handler(struct stream *strm, int err, void *arg)
 		sdp_media_name(stream_sdpmedia(strm)), err);
 
 	call->scode = 701;
-	set_state(call, STATE_TERMINATED);
+	set_state(call, CALL_STATE_TERMINATED);
 
 	call_stream_stop(call);
 	call_event_handler(call, CALL_EVENT_CLOSED, "rtp stream error");
@@ -558,7 +735,7 @@ int call_alloc(struct call **callp, const struct config *cfg, struct list *lst,
 
 	call->acc    = mem_ref(acc);
 	call->ua     = ua;
-	call->state  = STATE_IDLE;
+	call->state  = CALL_STATE_IDLE;
 	call->eh     = eh;
 	call->arg    = arg;
 	call->af     = prm->af;
@@ -585,9 +762,9 @@ int call_alloc(struct call **callp, const struct config *cfg, struct list *lst,
 
 	/* Initialise media NAT handling */
 	if (acc->mnat) {
-		err = acc->mnat->sessh(&call->mnats,
+		err = acc->mnat->sessh(&call->mnats, acc->mnat,
 				       dnsc, call->af,
-				       acc->stun_host, acc->stun_port,
+				       acc->stun_host,
 				       acc->stun_user, acc->stun_pass,
 				       call->sdp, !got_offer,
 				       mnat_handler, call);
@@ -613,15 +790,17 @@ int call_alloc(struct call **callp, const struct config *cfg, struct list *lst,
 	}
 
 	/* Audio stream */
-	err = audio_alloc(&call->audio, &stream_prm, cfg, call,
+	err = audio_alloc(&call->audio, &call->streaml, &stream_prm, cfg, acc,
 			  call->sdp, ++label,
 			  acc->mnat, call->mnats, acc->menc, call->mencs,
 			  acc->ptime, account_aucodecl(call->acc), !got_offer,
-			  audio_event_handler, audio_error_handler, call);
+			  audio_event_handler, audio_level_handler,
+			  audio_error_handler, call);
 	if (err)
 		goto out;
 
-#ifdef USE_VIDEO
+	audio_set_media_context(call->audio, &call->ctx);
+
 	/* We require at least one video codec, and at least one
 	   video source or video display */
 	use_video = (vidmode != VIDMODE_OFF)
@@ -633,30 +812,19 @@ int call_alloc(struct call **callp, const struct config *cfg, struct list *lst,
 
 	/* Video stream */
 	if (use_video) {
-		err = video_alloc(&call->video, &stream_prm, cfg,
-				  call, call->sdp, ++label,
+		err = video_alloc(&call->video, &call->streaml,
+				  &stream_prm, cfg,
+				  call->sdp, ++label,
 				  acc->mnat, call->mnats,
 				  acc->menc, call->mencs,
 				  "main",
 				  account_vidcodecl(call->acc),
+				  baresip_vidfiltl(), !got_offer,
 				  video_error_handler, call);
 		if (err)
 			goto out;
  	}
 
-	if (str_isset(cfg->bfcp.proto)) {
-
-		err = bfcp_alloc(&call->bfcp, call->sdp,
-				 cfg->bfcp.proto, !got_offer,
-				 acc->mnat, call->mnats);
-		if (err)
-			goto out;
-	}
-#else
-	(void)use_video;
-	(void)vidmode;
-#endif
-
 	/* inherit certain properties from original call */
 	if (xcall) {
 		call->not = mem_ref(xcall->not);
@@ -664,7 +832,10 @@ int call_alloc(struct call **callp, const struct config *cfg, struct list *lst,
 
 	FOREACH_STREAM {
 		struct stream *strm = le->data;
-		stream_set_error_handler(strm, stream_error_handler, call);
+		stream_set_session_handlers(strm, stream_mnatconn_handler,
+					    stream_rtpestab_handler,
+					    stream_rtcp_handler,
+					    stream_error_handler, call);
 	}
 
 	if (cfg->avt.rtp_timeout) {
@@ -774,7 +945,7 @@ int call_connect(struct call *call, const struct pl *paddr)
 	if (err)
 		return err;
 
-	set_state(call, STATE_OUTGOING);
+	set_state(call, CALL_STATE_OUTGOING);
 
 	/* If we are using asyncronous medianat like STUN/TURN, then
 	 * wait until completed before sending the INVITE */
@@ -812,19 +983,24 @@ int call_modify(struct call *call)
 }
 
 
-int call_hangup(struct call *call, uint16_t scode, const char *reason)
+/**
+ * Hangup the call
+ *
+ * @param call   Call to hangup
+ * @param scode  Optional status code
+ * @param reason Optional reason
+ */
+void call_hangup(struct call *call, uint16_t scode, const char *reason)
 {
-	int err = 0;
-
 	if (!call)
-		return EINVAL;
+		return;
 
 	if (call->config_avt.rtp_stats)
 		call_set_xrtpstat(call);
 
 	switch (call->state) {
 
-	case STATE_INCOMING:
+	case CALL_STATE_INCOMING:
 		if (scode < 400) {
 			scode = 486;
 			reason = "Rejected";
@@ -839,18 +1015,26 @@ int call_hangup(struct call *call, uint16_t scode, const char *reason)
 		     sip_dialog_callid(sipsess_dialog(call->sess)),
 		     call->peer_uri);
 
+		if (call->not)
+			call_notify_sipfrag(call, 487, "Request Terminated");
+
 		call->sess = mem_deref(call->sess);
 		break;
 	}
 
-	set_state(call, STATE_TERMINATED);
+	set_state(call, CALL_STATE_TERMINATED);
 
 	call_stream_stop(call);
-
-	return err;
 }
 
 
+/**
+ * Answer an incoming call with early media
+ *
+ * @param call Call to answer
+ *
+ * @return 0 if success, otherwise errorcode
+ */
 int call_progress(struct call *call)
 {
 	struct mbuf *desc;
@@ -866,8 +1050,8 @@ int call_progress(struct call *call)
 		return err;
 
 	err = sipsess_progress(call->sess, 183, "Session Progress",
-			       desc, "Allow: %s\r\n",
-			       ua_allowed_methods(call->ua));
+			       desc, "Allow: %H\r\n",
+			       ua_print_allowed, call->ua);
 
 	if (!err)
 		call_stream_start(call, false);
@@ -878,7 +1062,16 @@ int call_progress(struct call *call)
 }
 
 
-int call_answer(struct call *call, uint16_t scode)
+/**
+ * Answer an incoming call
+ *
+ * @param call  Call to answer
+ * @param scode Status code
+ * @param vmode Wanted video mode
+ *
+ * @return 0 if success, otherwise errorcode
+ */
+int call_answer(struct call *call, uint16_t scode, enum vidmode vmode)
 {
 	struct mbuf *desc;
 	int err;
@@ -886,13 +1079,17 @@ int call_answer(struct call *call, uint16_t scode)
 	if (!call || !call->sess)
 		return EINVAL;
 
-	if (STATE_INCOMING != call->state) {
+	if (CALL_STATE_INCOMING != call->state) {
 		info("call: answer: call is not in incoming state (%s)\n",
 		     state_name(call->state));
 		return 0;
 	}
 
-	info("answering call from %s with %u\n", call->peer_uri, scode);
+	if (vmode == VIDMODE_OFF)
+		call->video = mem_deref(call->video);
+
+	info("call: answering call on line %u from %s with %u\n",
+			call->linenum, call->peer_uri, scode);
 
 	if (call->got_offer) {
 
@@ -901,12 +1098,15 @@ int call_answer(struct call *call, uint16_t scode)
 			return err;
 	}
 
+	ua_event(call->ua, UA_EVENT_CALL_LOCAL_SDP, call,
+		 "%s", !call->got_offer ? "offer" : "answer");
+
 	err = sdp_encode(&desc, call->sdp, !call->got_offer);
 	if (err)
 		return err;
 
 	err = sipsess_answer(call->sess, scode, "Answering", desc,
-			     "Allow: %s\r\n", ua_allowed_methods(call->ua));
+			     "Allow: %H\r\n", ua_print_allowed, call->ua);
 
 	mem_deref(desc);
 
@@ -942,11 +1142,7 @@ bool call_has_video(const struct call *call)
 	if (!call)
 		return false;
 
-#ifdef USE_VIDEO
 	return sdp_media_has_media(stream_sdpmedia(video_strm(call->video)));
-#else
-	return false;
-#endif
 }
 
 
@@ -979,8 +1175,29 @@ int call_hold(struct call *call, bool hold)
 }
 
 
+/**
+ * Sets the video direction of the given call
+ *
+ * @param call  Call object
+ * @param dir   SDP media direction
+ *
+ * @return 0 if success, otherwise errorcode
+ */
+int call_set_video_dir(struct call *call, enum sdp_dir dir)
+{
+	if (!call)
+		return EINVAL;
+
+	stream_set_ldir(video_strm(call_video(call)), dir);
+	return call_modify(call);
+}
+
+
 int call_sdp_get(const struct call *call, struct mbuf **descp, bool offer)
 {
+	if (!call)
+		return EINVAL;
+
 	return sdp_encode(descp, call->sdp, offer);
 }
 
@@ -1102,8 +1319,8 @@ int call_status(struct re_printf *pf, const struct call *call)
 
 	switch (call->state) {
 
-	case STATE_EARLY:
-	case STATE_ESTABLISHED:
+	case CALL_STATE_EARLY:
+	case CALL_STATE_ESTABLISHED:
 		break;
 	default:
 		return 0;
@@ -1116,10 +1333,8 @@ int call_status(struct re_printf *pf, const struct call *call)
 
 	err |= re_hprintf(pf, " (bit/s)");
 
-#ifdef USE_VIDEO
 	if (call->video)
 		err |= video_print(pf, call->video);
-#endif
 
 	/* remove old junk */
 	err |= re_hprintf(pf, "    ");
@@ -1213,6 +1428,8 @@ static int sipsess_offer_handler(struct mbuf **descp,
 
 	if (got_offer) {
 
+		call->got_offer = true;
+
 		/* Decode SDP Offer */
 		err = sdp_decode(call->sdp, msg->mb, true);
 		if (err) {
@@ -1240,6 +1457,8 @@ static int sipsess_answer_handler(const struct sip_msg *msg, void *arg)
 
 	debug("call: got SDP answer (%zu bytes)\n", mbuf_get_left(msg->mb));
 
+	call->got_offer = false;
+
 	if (msg_ctype_cmp(&msg->ctyp, "multipart", "mixed"))
 		(void)sdp_decode_multipart(&msg->ctyp.params, msg->mb);
 
@@ -1260,15 +1479,14 @@ static int sipsess_answer_handler(const struct sip_msg *msg, void *arg)
 static void sipsess_estab_handler(const struct sip_msg *msg, void *arg)
 {
 	struct call *call = arg;
+	(void)msg;
 
 	MAGIC_CHECK(call);
 
-	(void)msg;
-
-	if (call->state == STATE_ESTABLISHED)
+	if (call->state == CALL_STATE_ESTABLISHED)
 		return;
 
-	set_state(call, STATE_ESTABLISHED);
+	set_state(call, CALL_STATE_ESTABLISHED);
 
 	call_stream_start(call, true);
 
@@ -1292,7 +1510,6 @@ static void sipsess_estab_handler(const struct sip_msg *msg, void *arg)
 }
 
 
-#ifdef USE_VIDEO
 static void call_handle_info_req(struct call *call, const struct sip_msg *req)
 {
 	struct pl body;
@@ -1311,7 +1528,6 @@ static void call_handle_info_req(struct call *call, const struct sip_msg *req)
 		video_update_picture(call->video);
 	}
 }
-#endif
 
 
 static void dtmfend_handler(void *arg)
@@ -1345,7 +1561,8 @@ static void sipsess_info_handler(struct sip *sip, const struct sip_msg *msg,
 			char s = toupper(sig.p[0]);
 			uint32_t duration = pl_u32(&dur);
 
-			info("received DTMF: '%c' (duration=%r)\n", s, &dur);
+			info("call: received DTMF: '%c' (duration=%r)\n",
+			     s, &dur);
 
 			(void)sip_reply(sip, msg, 200, "OK");
 
@@ -1356,13 +1573,11 @@ static void sipsess_info_handler(struct sip *sip, const struct sip_msg *msg,
 			}
 		}
 	}
-#ifdef USE_VIDEO
 	else if (msg_ctype_cmp(&msg->ctyp,
 			       "application", "media_control+xml")) {
 		call_handle_info_req(call, msg);
 		(void)sip_reply(sip, msg, 200, "OK");
 	}
-#endif
 	else {
 		(void)sip_reply(sip, msg, 488, "Not Acceptable Here");
 	}
@@ -1410,7 +1625,7 @@ static void sipsess_refer_handler(struct sip *sip, const struct sip_msg *msg,
 			      ua_cuser(call->ua), "message/sipfrag",
 			      auth_handler, call->acc, true,
 			      sipnot_close_handler, call,
-			      "Allow: %s\r\n", ua_allowed_methods(call->ua));
+			      "Allow: %H\r\n", ua_print_allowed, call->ua);
 	if (err) {
 		warning("call: refer: sipevent_accept failed: %m\n", err);
 		return;
@@ -1468,7 +1683,7 @@ static bool have_common_audio_codecs(const struct call *call)
 	const struct sdp_format *sc;
 	struct aucodec *ac;
 
-	sc = sdp_media_rformat(stream_sdpmedia(audio_strm(call->audio)), NULL);
+	sc = sdp_media_rcodec(stream_sdpmedia(audio_strm(call->audio)));
 	if (!sc)
 		return false;
 
@@ -1558,8 +1773,8 @@ int call_accept(struct call *call, struct sipsess_sock *sess_sock,
 			     sipsess_estab_handler, sipsess_info_handler,
 			     call->acc->refer ? sipsess_refer_handler : NULL,
 			     sipsess_close_handler,
-			     call, "Allow: %s\r\n",
-			     ua_allowed_methods(call->ua));
+			     call, "Allow: %H\r\n",
+			     ua_print_allowed, call->ua);
 
 	if (err) {
 		warning("call: sipsess_accept: %m\n", err);
@@ -1571,7 +1786,7 @@ int call_accept(struct call *call, struct sipsess_sock *sess_sock,
 	if (err)
 		return err;
 
-	set_state(call, STATE_INCOMING);
+	set_state(call, CALL_STATE_INCOMING);
 
 	/* New call */
 	if (call->config_call.local_timeout) {
@@ -1623,11 +1838,11 @@ static void sipsess_progr_handler(const struct sip_msg *msg, void *arg)
 	switch (msg->scode) {
 
 	case 180:
-		set_state(call, STATE_RINGING);
+		set_state(call, CALL_STATE_RINGING);
 		break;
 
 	case 183:
-		set_state(call, STATE_EARLY);
+		set_state(call, CALL_STATE_EARLY);
 		break;
 	}
 
@@ -1652,7 +1867,9 @@ static int send_invite(struct call *call)
 	struct mbuf *desc;
 	int err;
 
-	routev[0] = ua_outbound(call->ua);
+	routev[0] = account_outbound(call->acc, 0);
+
+	ua_event(call->ua, UA_EVENT_CALL_LOCAL_SDP, call, "offer");
 
 	err = call_sdp_get(call, &desc, true);
 	if (err)
@@ -1679,8 +1896,8 @@ static int send_invite(struct call *call)
 			      sipsess_info_handler,
 			      call->acc->refer ? sipsess_refer_handler : NULL,
 			      sipsess_close_handler, call,
-			      "Allow: %s\r\n%H%H",
-			      ua_allowed_methods(call->ua),
+			      "Allow: %H\r\n%H%H",
+			      ua_print_allowed, call->ua,
 			      ua_print_supported, call->ua,
 			      custom_hdrs_print, &call->custom_hdrs);
 	if (err) {
@@ -1755,12 +1972,7 @@ struct audio *call_audio(const struct call *call)
  */
 struct video *call_video(const struct call *call)
 {
-#ifdef USE_VIDEO
 	return call ? call->video : NULL;
-#else
-	(void)call;
-	return NULL;
-#endif
 }
 
 
@@ -1972,6 +2184,22 @@ uint16_t call_scode(const struct call *call)
 }
 
 
+/**
+ * Get state of the call
+ *
+ * @param call Call object
+ *
+ * @return Call state or CALL_STATE_UNKNOWN if call object is NULL
+ */
+enum call_state call_state(const struct call *call)
+{
+	if (!call)
+		return CALL_STATE_UNKNOWN;
+
+	return call->state;
+}
+
+
 /**
  * Set the callback handlers for a call object
  *
@@ -2004,7 +2232,7 @@ void call_set_xrtpstat(struct call *call)
 
 	sipsess_set_close_headers(call->sess,
 				  "X-RTP-Stat: %H\r\n",
-				  audio_print_rtpstat, call->audio);
+				  rtpstat_print, call);
 }
 
 
@@ -2085,6 +2313,29 @@ struct call *call_find_linenum(const struct list *calls, uint32_t linenum)
 }
 
 
+/**
+ * Find a call by call-id
+ *
+ * @param calls   List of calls
+ * @param id      Call-id string
+ *
+ * @return Call object if found, NULL if not found
+ */
+struct call *call_find_id(const struct list *calls, const char *id)
+{
+	struct le *le;
+
+	for (le = list_head(calls); le; le = le->next) {
+		struct call *call = le->data;
+
+		if (0 == str_cmp(id, call->id))
+			return call;
+	}
+
+	return NULL;
+}
+
+
 /**
  * Set the current call
  *
