@@ -11,14 +11,6 @@
 #include "core.h"
 
 
-#undef MOD_PRE
-#define MOD_PRE ""  /**< Module prefix */
-
-
-#undef SA_INIT
-#define SA_INIT { { {0} }, 0}
-
-
 #ifndef PREFIX
 #define PREFIX "/usr"
 #endif
@@ -29,7 +21,6 @@ static struct config core_config = {
 
 	/** SIP User-Agent */
 	{
-		16,
 		"",
 		"",
 		"",
@@ -52,16 +43,15 @@ static struct config core_config = {
 		0,
 		0,
 		0,
-		false,
 		AUDIO_MODE_POLL,
 		false,
 		AUFMT_S16LE,
 		AUFMT_S16LE,
 		AUFMT_S16LE,
 		AUFMT_S16LE,
+		{20, 160},
 	},
 
-#ifdef USE_VIDEO
 	/** Video */
 	{
 		"", "",
@@ -72,7 +62,6 @@ static struct config core_config = {
 		true,
 		VID_FMT_YUV420P,
 	},
-#endif
 
 	/** Audio/Video Transport */
 	{
@@ -87,22 +76,11 @@ static struct config core_config = {
 
 	/* Network */
 	{
+		AF_UNSPEC,
 		"",
-		{ {""} },
+		{ {"",0} },
 		0
 	},
-
-#ifdef USE_VIDEO
-	/* BFCP */
-	{
-		""
-	},
-#endif
-
-	/* SDP */
-	{
-		false
-	},
 };
 
 
@@ -115,7 +93,7 @@ static int range_print(struct re_printf *pf, const struct range *rng)
 }
 
 
-static int dns_server_handler(const struct pl *pl, void *arg)
+static int dns_handler(const struct pl *pl, void *arg, bool fallback)
 {
 	struct config_net *cfg = arg;
 	const size_t max_count = ARRAY_SIZE(cfg->nsv);
@@ -130,6 +108,9 @@ static int dns_server_handler(const struct pl *pl, void *arg)
 	/* Append dns_server to the network config */
 	err = pl_strcpy(pl, cfg->nsv[cfg->nsc].addr,
 			sizeof(cfg->nsv[0].addr));
+
+	cfg->nsv[cfg->nsc].fallback = fallback;
+
 	if (err) {
 		warning("config: dns_server: could not copy string (%r)\n",
 			pl);
@@ -142,6 +123,26 @@ static int dns_server_handler(const struct pl *pl, void *arg)
 }
 
 
+static int dns_server_handler(const struct pl *pl, void *arg)
+{
+	int err;
+
+	err = dns_handler(pl, arg, false);
+
+	return err;
+}
+
+
+static int dns_fallback_handler(const struct pl *pl, void *arg)
+{
+	int err;
+
+	err = dns_handler(pl, arg, true);
+
+	return err;
+}
+
+
 static enum aufmt resolve_aufmt(const struct pl *fmt)
 {
 	if (0 == pl_strcasecmp(fmt, "s16"))     return AUFMT_S16LE;
@@ -177,7 +178,6 @@ static int conf_get_aufmt(const struct conf *conf, const char *name,
 }
 
 
-#ifdef USE_VIDEO
 static int conf_get_vidfmt(const struct conf *conf, const char *name,
 			   int *fmtp)
 {
@@ -204,7 +204,6 @@ static int conf_get_vidfmt(const struct conf *conf, const char *name,
 
 	return ENOENT;
 }
-#endif
 
 
 /**
@@ -217,12 +216,11 @@ static int conf_get_vidfmt(const struct conf *conf, const char *name,
  */
 int config_parse_conf(struct config *cfg, const struct conf *conf)
 {
-	struct pl pollm, as, ap;
+	struct pl pollm;
 	enum poll_method method;
 	struct vidsz size = {0, 0};
 	struct pl txmode;
 	uint32_t v;
-	bool tmp;
 	int err = 0;
 
 	if (!cfg || !conf)
@@ -243,7 +241,6 @@ int config_parse_conf(struct config *cfg, const struct conf *conf)
 	}
 
 	/* SIP */
-	(void)conf_get_u32(conf, "sip_trans_bsize", &cfg->sip.trans_bsize);
 	(void)conf_get_str(conf, "sip_listen", cfg->sip.local,
 			   sizeof(cfg->sip.local));
 	(void)conf_get_str(conf, "sip_certificate", cfg->sip.cert,
@@ -281,10 +278,6 @@ int config_parse_conf(struct config *cfg, const struct conf *conf)
 	(void)conf_get_u32(conf, "ausrc_channels", &cfg->audio.channels_src);
 	(void)conf_get_u32(conf, "auplay_channels", &cfg->audio.channels_play);
 
-	if (0 == conf_get(conf, "audio_source", &as) &&
-	    0 == conf_get(conf, "audio_player", &ap))
-		cfg->audio.src_first = as.p < ap.p;
-
 	if (0 == conf_get(conf, "audio_txmode", &txmode)) {
 
 		if (0 == pl_strcasecmp(&txmode, "poll"))
@@ -303,7 +296,12 @@ int config_parse_conf(struct config *cfg, const struct conf *conf)
 	conf_get_aufmt(conf, "auenc_format", &cfg->audio.enc_fmt);
 	conf_get_aufmt(conf, "audec_format", &cfg->audio.dec_fmt);
 
-#ifdef USE_VIDEO
+	conf_get_range(conf, "audio_buffer", &cfg->audio.buffer);
+	if (!cfg->audio.buffer.min || !cfg->audio.buffer.max) {
+		warning("config: audio_buffer cannot be zero\n");
+		return EINVAL;
+	}
+
 	/* Video */
 	(void)conf_get_csv(conf, "video_source",
 			   cfg->video.src_mod, sizeof(cfg->video.src_mod),
@@ -320,9 +318,6 @@ int config_parse_conf(struct config *cfg, const struct conf *conf)
 	(void)conf_get_bool(conf, "video_fullscreen", &cfg->video.fullscreen);
 
 	conf_get_vidfmt(conf, "videnc_format", &cfg->video.enc_fmt);
-#else
-	(void)size;
-#endif
 
 	/* AVT - Audio/Video Transport */
 	if (0 == conf_get_u32(conf, "rtp_tos", &v))
@@ -334,12 +329,6 @@ int config_parse_conf(struct config *cfg, const struct conf *conf)
 		cfg->avt.rtp_bw.max *= 1000;
 	}
 
-#if 1
-	if (0 == conf_get_bool(conf, "rtcp_enable", &tmp) && !tmp) {
-		warning("config: rtcp_enable ignored, always enabled\n");
-	}
-#endif
-
 	(void)conf_get_bool(conf, "rtcp_mux", &cfg->avt.rtcp_mux);
 	(void)conf_get_range(conf, "jitter_buffer_delay",
 			     &cfg->avt.jbuf_del);
@@ -352,18 +341,11 @@ int config_parse_conf(struct config *cfg, const struct conf *conf)
 
 	/* Network */
 	(void)conf_apply(conf, "dns_server", dns_server_handler, &cfg->net);
+	(void)conf_apply(conf, "dns_fallback",
+			   dns_fallback_handler, &cfg->net);
 	(void)conf_get_str(conf, "net_interface",
 			   cfg->net.ifname, sizeof(cfg->net.ifname));
 
-#ifdef USE_VIDEO
-	/* BFCP */
-	(void)conf_get_str(conf, "bfcp_proto", cfg->bfcp.proto,
-			   sizeof(cfg->bfcp.proto));
-#endif
-
-	/* SDP */
-	(void)conf_get_bool(conf, "sdp_ebuacip", &cfg->sdp.ebuacip);
-
 	return err;
 }
 
@@ -386,9 +368,9 @@ int config_print(struct re_printf *pf, const struct config *cfg)
 	err = re_hprintf(pf,
 			 "\n"
 			 "# SIP\n"
-			 "sip_trans_bsize\t\t%u\n"
 			 "sip_listen\t\t%s\n"
 			 "sip_certificate\t%s\n"
+			 "sip_cafile\t\t%s\n"
 			 "\n"
 			 "# Call\n"
 			 "call_local_timeout\t%u\n"
@@ -405,7 +387,6 @@ int config_print(struct re_printf *pf, const struct config *cfg)
 			 "ausrc_channels\t\t%u\n"
 			 "audio_level\t\t%s\n"
 			 "\n"
-#ifdef USE_VIDEO
 			 "# Video\n"
 			 "video_source\t\t%s,%s\n"
 			 "video_display\t\t%s,%s\n"
@@ -415,7 +396,6 @@ int config_print(struct re_printf *pf, const struct config *cfg)
 			 "video_fullscreen\t%s\n"
 			 "videnc_format\t\t%s\n"
 			 "\n"
-#endif
 			 "# AVT\n"
 			 "rtp_tos\t\t\t%u\n"
 			 "rtp_ports\t\t%H\n"
@@ -428,14 +408,9 @@ int config_print(struct re_printf *pf, const struct config *cfg)
 			 "# Network\n"
 			 "net_interface\t\t%s\n"
 			 "\n"
-#ifdef USE_VIDEO
-			 "# BFCP\n"
-			 "bfcp_proto\t\t%s\n"
-			 "\n"
-#endif
 			 ,
 
-			 cfg->sip.trans_bsize, cfg->sip.local, cfg->sip.cert,
+			 cfg->sip.local, cfg->sip.cert, cfg->sip.cafile,
 
 			 cfg->call.local_timeout,
 			 cfg->call.max_calls,
@@ -448,14 +423,12 @@ int config_print(struct re_printf *pf, const struct config *cfg)
 			 cfg->audio.channels_play, cfg->audio.channels_src,
 			 cfg->audio.level ? "yes" : "no",
 
-#ifdef USE_VIDEO
 			 cfg->video.src_mod, cfg->video.src_dev,
 			 cfg->video.disp_mod, cfg->video.disp_dev,
 			 cfg->video.width, cfg->video.height,
 			 cfg->video.bitrate, cfg->video.fps,
 			 cfg->video.fullscreen ? "yes" : "no",
 			 vidfmt_name(cfg->video.enc_fmt),
-#endif
 
 			 cfg->avt.rtp_tos,
 			 range_print, &cfg->avt.rtp_ports,
@@ -466,19 +439,29 @@ int config_print(struct re_printf *pf, const struct config *cfg)
 			 cfg->avt.rtp_timeout,
 
 			 cfg->net.ifname
-
-#ifdef USE_VIDEO
-			 ,cfg->bfcp.proto
-#endif
 		   );
 
 	return err;
 }
 
 
+static const char *default_cafile(void)
+{
+#if defined (DEFAULT_CAFILE)
+	return DEFAULT_CAFILE;
+#elif defined (DARWIN)
+	return "/etc/ssl/cert.pem";
+#else
+	return "/etc/ssl/certs/ca-certificates.crt";
+#endif
+}
+
+
 static const char *default_audio_device(void)
 {
-#if defined (ANDROID)
+#if defined (DEFAULT_AUDIO_DEVICE)
+	return DEFAULT_AUDIO_DEVICE;
+#elif defined (ANDROID)
 	return "opensles,nil";
 #elif defined (DARWIN)
 	return "coreaudio,default";
@@ -494,7 +477,6 @@ static const char *default_audio_device(void)
 }
 
 
-#ifdef USE_VIDEO
 static const char *default_video_device(void)
 {
 #ifdef DARWIN
@@ -516,15 +498,27 @@ static const char *default_video_device(void)
 static const char *default_video_display(void)
 {
 #ifdef DARWIN
-	return "opengl,nil";
+	return "sdl,nil";
 #elif defined (WIN32)
-	return "sdl2,nil";
+	return "sdl,nil";
 #else
 	return "x11,nil";
 #endif
 }
+
+
+static const char *default_avcodec_hwaccel(void)
+{
+#if defined (LINUX)
+	return "vaapi";
+#elif defined (DARWIN)
+	return "videotoolbox";
+#else
+	return "none";
 #endif
 
+}
+
 
 static int default_interface_print(struct re_printf *pf, void *unused)
 {
@@ -556,9 +550,9 @@ static int core_config_template(struct re_printf *pf, const struct config *cfg)
 #endif
 				"\n"
 			  "\n# SIP\n"
-			  "sip_trans_bsize\t\t128\n"
 			  "#sip_listen\t\t0.0.0.0:5060\n"
 			  "#sip_certificate\tcert.pem\n"
+			  "#sip_cafile\t\t%s\n"
 			  "\n"
 			  "# Call\n"
 			  "call_local_timeout\t%u\n"
@@ -585,15 +579,17 @@ static int core_config_template(struct re_printf *pf, const struct config *cfg)
 			  "auplay_format\t\ts16\t\t# s16, float, ..\n"
 			  "auenc_format\t\ts16\t\t# s16, float, ..\n"
 			  "audec_format\t\ts16\t\t# s16, float, ..\n"
+			  "audio_buffer\t\t%H\t\t# ms\n"
 			  ,
 			  poll_method_name(poll_method_best()),
+			  default_cafile(),
 			  cfg->call.local_timeout,
 			  cfg->call.max_calls,
 			  default_audio_device(),
 			  default_audio_device(),
-			  default_audio_device());
+			  default_audio_device(),
+			  range_print, &cfg->audio.buffer);
 
-#ifdef USE_VIDEO
 	err |= re_hprintf(pf,
 			  "\n# Video\n"
 			  "#video_source\t\t%s\n"
@@ -601,7 +597,7 @@ static int core_config_template(struct re_printf *pf, const struct config *cfg)
 			  "video_size\t\t%dx%d\n"
 			  "video_bitrate\t\t%u\n"
 			  "video_fps\t\t%.2f\n"
-			  "video_fullscreen\tyes\n"
+			  "video_fullscreen\tno\n"
 			  "videnc_format\t\t%s\n"
 			  ,
 			  default_video_device(),
@@ -609,7 +605,6 @@ static int core_config_template(struct re_printf *pf, const struct config *cfg)
 			  cfg->video.width, cfg->video.height,
 			  cfg->video.bitrate, cfg->video.fps,
 			  vidfmt_name(cfg->video.enc_fmt));
-#endif
 
 	err |= re_hprintf(pf,
 			  "\n# AVT - Audio/Video Transport\n"
@@ -621,17 +616,13 @@ static int core_config_template(struct re_printf *pf, const struct config *cfg)
 			  "rtp_stats\t\tno\n"
 			  "#rtp_timeout\t\t60\n"
 			  "\n# Network\n"
-			  "#dns_server\t\t10.0.0.1:53\n"
+			  "#dns_server\t\t1.1.1.1:53\n"
+			  "#dns_server\t\t1.0.0.1:53\n"
+			  "#dns_fallback\t\t8.8.8.8:53\n"
 			  "#net_interface\t\t%H\n",
 			  cfg->avt.jbuf_del.min, cfg->avt.jbuf_del.max,
 			  default_interface_print, NULL);
 
-#ifdef USE_VIDEO
-	err |= re_hprintf(pf,
-			  "\n# BFCP\n"
-			  "#bfcp_proto\t\tudp\n");
-#endif
-
 	return err;
 }
 
@@ -747,161 +738,152 @@ int config_write_template(const char *file, const struct config *cfg)
 
 	(void)re_fprintf(f, "\n# UI Modules\n");
 #if defined (WIN32)
-	(void)re_fprintf(f, "module\t\t\t" MOD_PRE "wincons" MOD_EXT "\n");
+	(void)re_fprintf(f, "module\t\t\t" "wincons" MOD_EXT "\n");
 #else
-	(void)re_fprintf(f, "module\t\t\t" MOD_PRE "stdio" MOD_EXT "\n");
+	(void)re_fprintf(f, "module\t\t\t" "stdio" MOD_EXT "\n");
 #endif
-	(void)re_fprintf(f, "#module\t\t\t" MOD_PRE "cons" MOD_EXT "\n");
-	(void)re_fprintf(f, "#module\t\t\t" MOD_PRE "evdev" MOD_EXT "\n");
-	(void)re_fprintf(f, "#module\t\t\t" MOD_PRE "httpd" MOD_EXT "\n");
+	(void)re_fprintf(f, "#module\t\t\t" "cons" MOD_EXT "\n");
+	(void)re_fprintf(f, "#module\t\t\t" "evdev" MOD_EXT "\n");
+	(void)re_fprintf(f, "#module\t\t\t" "httpd" MOD_EXT "\n");
 
 	(void)re_fprintf(f, "\n# Audio codec Modules (in order)\n");
-	(void)re_fprintf(f, "#module\t\t\t" MOD_PRE "opus" MOD_EXT "\n");
-	(void)re_fprintf(f, "#module\t\t\t" MOD_PRE "amr" MOD_EXT "\n");
-	(void)re_fprintf(f, "#module\t\t\t" MOD_PRE "g7221" MOD_EXT "\n");
-	(void)re_fprintf(f, "#module\t\t\t" MOD_PRE "g722" MOD_EXT "\n");
-	(void)re_fprintf(f, "#module\t\t\t" MOD_PRE "g726" MOD_EXT "\n");
-	(void)re_fprintf(f, "module\t\t\t" MOD_PRE "g711" MOD_EXT "\n");
-	(void)re_fprintf(f, "#module\t\t\t" MOD_PRE "gsm" MOD_EXT "\n");
-	(void)re_fprintf(f, "#module\t\t\t" MOD_PRE "l16" MOD_EXT "\n");
-	(void)re_fprintf(f, "#module\t\t\t" MOD_PRE "bv32" MOD_EXT "\n");
-	(void)re_fprintf(f, "#module\t\t\t" MOD_PRE "mpa" MOD_EXT "\n");
-	(void)re_fprintf(f, "#module\t\t\t" MOD_PRE "codec2" MOD_EXT "\n");
-	(void)re_fprintf(f, "#module\t\t\t" MOD_PRE "ilbc" MOD_EXT "\n");
-	(void)re_fprintf(f, "#module\t\t\t" MOD_PRE "isac" MOD_EXT "\n");
+	(void)re_fprintf(f, "#module\t\t\t" "opus" MOD_EXT "\n");
+	(void)re_fprintf(f, "#module\t\t\t" "amr" MOD_EXT "\n");
+	(void)re_fprintf(f, "#module\t\t\t" "g7221" MOD_EXT "\n");
+	(void)re_fprintf(f, "#module\t\t\t" "g722" MOD_EXT "\n");
+	(void)re_fprintf(f, "#module\t\t\t" "g726" MOD_EXT "\n");
+	(void)re_fprintf(f, "module\t\t\t" "g711" MOD_EXT "\n");
+	(void)re_fprintf(f, "#module\t\t\t" "gsm" MOD_EXT "\n");
+	(void)re_fprintf(f, "#module\t\t\t" "l16" MOD_EXT "\n");
+	(void)re_fprintf(f, "#module\t\t\t" "mpa" MOD_EXT "\n");
+	(void)re_fprintf(f, "#module\t\t\t" "codec2" MOD_EXT "\n");
+	(void)re_fprintf(f, "#module\t\t\t" "ilbc" MOD_EXT "\n");
+	(void)re_fprintf(f, "#module\t\t\t" "isac" MOD_EXT "\n");
 
 	(void)re_fprintf(f, "\n# Audio filter Modules (in encoding order)\n");
-	(void)re_fprintf(f, "module\t\t\t" MOD_PRE "vumeter" MOD_EXT "\n");
-	(void)re_fprintf(f, "#module\t\t\t" MOD_PRE "sndfile" MOD_EXT "\n");
-	(void)re_fprintf(f, "#module\t\t\t" MOD_PRE "speex_aec" MOD_EXT "\n");
-	(void)re_fprintf(f, "#module\t\t\t" MOD_PRE "speex_pp" MOD_EXT "\n");
-	(void)re_fprintf(f, "#module\t\t\t" MOD_PRE "plc" MOD_EXT "\n");
+	(void)re_fprintf(f, "#module\t\t\t" "vumeter" MOD_EXT "\n");
+	(void)re_fprintf(f, "#module\t\t\t" "sndfile" MOD_EXT "\n");
+	(void)re_fprintf(f, "#module\t\t\t" "speex_pp" MOD_EXT "\n");
+	(void)re_fprintf(f, "#module\t\t\t" "plc" MOD_EXT "\n");
+	(void)re_fprintf(f, "#module\t\t\t" "webrtc_aec" MOD_EXT "\n");
 
 	(void)re_fprintf(f, "\n# Audio driver Modules\n");
 #if defined (ANDROID)
-	(void)re_fprintf(f, "module\t\t\t" MOD_PRE "opensles" MOD_EXT "\n");
+	(void)re_fprintf(f, "module\t\t\t" "opensles" MOD_EXT "\n");
 #elif defined (DARWIN)
-	(void)re_fprintf(f, "module\t\t\t" MOD_PRE "coreaudio" MOD_EXT "\n");
-	(void)re_fprintf(f, "#module\t\t\t" MOD_PRE "audiounit" MOD_EXT "\n");
+	(void)re_fprintf(f, "module\t\t\t" "coreaudio" MOD_EXT "\n");
+	(void)re_fprintf(f, "#module\t\t\t" "audiounit" MOD_EXT "\n");
 #elif defined (FREEBSD)
-	(void)re_fprintf(f, "module\t\t\t" MOD_PRE "oss" MOD_EXT "\n");
+	(void)re_fprintf(f, "module\t\t\t" "oss" MOD_EXT "\n");
 #elif defined (OPENBSD)
-	(void)re_fprintf(f, "module\t\t\t" MOD_PRE "sndio" MOD_EXT "\n");
+	(void)re_fprintf(f, "module\t\t\t" "sndio" MOD_EXT "\n");
 #elif defined (WIN32)
-	(void)re_fprintf(f, "module\t\t\t" MOD_PRE "winwave" MOD_EXT "\n");
+	(void)re_fprintf(f, "module\t\t\t" "winwave" MOD_EXT "\n");
 #else
-	(void)re_fprintf(f, "module\t\t\t" MOD_PRE "alsa" MOD_EXT "\n");
-	(void)re_fprintf(f, "#module\t\t\t" MOD_PRE "pulse" MOD_EXT "\n");
+	if (!strncmp(default_audio_device(), "pulse", 5)) {
+		(void)re_fprintf(f, "#module\t\t\t" "alsa" MOD_EXT "\n");
+		(void)re_fprintf(f, "module\t\t\t" "pulse" MOD_EXT "\n");
+	}
+	else {
+		(void)re_fprintf(f, "module\t\t\t" "alsa" MOD_EXT "\n");
+		(void)re_fprintf(f, "#module\t\t\t" "pulse" MOD_EXT "\n");
+	}
 #endif
-	(void)re_fprintf(f, "#module\t\t\t" MOD_PRE "jack" MOD_EXT "\n");
-	(void)re_fprintf(f, "#module\t\t\t" MOD_PRE "portaudio" MOD_EXT "\n");
-	(void)re_fprintf(f, "#module\t\t\t" MOD_PRE "aubridge" MOD_EXT "\n");
-	(void)re_fprintf(f, "#module\t\t\t" MOD_PRE "aufile" MOD_EXT "\n");
+	(void)re_fprintf(f, "#module\t\t\t" "jack" MOD_EXT "\n");
+	(void)re_fprintf(f, "#module\t\t\t" "portaudio" MOD_EXT "\n");
+	(void)re_fprintf(f, "#module\t\t\t" "aubridge" MOD_EXT "\n");
+	(void)re_fprintf(f, "#module\t\t\t" "aufile" MOD_EXT "\n");
+	(void)re_fprintf(f, "#module\t\t\t" "ausine" MOD_EXT "\n");
 
-#ifdef USE_VIDEO
 
 	(void)re_fprintf(f, "\n# Video codec Modules (in order)\n");
-#ifdef USE_AVCODEC
-	(void)re_fprintf(f, "module\t\t\t" MOD_PRE "avcodec" MOD_EXT "\n");
-#else
-	(void)re_fprintf(f, "#module\t\t\t" MOD_PRE "avcodec" MOD_EXT "\n");
-#endif
-	(void)re_fprintf(f, "#module\t\t\t" MOD_PRE "vp8" MOD_EXT "\n");
-	(void)re_fprintf(f, "#module\t\t\t" MOD_PRE "vp9" MOD_EXT "\n");
-	(void)re_fprintf(f, "#module\t\t\t" MOD_PRE "h265" MOD_EXT "\n");
+	(void)re_fprintf(f, "#module\t\t\t" "avcodec" MOD_EXT "\n");
+	(void)re_fprintf(f, "#module\t\t\t" "vp8" MOD_EXT "\n");
+	(void)re_fprintf(f, "#module\t\t\t" "vp9" MOD_EXT "\n");
 
 	(void)re_fprintf(f, "\n# Video filter Modules (in encoding order)\n");
-	(void)re_fprintf(f, "#module\t\t\t" MOD_PRE "selfview" MOD_EXT "\n");
-	(void)re_fprintf(f, "#module\t\t\t" MOD_PRE "snapshot" MOD_EXT "\n");
-	(void)re_fprintf(f, "#module\t\t\t" MOD_PRE "swscale" MOD_EXT "\n");
-	(void)re_fprintf(f, "#module\t\t\t" MOD_PRE "vidinfo" MOD_EXT "\n");
+	(void)re_fprintf(f, "#module\t\t\t" "selfview" MOD_EXT "\n");
+	(void)re_fprintf(f, "#module\t\t\t" "snapshot" MOD_EXT "\n");
+	(void)re_fprintf(f, "#module\t\t\t" "swscale" MOD_EXT "\n");
+	(void)re_fprintf(f, "#module\t\t\t" "vidinfo" MOD_EXT "\n");
+	(void)re_fprintf(f, "#module\t\t\t" "avfilter" MOD_EXT "\n");
 
 	(void)re_fprintf(f, "\n# Video source modules\n");
 #if defined (DARWIN)
 
 #ifdef QTCAPTURE_RUNLOOP
-	(void)re_fprintf(f, "module\t\t\t" MOD_PRE "qtcapture" MOD_EXT "\n");
+	(void)re_fprintf(f, "module\t\t\t" "qtcapture" MOD_EXT "\n");
 #else
-	(void)re_fprintf(f, "module\t\t\t" MOD_PRE "avcapture" MOD_EXT "\n");
+	(void)re_fprintf(f, "module\t\t\t" "avcapture" MOD_EXT "\n");
 #endif
 
 #elif defined (WIN32)
-	(void)re_fprintf(f, "module\t\t\t" MOD_PRE "dshow" MOD_EXT "\n");
+	(void)re_fprintf(f, "module\t\t\t" "dshow" MOD_EXT "\n");
 
 #else
-	(void)re_fprintf(f, "#module\t\t\t" MOD_PRE "v4l" MOD_EXT "\n");
-	(void)re_fprintf(f, "#module\t\t\t" MOD_PRE "v4l2" MOD_EXT "\n");
-	(void)re_fprintf(f, "#module\t\t\t" MOD_PRE "v4l2_codec" MOD_EXT "\n");
+	(void)re_fprintf(f, "#module\t\t\t" "v4l2" MOD_EXT "\n");
+	(void)re_fprintf(f, "#module\t\t\t" "v4l2_codec" MOD_EXT "\n");
 #endif
-#ifdef USE_AVFORMAT
-	(void)re_fprintf(f, "#module\t\t\t" MOD_PRE "avformat" MOD_EXT "\n");
-#endif
-	(void)re_fprintf(f, "#module\t\t\t" MOD_PRE "x11grab" MOD_EXT "\n");
-	(void)re_fprintf(f, "#module\t\t\t" MOD_PRE "cairo" MOD_EXT "\n");
-	(void)re_fprintf(f, "#module\t\t\t" MOD_PRE "vidbridge" MOD_EXT "\n");
+	(void)re_fprintf(f, "#module\t\t\t" "x11grab" MOD_EXT "\n");
+	(void)re_fprintf(f, "#module\t\t\t" "cairo" MOD_EXT "\n");
+	(void)re_fprintf(f, "#module\t\t\t" "vidbridge" MOD_EXT "\n");
 
 	(void)re_fprintf(f, "\n# Video display modules\n");
-#ifdef DARWIN
-	(void)re_fprintf(f, "#module\t\t\t" MOD_PRE "opengl" MOD_EXT "\n");
-#endif
 #ifdef LINUX
-	(void)re_fprintf(f, "#module\t\t\t" MOD_PRE "directfb" MOD_EXT "\n");
+	(void)re_fprintf(f, "#module\t\t\t" "directfb" MOD_EXT "\n");
 #endif
-	(void)re_fprintf(f, "#module\t\t\t" MOD_PRE "x11" MOD_EXT "\n");
-	(void)re_fprintf(f, "#module\t\t\t" MOD_PRE "sdl2" MOD_EXT "\n");
-	(void)re_fprintf(f, "#module\t\t\t" MOD_PRE "fakevideo" MOD_EXT "\n");
+	(void)re_fprintf(f, "#module\t\t\t" "x11" MOD_EXT "\n");
+	(void)re_fprintf(f, "#module\t\t\t" "sdl" MOD_EXT "\n");
+	(void)re_fprintf(f, "#module\t\t\t" "fakevideo" MOD_EXT "\n");
 
-#endif /* USE_VIDEO */
 
 	(void)re_fprintf(f, "\n# Audio/Video source modules\n");
-	(void)re_fprintf(f, "#module\t\t\t" MOD_PRE "rst" MOD_EXT "\n");
-	(void)re_fprintf(f, "#module\t\t\t" MOD_PRE "gst1" MOD_EXT "\n");
-	(void)re_fprintf(f, "#module\t\t\t" MOD_PRE "gst_video1" MOD_EXT "\n");
+	(void)re_fprintf(f, "#module\t\t\t" "avformat" MOD_EXT "\n");
+	(void)re_fprintf(f, "#module\t\t\t" "rst" MOD_EXT "\n");
+	(void)re_fprintf(f, "#module\t\t\t" "gst" MOD_EXT "\n");
+	(void)re_fprintf(f, "#module\t\t\t" "gst_video" MOD_EXT "\n");
+
+	(void)re_fprintf(f, "\n# Compatibility modules\n");
+	(void)re_fprintf(f, "#module\t\t\t" "ebuacip" MOD_EXT "\n");
 
 	(void)re_fprintf(f, "\n# Media NAT modules\n");
-	(void)re_fprintf(f, "module\t\t\t" MOD_PRE "stun" MOD_EXT "\n");
-	(void)re_fprintf(f, "module\t\t\t" MOD_PRE "turn" MOD_EXT "\n");
-	(void)re_fprintf(f, "module\t\t\t" MOD_PRE "ice" MOD_EXT "\n");
-	(void)re_fprintf(f, "#module\t\t\t" MOD_PRE "natpmp" MOD_EXT "\n");
-	(void)re_fprintf(f, "#module\t\t\t" MOD_PRE "pcp" MOD_EXT "\n");
+	(void)re_fprintf(f, "module\t\t\t" "stun" MOD_EXT "\n");
+	(void)re_fprintf(f, "module\t\t\t" "turn" MOD_EXT "\n");
+	(void)re_fprintf(f, "module\t\t\t" "ice" MOD_EXT "\n");
+	(void)re_fprintf(f, "#module\t\t\t" "natpmp" MOD_EXT "\n");
+	(void)re_fprintf(f, "#module\t\t\t" "pcp" MOD_EXT "\n");
 
 	(void)re_fprintf(f, "\n# Media encryption modules\n");
-	(void)re_fprintf(f, "#module\t\t\t" MOD_PRE "srtp" MOD_EXT "\n");
-	(void)re_fprintf(f, "#module\t\t\t" MOD_PRE "dtls_srtp" MOD_EXT "\n");
-	(void)re_fprintf(f, "#module\t\t\t" MOD_PRE "zrtp" MOD_EXT "\n");
+	(void)re_fprintf(f, "#module\t\t\t" "srtp" MOD_EXT "\n");
+	(void)re_fprintf(f, "#module\t\t\t" "dtls_srtp" MOD_EXT "\n");
+	(void)re_fprintf(f, "#module\t\t\t" "zrtp" MOD_EXT "\n");
 	(void)re_fprintf(f, "\n");
 
 	(void)re_fprintf(f, "\n#------------------------------------"
 			 "------------------------------------------\n");
 	(void)re_fprintf(f, "# Temporary Modules (loaded then unloaded)\n");
 	(void)re_fprintf(f, "\n");
-	(void)re_fprintf(f, "module_tmp\t\t" MOD_PRE "uuid" MOD_EXT "\n");
-	(void)re_fprintf(f, "module_tmp\t\t" MOD_PRE "account" MOD_EXT "\n");
+	(void)re_fprintf(f, "module_tmp\t\t" "uuid" MOD_EXT "\n");
+	(void)re_fprintf(f, "module_tmp\t\t" "account" MOD_EXT "\n");
 	(void)re_fprintf(f, "\n");
 
 	(void)re_fprintf(f, "\n#------------------------------------"
 			 "------------------------------------------\n");
 	(void)re_fprintf(f, "# Application Modules\n");
 	(void)re_fprintf(f, "\n");
-	(void)re_fprintf(f, "module_app\t\t" MOD_PRE "auloop"MOD_EXT"\n");
-	(void)re_fprintf(f, "#module_app\t\t" MOD_PRE "b2bua"MOD_EXT"\n");
-	(void)re_fprintf(f, "module_app\t\t"  MOD_PRE "contact"MOD_EXT"\n");
-	(void)re_fprintf(f, "module_app\t\t"  MOD_PRE "debug_cmd"MOD_EXT"\n");
-#ifdef LINUX
-	(void)re_fprintf(f, "#module_app\t\t"  MOD_PRE "dtmfio"MOD_EXT"\n");
-#endif
-	(void)re_fprintf(f, "#module_app\t\t"  MOD_PRE "echo"MOD_EXT"\n");
-	(void)re_fprintf(f, "#module_app\t\t" MOD_PRE "gtk" MOD_EXT "\n");
-	(void)re_fprintf(f, "module_app\t\t"  MOD_PRE "menu"MOD_EXT"\n");
-	(void)re_fprintf(f, "#module_app\t\t"  MOD_PRE "mwi"MOD_EXT"\n");
-	(void)re_fprintf(f, "#module_app\t\t" MOD_PRE "natbd"MOD_EXT"\n");
-	(void)re_fprintf(f, "#module_app\t\t" MOD_PRE "presence"MOD_EXT"\n");
-	(void)re_fprintf(f, "#module_app\t\t" MOD_PRE "syslog"MOD_EXT"\n");
-	(void)re_fprintf(f, "#module_app\t\t" MOD_PRE "mqtt" MOD_EXT "\n");
-	(void)re_fprintf(f, "#module_app\t\t" MOD_PRE "ctrl_tcp" MOD_EXT "\n");
-#ifdef USE_VIDEO
-	(void)re_fprintf(f, "module_app\t\t" MOD_PRE "vidloop"MOD_EXT"\n");
-#endif
+	(void)re_fprintf(f, "module_app\t\t" "auloop"MOD_EXT"\n");
+	(void)re_fprintf(f, "#module_app\t\t" "b2bua"MOD_EXT"\n");
+	(void)re_fprintf(f, "module_app\t\t"  "contact"MOD_EXT"\n");
+	(void)re_fprintf(f, "module_app\t\t"  "debug_cmd"MOD_EXT"\n");
+	(void)re_fprintf(f, "#module_app\t\t"  "echo"MOD_EXT"\n");
+	(void)re_fprintf(f, "#module_app\t\t" "gtk" MOD_EXT "\n");
+	(void)re_fprintf(f, "module_app\t\t"  "menu"MOD_EXT"\n");
+	(void)re_fprintf(f, "#module_app\t\t"  "mwi"MOD_EXT"\n");
+	(void)re_fprintf(f, "#module_app\t\t" "presence"MOD_EXT"\n");
+	(void)re_fprintf(f, "#module_app\t\t" "syslog"MOD_EXT"\n");
+	(void)re_fprintf(f, "#module_app\t\t" "mqtt" MOD_EXT "\n");
+	(void)re_fprintf(f, "#module_app\t\t" "ctrl_tcp" MOD_EXT "\n");
+	(void)re_fprintf(f, "module_app\t\t" "vidloop"MOD_EXT"\n");
 	(void)re_fprintf(f, "\n");
 
 	(void)re_fprintf(f, "\n#------------------------------------"
@@ -909,14 +891,14 @@ int config_write_template(const char *file, const struct config *cfg)
 	(void)re_fprintf(f, "# Module parameters\n");
 	(void)re_fprintf(f, "\n");
 
-	(void)re_fprintf(f, "\n");
-	(void)re_fprintf(f, "cons_listen\t\t0.0.0.0:5555\n");
+	(void)re_fprintf(f, "\n# UI Modules parameters\n");
+	(void)re_fprintf(f, "cons_listen\t\t0.0.0.0:5555 # cons\n");
 
 	(void)re_fprintf(f, "\n");
-	(void)re_fprintf(f, "http_listen\t\t0.0.0.0:8000\n");
+	(void)re_fprintf(f, "http_listen\t\t0.0.0.0:8000 # httpd - server\n");
 
 	(void)re_fprintf(f, "\n");
-	(void)re_fprintf(f, "ctrl_tcp_listen\t\t0.0.0.0:4444\n");
+	(void)re_fprintf(f, "ctrl_tcp_listen\t\t0.0.0.0:4444 # ctrl_tcp\n");
 
 	(void)re_fprintf(f, "\n");
 	(void)re_fprintf(f, "evdev_device\t\t/dev/input/event0\n");
@@ -926,27 +908,33 @@ int config_write_template(const char *file, const struct config *cfg)
 	(void)re_fprintf(f, "#opus_stereo\t\tyes\n");
 	(void)re_fprintf(f, "#opus_sprop_stereo\tyes\n");
 	(void)re_fprintf(f, "#opus_cbr\t\tno\n");
-	(void)re_fprintf(f, "#opus_inband_fec\tno\n");
+	(void)re_fprintf(f, "#opus_inbandfec\t\tno\n");
 	(void)re_fprintf(f, "#opus_dtx\t\tno\n");
 	(void)re_fprintf(f, "#opus_mirror\t\tno\n");
-	(void)re_fprintf(f, "#opus_complexity\t\t10\n");
-	(void)re_fprintf(f, "#opus_application\t\taudio\t# {voip,audio}\n");
+	(void)re_fprintf(f, "#opus_complexity\t10\n");
+	(void)re_fprintf(f, "#opus_application\taudio\t# {voip,audio}\n");
+	(void)re_fprintf(f, "#opus_samplerate\t48000\n");
+	(void)re_fprintf(f, "#opus_packet_loss\t10\t# 0-100 percent\n");
+
+	(void)re_fprintf(f, "\n# Opus Multistream codec parameters\n");
+	(void)re_fprintf(f,
+			 "#opus_ms_channels\t2\t#total channels (2 or 4)\n");
+	(void)re_fprintf(f,
+			 "#opus_ms_streams\t\t2\t#number of streams\n");
+	(void)re_fprintf(f,
+			"#opus_ms_c_streams\t2\t#number of coupled streams\n");
 
 	(void)re_fprintf(f, "\n");
 	(void)re_fprintf(f, "vumeter_stderr\t\tyes\n");
 
+	(void)re_fprintf(f, "\n");
+	(void)re_fprintf(f, "#jack_connect_ports\tyes\n");
+
 	(void)re_fprintf(f,
 			"\n# Selfview\n"
 			"video_selfview\t\twindow # {window,pip}\n"
 			"#selfview_size\t\t64x64\n");
 
-	(void)re_fprintf(f,
-			"\n# ICE\n"
-			"ice_turn\t\tno\n"
-			"ice_debug\t\tno\n"
-			"ice_nomination\t\tregular\t# {regular,aggressive}\n"
-			"ice_mode\t\tfull\t# {full,lite}\n");
-
 	(void)re_fprintf(f,
 			"\n# ZRTP\n"
 			"#zrtp_hash\t\tno  # Disable SDP zrtp-hash "
@@ -955,11 +943,37 @@ int config_write_template(const char *file, const struct config *cfg)
 	(void)re_fprintf(f,
 			"\n# Menu\n"
 			"#menu_bell\t\tyes\n"
-			"#redial_attempts\t3 # Num or <inf>\n"
+			"#redial_attempts\t0 # Num or <inf>\n"
 			"#redial_delay\t\t5 # Delay in seconds\n"
-			"#ringback_disabled\tyes\n"
+			"#ringback_disabled\tno\n"
 			"#statmode_default\toff\n");
 
+	(void)re_fprintf(f,
+			"\n# avcodec\n"
+			"#avcodec_h264enc\tlibx264\n"
+			"#avcodec_h264dec\th264\n"
+			"#avcodec_h265enc\tlibx265\n"
+			"#avcodec_h265dec\thevc\n"
+			"#avcodec_hwaccel\t%s\n",
+			default_avcodec_hwaccel());
+
+	(void)re_fprintf(f,
+			 "\n# mqtt\n"
+			 "#mqtt_broker_host\t127.0.0.1\n"
+			 "#mqtt_broker_port\t1883\n"
+			 "#mqtt_broker_clientid\tbaresip01\n"
+			 "#mqtt_broker_user\tuser\n"
+			 "#mqtt_broker_password\tpass\n"
+			 "#mqtt_basetopic\t\tbaresip/01\n");
+
+	(void)re_fprintf(f,
+			 "\n# sndfile\n"
+			 "#snd_path\t\t/tmp\n");
+
+	(void)re_fprintf(f,
+			 "\n# EBU ACIP\n"
+			 "#ebuacip_jb_type\tfixed\t# auto,fixed\n");
+
 	if (f)
 		(void)fclose(f);
 
