@@ -26,10 +26,6 @@
  */
 enum {
 	PT_CN       = 13,
-	PT_STAT_MIN = 0,
-	PT_STAT_MAX = 95,
-	PT_DYN_MIN  = 96,
-	PT_DYN_MAX  = 127
 };
 
 
@@ -58,7 +54,7 @@ struct account {
 
 	/* parameters: */
 	enum answermode answermode;  /**< Answermode for incoming calls      */
-	struct le acv[8];            /**< List elements for aucodecl         */
+	struct le acv[16];           /**< List elements for aucodecl         */
 	struct list aucodecl;        /**< List of preferred audio-codecs     */
 	char *auth_user;             /**< Authentication username            */
 	char *auth_pass;             /**< Authentication password            */
@@ -69,17 +65,22 @@ struct account {
 	char *outboundv[2];          /**< Optional SIP outbound proxies      */
 	uint32_t ptime;              /**< Configured packet time in [ms]     */
 	uint32_t regint;             /**< Registration interval in [seconds] */
+	uint32_t rwait;              /**< R. Int. in [%] from proxy expiry   */
 	uint32_t pubint;             /**< Publication interval in [seconds]  */
 	char *regq;                  /**< Registration Q-value               */
 	char *sipnat;                /**< SIP Nat mechanism                  */
 	char *stun_user;             /**< STUN Username                      */
 	char *stun_pass;             /**< STUN Password                      */
-	char *stun_host;             /**< STUN Hostname                      */
-	uint16_t stun_port;          /**< STUN Port number                   */
+	struct stun_uri *stun_host;  /**< STUN Server                        */
 	struct le vcv[4];            /**< List elements for vidcodecl        */
 	struct list vidcodecl;       /**< List of preferred video-codecs     */
 	bool mwi;                    /**< MWI on/off                         */
 	bool refer;                  /**< REFER method on/off                */
+	char *ausrc_mod;
+	char *ausrc_dev;
+	char *auplay_mod;
+	char *auplay_dev;
+	char *extra;                 /**< Extra parameters                   */
 };
 
 
@@ -109,18 +110,6 @@ struct audio;
 
 int  audio_send_digit(struct audio *a, char key);
 void audio_sdp_attr_decode(struct audio *a);
-int  audio_print_rtpstat(struct re_printf *pf, const struct audio *au);
-
-
-/*
- * BFCP
- */
-
-struct bfcp;
-int bfcp_alloc(struct bfcp **bfcpp, struct sdp_session *sdp_sess,
-	       const char *proto, bool offerer,
-	       const struct mnat *mnat, struct mnat_sess *mnat_sess);
-int bfcp_start(struct bfcp *bfcp);
 
 
 /*
@@ -151,9 +140,6 @@ int  call_alloc(struct call **callp, const struct config *cfg,
 		call_event_h *eh, void *arg);
 int  call_accept(struct call *call, struct sipsess_sock *sess_sock,
 		 const struct sip_msg *msg);
-int  call_hangup(struct call *call, uint16_t scode, const char *reason);
-int  call_progress(struct call *call);
-int  call_answer(struct call *call, uint16_t scode);
 int  call_sdp_get(const struct call *call, struct mbuf **descp, bool offer);
 int  call_jbuf_stat(struct re_printf *pf, const struct call *call);
 int  call_info(struct re_printf *pf, const struct call *call);
@@ -188,22 +174,6 @@ int conf_get_float(const struct conf *conf, const char *name, double *val);
 int mctrl_handle_media_control(struct pl *body, bool *pfu);
 
 
-/*
- * Media NAT traversal
- */
-
-struct mnat {
-	struct le le;
-	const char *id;
-	const char *ftag;
-	mnat_sess_h *sessh;
-	mnat_media_h *mediah;
-	mnat_update_h *updateh;
-};
-
-const struct mnat *mnat_find(const struct list *mnatl, const char *id);
-
-
 /*
  * Metric
  */
@@ -211,6 +181,7 @@ const struct mnat *mnat_find(const struct list *mnatl, const char *id);
 struct metric {
 	/* internal stuff: */
 	struct tmr tmr;
+	struct lock *lock;
 	uint64_t ts_start;
 	bool started;
 
@@ -225,7 +196,7 @@ struct metric {
 	uint32_t n_bytes_last;
 };
 
-void     metric_init(struct metric *metric);
+int      metric_init(struct metric *metric);
 void     metric_reset(struct metric *metric);
 void     metric_add_packet(struct metric *metric, size_t packetsize);
 double   metric_avg_bitrate(const struct metric *metric);
@@ -250,7 +221,9 @@ int  reg_register(struct reg *reg, const char *reg_uri,
 void reg_unregister(struct reg *reg);
 bool reg_isok(const struct reg *reg);
 int  reg_debug(struct re_printf *pf, const struct reg *reg);
+int  reg_json_api(struct odict *od, const struct reg *reg);
 int  reg_status(struct re_printf *pf, const struct reg *reg);
+int  reg_af(const struct reg *reg);
 
 
 /*
@@ -283,28 +256,37 @@ int rtpext_encode(struct mbuf *mb, unsigned id, unsigned len,
 int rtpext_decode(struct rtpext *ext, struct mbuf *mb);
 
 
+/*
+ * RTP Stats
+ */
+
+int rtpstat_print(struct re_printf *pf, const struct call *call);
+
+
 /*
  * SDP
  */
 
 int sdp_decode_multipart(const struct pl *ctype_prm, struct mbuf *mb);
-const struct sdp_format *sdp_media_format_cycle(struct sdp_media *m);
 
 
 /*
  * Stream
  */
 
+enum media_type {
+	MEDIA_AUDIO = 0,
+	MEDIA_VIDEO,
+};
+
 struct rtp_header;
 
 enum {STREAM_PRESZ = 4+12}; /* same as RTP_HEADER_SIZE */
 
 typedef void (stream_rtp_h)(const struct rtp_header *hdr,
 			    struct rtpext *extv, size_t extc,
-			    struct mbuf *mb, void *arg);
-typedef void (stream_rtcp_h)(struct rtcp_msg *msg, void *arg);
+			    struct mbuf *mb, unsigned lostc, void *arg);
 
-typedef void (stream_error_h)(struct stream *strm, int err, void *arg);
 
 /** Defines a generic media stream */
 struct stream {
@@ -313,20 +295,24 @@ struct stream {
 #endif
 	struct le le;            /**< Linked list element                   */
 	struct config_avt cfg;   /**< Stream configuration                  */
-	struct call *call;       /**< Ref. to call object                   */
 	struct sdp_media *sdp;   /**< SDP Media line                        */
 	struct rtp_sock *rtp;    /**< RTP Socket                            */
 	struct rtcp_stats rtcp_stats;/**< RTCP statistics                   */
 	struct jbuf *jbuf;       /**< Jitter Buffer for incoming RTP        */
+	const struct mnat *mnat; /**< Media NAT traversal module            */
 	struct mnat_media *mns;  /**< Media NAT traversal state             */
 	const struct menc *menc; /**< Media encryption module               */
 	struct menc_sess *mencs; /**< Media encryption session state        */
 	struct menc_media *mes;  /**< Media Encryption media state          */
 	struct metric metric_tx; /**< Metrics for transmit                  */
 	struct metric metric_rx; /**< Metrics for receiving                 */
+	struct sa raddr_rtp;     /**< Remote RTP address                    */
+	struct sa raddr_rtcp;    /**< Remote RTCP address                   */
+	enum media_type type;    /**< Media type, e.g. audio/video          */
 	char *cname;             /**< RTCP Canonical end-point identifier   */
 	uint32_t ssrc_rx;        /**< Incoming syncronizing source          */
 	uint32_t pseq;           /**< Sequence number for incoming RTP      */
+	bool pseq_set;           /**< True if sequence number is set        */
 	int pt_enc;              /**< Payload type for encoding             */
 	bool rtcp_mux;           /**< RTP/RTCP multiplex supported by peer  */
 	bool jbuf_started;       /**< True if jitter-buffer was started     */
@@ -336,36 +322,42 @@ struct stream {
 	uint32_t rtp_timeout_ms; /**< RTP Timeout value in [ms]             */
 	bool rtp_estab;          /**< True if RTP stream is established     */
 	bool hold;               /**< Stream is on-hold (local)             */
+	bool mnat_connected;     /**< Media NAT is connected                */
+	bool menc_secure;        /**< Media stream is secure                */
 	stream_rtp_h *rtph;      /**< Stream RTP handler                    */
 	stream_rtcp_h *rtcph;    /**< Stream RTCP handler                   */
 	void *arg;               /**< Handler argument                      */
+	stream_mnatconn_h *mnatconnh;/**< Medianat connected handler        */
+	stream_rtpestab_h *rtpestabh;/**< RTP established handler           */
+	stream_rtcp_h *sessrtcph;    /**< Stream RTCP handler               */
 	stream_error_h *errorh;  /**< Stream error handler                  */
-	void *errorh_arg;        /**< Error handler argument                */
+	void *sess_arg;          /**< Session handlers argument             */
 };
 
-int  stream_alloc(struct stream **sp, const struct stream_param *prm,
+int  stream_alloc(struct stream **sp, struct list *streaml,
+		  const struct stream_param *prm,
 		  const struct config_avt *cfg,
-		  struct call *call, struct sdp_session *sdp_sess,
-		  const char *name, int label,
+		  struct sdp_session *sdp_sess,
+		  enum media_type type, int label,
 		  const struct mnat *mnat, struct mnat_sess *mnat_sess,
 		  const struct menc *menc, struct menc_sess *menc_sess,
+		  bool offerer,
 		  stream_rtp_h *rtph, stream_rtcp_h *rtcph, void *arg);
 int  stream_send(struct stream *s, bool ext, bool marker, int pt, uint32_t ts,
 		 struct mbuf *mb);
 void stream_update_encoder(struct stream *s, int pt_enc);
 int  stream_jbuf_stat(struct re_printf *pf, const struct stream *s);
 void stream_hold(struct stream *s, bool hold);
+void stream_set_ldir(struct stream *s, enum sdp_dir dir);
 void stream_set_srate(struct stream *s, uint32_t srate_tx, uint32_t srate_rx);
 void stream_send_fir(struct stream *s, bool pli);
 void stream_reset(struct stream *s);
 void stream_set_bw(struct stream *s, uint32_t bps);
-void stream_set_error_handler(struct stream *strm,
-			      stream_error_h *errorh, void *arg);
-int  stream_debug(struct re_printf *pf, const struct stream *s);
 int  stream_print(struct re_printf *pf, const struct stream *s);
 void stream_enable_rtp_timeout(struct stream *strm, uint32_t timeout_ms);
 int  stream_jbuf_reset(struct stream *strm,
 		       uint32_t frames_min, uint32_t frames_max);
+bool stream_is_ready(const struct stream *strm);
 
 
 /*
@@ -376,8 +368,7 @@ struct ua;
 
 void         ua_printf(const struct ua *ua, const char *fmt, ...);
 
-struct tls  *uag_tls(void);
-const char  *ua_allowed_methods(const struct ua *ua);
+int ua_print_allowed(struct re_printf *pf, const struct ua *ua);
 
 
 /*
@@ -386,24 +377,11 @@ const char  *ua_allowed_methods(const struct ua *ua);
 
 struct video;
 
-typedef void (video_err_h)(int err, const char *str, void *arg);
 
-int  video_alloc(struct video **vp, const struct stream_param *stream_prm,
-		 const struct config *cfg,
-		 struct call *call, struct sdp_session *sdp_sess, int label,
-		 const struct mnat *mnat, struct mnat_sess *mnat_sess,
-		 const struct menc *menc, struct menc_sess *menc_sess,
-		 const char *content, const struct list *vidcodecl,
-		 video_err_h *errh, void *arg);
-int  video_start(struct video *v, const char *peer);
-void video_stop(struct video *v);
 bool video_is_started(const struct video *v);
-int  video_encoder_set(struct video *v, struct vidcodec *vc,
-		       int pt_tx, const char *params);
 int  video_decoder_set(struct video *v, struct vidcodec *vc, int pt_rx,
 		       const char *fmtp);
 void video_update_picture(struct video *v);
-void video_sdp_attr_decode(struct video *v);
 int  video_print(struct re_printf *pf, const struct video *v);
 
 
@@ -428,3 +406,4 @@ int      timestamp_wrap(uint32_t ts_new, uint32_t ts_old);
 void     timestamp_set(struct timestamp_recv *ts, uint32_t rtp_ts);
 uint64_t timestamp_duration(const struct timestamp_recv *ts);
 uint64_t timestamp_calc_extended(uint32_t num_wraps, uint32_t ts);
+double   timestamp_calc_seconds(uint64_t ts, uint32_t clock_rate);
