@@ -9,7 +9,7 @@
 
 
 struct message {
-	struct list lsnrl;
+	struct list lsnrl;          /* struct message_lsnr */
 	struct sip_lsnr *sip_lsnr;
 };
 
@@ -101,7 +101,6 @@ static bool request_handler(const struct sip_msg *msg, void *arg)
 int message_init(struct message **messagep)
 {
 	struct message *message;
-	int err = 0;
 
 	if (!messagep)
 		return EINVAL;
@@ -112,26 +111,22 @@ int message_init(struct message **messagep)
 
 	/* note: cannot create sip listener here, there is not UAs yet */
 
-	if (err)
-		mem_deref(message);
-	else
-		*messagep = message;
+	*messagep = message;
 
-	return err;
+	return 0;
 }
 
 
 /**
  * Listen to incoming SIP MESSAGE messages
  *
- * @param lsnrp   Pointer to allocate listener
  * @param message Messaging subsystem
  * @param recvh   Message receive handler
  * @param arg     Handler argument
  *
  * @return 0 if success, otherwise errorcode
  */
-int message_listen(struct message_lsnr **lsnrp, struct message *message,
+int message_listen(struct message *message,
 		   message_recv_h *recvh, void *arg)
 {
 	struct message_lsnr *lsnr;
@@ -150,20 +145,43 @@ int message_listen(struct message_lsnr **lsnrp, struct message *message,
 	}
 
 	lsnr = mem_zalloc(sizeof(*lsnr), listener_destructor);
+	if (!lsnr)
+		return ENOMEM;
 
 	lsnr->recvh = recvh;
 	lsnr->arg = arg;
 
 	list_append(&message->lsnrl, &lsnr->le, lsnr);
 
-	if (lsnrp)
-		*lsnrp = lsnr;
-
  out:
 	return err;
 }
 
 
+/**
+ * Stop listening to incoming SIP MESSAGE messages
+ *
+ * @param message Messaging subsystem
+ * @param recvh   Message receive handler
+ */
+void message_unlisten(struct message *message, message_recv_h *recvh)
+{
+	struct le *le;
+
+	if (!message)
+		return;
+
+	le = message->lsnrl.head;
+	while (le) {
+		struct message_lsnr *lsnr = le->data;
+		le = le->next;
+
+		if (lsnr->recvh == recvh)
+			mem_deref(lsnr);
+	}
+}
+
+
 /**
  * Send SIP instant MESSAGE to a peer
  *
