@@ -196,8 +196,8 @@ int reg_register(struct reg *reg, const char *reg_uri, const char *params,
 
 	reg->sipreg = mem_deref(reg->sipreg);
 	err = sipreg_register(&reg->sipreg, uag_sip(), reg_uri,
-			      ua_aor(reg->ua),
-			      acc ? acc->dispname : NULL, ua_aor(reg->ua),
+			      account_aor(acc),
+			      acc ? acc->dispname : NULL, account_aor(acc),
 			      regint, ua_local_cuser(reg->ua),
 			      routev[0] ? routev : NULL,
 			      routev[0] ? 1 : 0,
@@ -205,11 +205,14 @@ int reg_register(struct reg *reg, const char *reg_uri, const char *params,
 			      sip_auth_handler, ua_account(reg->ua), true,
 			      register_handler, reg,
 			      params[0] ? &params[1] : NULL,
-			      "Allow: %s\r\n", ua_allowed_methods(reg->ua));
+			      "Allow: %H\r\n", ua_print_allowed, reg->ua);
 	if (err)
 		return err;
 
-	return 0;
+	if (acc->rwait)
+		err = sipreg_set_rwait(reg->sipreg, acc->rwait);
+
+	return err;
 }
 
 
@@ -242,6 +245,14 @@ static const char *print_scode(uint16_t scode)
 }
 
 
+/**
+ * Print the registration debug information
+ *
+ * @param pf  Print function
+ * @param reg Registration object
+ *
+ * @return 0 if success, otherwise errorcode
+ */
 int reg_debug(struct re_printf *pf, const struct reg *reg)
 {
 	int err = 0;
@@ -254,6 +265,35 @@ int reg_debug(struct re_printf *pf, const struct reg *reg)
 	err |= re_hprintf(pf, " scode:  %u (%s)\n",
 			  reg->scode, print_scode(reg->scode));
 	err |= re_hprintf(pf, " srv:    %s\n", reg->srv);
+	err |= re_hprintf(pf, " af:     %s\n", af_name(reg->af));
+
+	return err;
+}
+
+
+/**
+ * Print the registration information in JSON
+ *
+ * @param od  Registration dict
+ * @param reg Registration object
+ *
+ * @return 0 if success, otherwise errorcode
+ */
+int reg_json_api(struct odict *od, const struct reg *reg)
+{
+	int err = 0;
+
+	if (!reg)
+		return 0;
+
+	err |= odict_entry_add(od, "id", ODICT_INT, (int64_t) reg->id);
+	err |= odict_entry_add(od, "state", ODICT_BOOL, reg_isok(reg));
+	err |= odict_entry_add(od, "code", ODICT_INT, (int64_t) reg->scode);
+	if (reg->srv)
+		err |= odict_entry_add(od, "srv", ODICT_STRING, reg->srv);
+
+	err |= odict_entry_add(od, "ipv", ODICT_STRING,
+			af_name(reg->af));
 
 	return err;
 }
@@ -264,5 +304,15 @@ int reg_status(struct re_printf *pf, const struct reg *reg)
 	if (!reg)
 		return 0;
 
-	return re_hprintf(pf, " %s %s", print_scode(reg->scode), reg->srv);
+	return re_hprintf(pf, " %s %s Expires %us", print_scode(reg->scode),
+			reg->srv, sipreg_proxy_expires(reg->sipreg));
+}
+
+
+int reg_af(const struct reg *reg)
+{
+	if (!reg)
+		return 0;
+
+	return reg->af;
 }
