@@ -17,10 +17,74 @@
 
 enum {
 	RTP_RECV_SIZE = 8192,
-	RTP_CHECK_INTERVAL = 1000  /* how often to check for RTP [ms] */
+	RTP_CHECK_INTERVAL = 1000,  /* how often to check for RTP [ms] */
+	PORT_DISCARD = 9,
 };
 
 
+static void print_rtp_stats(const struct stream *s)
+{
+	bool started = s->metric_tx.n_packets>0 || s->metric_rx.n_packets>0;
+
+	if (!started)
+		return;
+
+	info("\n%-9s       Transmit:     Receive:\n"
+	     "packets:        %7u      %7u\n"
+	     "avg. bitrate:   %7.1f      %7.1f  (kbit/s)\n"
+	     "errors:         %7d      %7d\n"
+	     ,
+	     sdp_media_name(s->sdp),
+	     s->metric_tx.n_packets, s->metric_rx.n_packets,
+	     1.0*metric_avg_bitrate(&s->metric_tx)/1000.0,
+	     1.0*metric_avg_bitrate(&s->metric_rx)/1000.0,
+	     s->metric_tx.n_err, s->metric_rx.n_err
+	     );
+
+	if (s->rtcp_stats.tx.sent || s->rtcp_stats.rx.sent) {
+
+		info("pkt.report:     %7u      %7u\n"
+		     "lost:           %7d      %7d\n"
+		     "jitter:         %7.1f      %7.1f  (ms)\n",
+		     s->rtcp_stats.tx.sent, s->rtcp_stats.rx.sent,
+		     s->rtcp_stats.tx.lost, s->rtcp_stats.rx.lost,
+		     1.0*s->rtcp_stats.tx.jit/1000,
+		     1.0*s->rtcp_stats.rx.jit/1000);
+	}
+}
+
+
+static void stream_destructor(void *arg)
+{
+	struct stream *s = arg;
+
+	if (s->cfg.rtp_stats)
+		print_rtp_stats(s);
+
+	metric_reset(&s->metric_tx);
+	metric_reset(&s->metric_rx);
+
+	tmr_cancel(&s->tmr_rtp);
+	list_unlink(&s->le);
+	mem_deref(s->sdp);
+	mem_deref(s->mes);
+	mem_deref(s->mencs);
+	mem_deref(s->mns);
+	mem_deref(s->jbuf);
+	mem_deref(s->rtp);
+	mem_deref(s->cname);
+}
+
+
+static bool mnat_ready(const struct stream *strm)
+{
+	if (strm->mnat && strm->mnat->wait_connected)
+		return strm->mnat_connected;
+	else
+		return true;
+}
+
+
 static void stream_close(struct stream *strm, int err)
 {
 	stream_error_h *errorh = strm->errorh;
@@ -28,9 +92,8 @@ static void stream_close(struct stream *strm, int err)
 	strm->terminated = true;
 	strm->errorh = NULL;
 
-	if (errorh) {
-		errorh(strm, err, strm->errorh_arg);
-	}
+	if (errorh)
+		errorh(strm, err, strm->sess_arg);
 }
 
 
@@ -103,62 +166,19 @@ static inline int lostcalc(struct stream *s, uint16_t seq)
 }
 
 
-static void print_rtp_stats(const struct stream *s)
+static const char *media_name(enum media_type type)
 {
-	bool started = s->metric_tx.n_packets>0 || s->metric_rx.n_packets>0;
+	switch (type) {
 
-	if (!started)
-		return;
-
-	info("\n%-9s       Transmit:     Receive:\n"
-	     "packets:        %7u      %7u\n"
-	     "avg. bitrate:   %7.1f      %7.1f  (kbit/s)\n"
-	     "errors:         %7d      %7d\n"
-	     ,
-	     sdp_media_name(s->sdp),
-	     s->metric_tx.n_packets, s->metric_rx.n_packets,
-	     1.0*metric_avg_bitrate(&s->metric_tx)/1000.0,
-	     1.0*metric_avg_bitrate(&s->metric_rx)/1000.0,
-	     s->metric_tx.n_err, s->metric_rx.n_err
-	     );
-
-	if (s->rtcp_stats.tx.sent || s->rtcp_stats.rx.sent) {
-
-		info("pkt.report:     %7u      %7u\n"
-		     "lost:           %7d      %7d\n"
-		     "jitter:         %7.1f      %7.1f  (ms)\n",
-		     s->rtcp_stats.tx.sent, s->rtcp_stats.rx.sent,
-		     s->rtcp_stats.tx.lost, s->rtcp_stats.rx.lost,
-		     1.0*s->rtcp_stats.tx.jit/1000,
-		     1.0*s->rtcp_stats.rx.jit/1000);
+	case MEDIA_AUDIO: return "audio";
+	case MEDIA_VIDEO: return "video";
+	default:          return "???";
 	}
 }
 
 
-static void stream_destructor(void *arg)
-{
-	struct stream *s = arg;
-
-	if (s->cfg.rtp_stats)
-		print_rtp_stats(s);
-
-	metric_reset(&s->metric_tx);
-	metric_reset(&s->metric_rx);
-
-	tmr_cancel(&s->tmr_rtp);
-	list_unlink(&s->le);
-	mem_deref(s->sdp);
-	mem_deref(s->mes);
-	mem_deref(s->mencs);
-	mem_deref(s->mns);
-	mem_deref(s->jbuf);
-	mem_deref(s->rtp);
-	mem_deref(s->cname);
-}
-
-
 static void handle_rtp(struct stream *s, const struct rtp_header *hdr,
-		       struct mbuf *mb)
+		       struct mbuf *mb, unsigned lostc)
 {
 	struct rtpext extv[8];
 	size_t extc = 0;
@@ -174,7 +194,7 @@ static void handle_rtp(struct stream *s, const struct rtp_header *hdr,
 		int err;
 
 		if (hdr->x.type != RTPEXT_TYPE_MAGIC) {
-			info("stream: unknown ext type ignored (0x%04x)\n",
+			debug("stream: unknown ext type ignored (0x%04x)\n",
 			     hdr->x.type);
 			goto handler;
 		}
@@ -207,7 +227,13 @@ static void handle_rtp(struct stream *s, const struct rtp_header *hdr,
 	}
 
  handler:
-	s->rtph(hdr, extv, extc, mb, s->arg);
+	s->rtph(hdr, extv, extc, mb, lostc, s->arg);
+}
+
+
+static inline bool is_rtcp_packet(unsigned pt)
+{
+	return 64 <= pt && pt <= 95;
 }
 
 
@@ -220,6 +246,12 @@ static void rtp_handler(const struct sa *src, const struct rtp_header *hdr,
 
 	MAGIC_CHECK(s);
 
+	if (is_rtcp_packet(hdr->pt)) {
+		info("stream: drop incoming RTCP packet on RTP port"
+		     " (pt=%u)\n", hdr->pt);
+		return;
+	}
+
 	s->ts_last = tmr_jiffies();
 
 	if (!mbuf_get_left(mb))
@@ -228,6 +260,12 @@ static void rtp_handler(const struct sa *src, const struct rtp_header *hdr,
 	if (!(sdp_media_ldir(s->sdp) & SDP_RECVONLY))
 		return;
 
+#if 0
+	/* The marker bit indicates the beginning of a talkspurt. */
+	if (hdr->m && s->type == MEDIA_AUDIO)
+		flush = true;
+#endif
+
 	metric_add_packet(&s->metric_rx, mbuf_get_left(mb));
 
 	if (!s->rtp_estab) {
@@ -235,33 +273,44 @@ static void rtp_handler(const struct sa *src, const struct rtp_header *hdr,
 		     ", receiving from %J\n",
 		     sdp_media_name(s->sdp), src);
 		s->rtp_estab = true;
+
+		if (s->rtpestabh)
+			s->rtpestabh(s, s->sess_arg);
 	}
 
-	if (hdr->ssrc != s->ssrc_rx) {
-		if (s->ssrc_rx) {
-			flush = true;
-			info("stream: %s: SSRC changed %x -> %x"
-			     " (%u bytes from %J)\n",
-			     sdp_media_name(s->sdp), s->ssrc_rx, hdr->ssrc,
-			     mbuf_get_left(mb), src);
-		}
+	if (!s->pseq_set) {
 		s->ssrc_rx = hdr->ssrc;
+		s->pseq = hdr->seq - 1;
+		s->pseq_set = true;
+	}
+	else if (hdr->ssrc != s->ssrc_rx) {
+
+		info("stream: %s: SSRC changed 0x%x -> 0x%x"
+		     " (%u bytes from %J)\n",
+		     sdp_media_name(s->sdp), s->ssrc_rx, hdr->ssrc,
+		     mbuf_get_left(mb), src);
+
+		s->ssrc_rx = hdr->ssrc;
+		s->pseq = hdr->seq - 1;
+		flush = true;
 	}
 
 	if (s->jbuf) {
 
 		struct rtp_header hdr2;
 		void *mb2 = NULL;
+		int lostc;
 
 		/* Put frame in Jitter Buffer */
-		if (flush)
+		if (flush && s->jbuf_started)
 			jbuf_flush(s->jbuf);
 
 		err = jbuf_put(s->jbuf, hdr, mb);
 		if (err) {
-			info("%s: dropping %u bytes from %J (%m)\n",
+			info("stream: %s: dropping %u bytes from %J"
+			     " [seq=%u, ts=%u] (%m)\n",
 			     sdp_media_name(s->sdp), mb->end,
-			     src, err);
+			     src, hdr->seq, hdr->ts, err);
 			s->metric_rx.n_err++;
 		}
 
@@ -275,18 +324,14 @@ static void rtp_handler(const struct sa *src, const struct rtp_header *hdr,
 
 		s->jbuf_started = true;
 
-		if (lostcalc(s, hdr2.seq) > 0)
-			handle_rtp(s, hdr, NULL);
+		lostc = lostcalc(s, hdr2.seq);
 
-		handle_rtp(s, &hdr2, mb2);
+		handle_rtp(s, &hdr2, mb2, lostc > 0 ? lostc : 0);
 
 		mem_deref(mb2);
 	}
 	else {
-		if (lostcalc(s, hdr->seq) > 0)
-			handle_rtp(s, hdr, NULL);
-
-		handle_rtp(s, hdr, mb);
+		handle_rtp(s, hdr, mb, 0);
 	}
 }
 
@@ -300,21 +345,18 @@ static void rtcp_handler(const struct sa *src, struct rtcp_msg *msg, void *arg)
 
 	s->ts_last = tmr_jiffies();
 
-	if (s->rtcph)
-		s->rtcph(msg, s->arg);
-
 	switch (msg->hdr.pt) {
 
 	case RTCP_SR:
 		(void)rtcp_stats(s->rtp, msg->r.sr.ssrc, &s->rtcp_stats);
-
-		if (s->cfg.rtp_stats)
-			call_set_xrtpstat(s->call);
-
-		ua_event(call_get_ua(s->call), UA_EVENT_CALL_RTCP, s->call,
-			 "%s", sdp_media_name(stream_sdpmedia(s)));
 		break;
 	}
+
+	if (s->rtcph)
+		s->rtcph(s, msg, s->arg);
+
+	if (s->sessrtcph)
+		s->sessrtcph(s, msg, s->sess_arg);
 }
 
 
@@ -353,12 +395,77 @@ static int stream_sock_alloc(struct stream *s, int af)
 }
 
 
-int stream_alloc(struct stream **sp, const struct stream_param *prm,
+/**
+ * Start media encryption
+ *
+ * @param strm   Stream object
+ *
+ * @return 0 if success, otherwise errorcode
+ */
+int stream_start_mediaenc(struct stream *strm)
+{
+	int err;
+
+	if (!strm)
+		return EINVAL;
+
+	if (strm->menc && strm->menc->mediah) {
+
+		info("stream: %s: starting mediaenc '%s' (wait_secure=%d)\n",
+		     media_name(strm->type), strm->menc->id,
+		     strm->menc->wait_secure);
+
+		err = strm->menc->mediah(&strm->mes, strm->mencs, strm->rtp,
+				 rtp_sock(strm->rtp),
+				 strm->rtcp_mux ? NULL : rtcp_sock(strm->rtp),
+				 &strm->raddr_rtp,
+				 strm->rtcp_mux ? NULL : &strm->raddr_rtcp,
+					 strm->sdp, strm);
+		if (err) {
+			warning("stream: start mediaenc error: %m\n", err);
+			return err;
+		}
+	}
+
+	return 0;
+}
+
+
+static void mnat_connected_handler(const struct sa *raddr1,
+				   const struct sa *raddr2, void *arg)
+{
+	struct stream *strm = arg;
+
+	info("stream: mnat '%s' connected: raddr %J %J\n",
+	     strm->mnat->id, raddr1, raddr2);
+
+	strm->raddr_rtp = *raddr1;
+
+	if (strm->rtcp_mux)
+		strm->raddr_rtcp = *raddr1;
+	else if (raddr2)
+		strm->raddr_rtcp = *raddr2;
+
+	strm->mnat_connected = true;
+
+	if (stream_is_ready(strm)) {
+
+		stream_start(strm);
+	}
+
+	if (strm->mnatconnh)
+		strm->mnatconnh(strm, strm->sess_arg);
+}
+
+
+int stream_alloc(struct stream **sp, struct list *streaml,
+		 const struct stream_param *prm,
 		 const struct config_avt *cfg,
-		 struct call *call, struct sdp_session *sdp_sess,
-		 const char *name, int label,
+		 struct sdp_session *sdp_sess,
+		 enum media_type type, int label,
 		 const struct mnat *mnat, struct mnat_sess *mnat_sess,
 		 const struct menc *menc, struct menc_sess *menc_sess,
+		 bool offerer,
 		 stream_rtp_h *rtph, stream_rtcp_h *rtcph, void *arg)
 {
 	struct stream *s;
@@ -374,7 +481,7 @@ int stream_alloc(struct stream **sp, const struct stream_param *prm,
 	MAGIC_INIT(s);
 
 	s->cfg   = *cfg;
-	s->call  = call;
+	s->type  = type;
 	s->rtph  = rtph;
 	s->rtcph = rtcph;
 	s->arg   = arg;
@@ -384,7 +491,8 @@ int stream_alloc(struct stream **sp, const struct stream_param *prm,
 		err = stream_sock_alloc(s, prm->af);
 		if (err) {
 			warning("stream: failed to create socket"
-				" for media '%s' (%m)\n", name, err);
+				" for media '%s' (%m)\n",
+				media_name(type), err);
 			goto out;
 		}
 	}
@@ -402,8 +510,8 @@ int stream_alloc(struct stream **sp, const struct stream_param *prm,
 			goto out;
 	}
 
-	err = sdp_media_add(&s->sdp, sdp_sess, name,
-			    s->rtp ? sa_port(rtp_local(s->rtp)) : 9,
+	err = sdp_media_add(&s->sdp, sdp_sess, media_name(type),
+			    s->rtp ? sa_port(rtp_local(s->rtp)) : PORT_DISCARD,
 			    (menc && menc->sdp_proto) ? menc->sdp_proto :
 			    sdp_proto_rtpavp);
 	if (err)
@@ -415,7 +523,8 @@ int stream_alloc(struct stream **sp, const struct stream_param *prm,
 	}
 
 	/* RFC 5506 */
-	err |= sdp_media_set_lattr(s->sdp, true, "rtcp-rsize", NULL);
+	if (offerer || sdp_media_rattr(s->sdp, "rtcp-rsize"))
+		err |= sdp_media_set_lattr(s->sdp, true, "rtcp-rsize", NULL);
 
 	/* RFC 5576 */
 	err |= sdp_media_set_lattr(s->sdp, true,
@@ -423,17 +532,21 @@ int stream_alloc(struct stream **sp, const struct stream_param *prm,
 				   rtp_sess_ssrc(s->rtp), prm->cname);
 
 	/* RFC 5761 */
-	if (cfg->rtcp_mux)
+	if (cfg->rtcp_mux &&
+	    (offerer || sdp_media_rattr(s->sdp, "rtcp-mux"))) {
+
 		err |= sdp_media_set_lattr(s->sdp, true, "rtcp-mux", NULL);
+	}
 
 	if (err)
 		goto out;
 
 	if (mnat && s->rtp) {
-		err = mnat->mediah(&s->mns, mnat_sess, IPPROTO_UDP,
+		s->mnat = mnat;
+		err = mnat->mediah(&s->mns, mnat_sess,
 				   rtp_sock(s->rtp),
-				   rtcp_sock(s->rtp),
-				   s->sdp);
+				   cfg->rtcp_mux ? NULL : rtcp_sock(s->rtp),
+				   s->sdp, mnat_connected_handler, s);
 		if (err)
 			goto out;
 	}
@@ -441,25 +554,20 @@ int stream_alloc(struct stream **sp, const struct stream_param *prm,
 	if (menc && s->rtp) {
 		s->menc  = menc;
 		s->mencs = mem_ref(menc_sess);
-		err = menc->mediah(&s->mes, menc_sess,
-				   s->rtp,
-				   IPPROTO_UDP,
-				   rtp_sock(s->rtp),
-				   rtcp_sock(s->rtp),
-				   s->sdp);
+
+		err = stream_start_mediaenc(s);
 		if (err)
 			goto out;
 	}
 
+	s->pt_enc = -1;
+
+	err  = metric_init(&s->metric_tx);
+	err |= metric_init(&s->metric_rx);
 	if (err)
 		goto out;
 
-	s->pt_enc = -1;
-
-	metric_init(&s->metric_tx);
-	metric_init(&s->metric_rx);
-
-	list_append(call_streaml(call), &s->le, s);
+	list_append(streaml, &s->le, s);
 
  out:
 	if (err)
@@ -492,20 +600,25 @@ int stream_send(struct stream *s, bool ext, bool marker, int pt, uint32_t ts,
 	if (!s)
 		return EINVAL;
 
-	if (!sa_isset(sdp_media_raddr(s->sdp), SA_ALL))
+	if (!sa_isset(&s->raddr_rtp, SA_ALL))
 		return 0;
 	if (!(sdp_media_rdir(s->sdp) & SDP_SENDONLY))
 		return 0;
 	if (s->hold)
 		return 0;
 
+	if (!stream_is_ready(s)) {
+		warning("stream: send: not ready\n");
+		return EINTR;
+	}
+
 	metric_add_packet(&s->metric_tx, mbuf_get_left(mb));
 
 	if (pt < 0)
 		pt = s->pt_enc;
 
 	if (pt >= 0) {
-		err = rtp_send(s->rtp, sdp_media_raddr(s->sdp), ext,
+		err = rtp_send(s->rtp, &s->raddr_rtp, ext,
 			       marker, pt, ts, mb);
 		if (err)
 			s->metric_tx.n_err++;
@@ -517,8 +630,6 @@ int stream_send(struct stream *s, bool ext, bool marker, int pt, uint32_t ts,
 
 static void stream_remote_set(struct stream *s)
 {
-	struct sa rtcp;
-
 	if (!s)
 		return;
 
@@ -529,14 +640,23 @@ static void stream_remote_set(struct stream *s)
 			info("%s: RTP/RTCP multiplexing enabled\n",
 			     sdp_media_name(s->sdp));
 		s->rtcp_mux = true;
+
+		sdp_media_set_lattr(s->sdp, true, "rtcp-mux", NULL);
 	}
 
 	rtcp_enable_mux(s->rtp, s->rtcp_mux);
 
-	sdp_media_raddr_rtcp(s->sdp, &rtcp);
+	sa_cpy(&s->raddr_rtp, sdp_media_raddr(s->sdp));
 
-	rtcp_start(s->rtp, s->cname,
-		   s->rtcp_mux ? sdp_media_raddr(s->sdp): &rtcp);
+	if (s->rtcp_mux)
+		s->raddr_rtcp = s->raddr_rtp;
+	else
+		sdp_media_raddr_rtcp(s->sdp, &s->raddr_rtcp);
+
+	if (stream_is_ready(s)) {
+
+		stream_start(s);
+	}
 }
 
 
@@ -553,6 +673,8 @@ void stream_update(struct stream *s)
 	if (!s)
 		return;
 
+	info("stream: update '%s'\n", media_name(s->type));
+
 	fmt = sdp_media_rformat(s->sdp, NULL);
 
 	s->pt_enc = fmt ? fmt->pt : -1;
@@ -560,12 +682,9 @@ void stream_update(struct stream *s)
 	if (sdp_media_has_media(s->sdp))
 		stream_remote_set(s);
 
-	if (s->menc && s->menc->mediah) {
-		err = s->menc->mediah(&s->mes, s->mencs, s->rtp,
-				      IPPROTO_UDP,
-				      rtp_sock(s->rtp),
-				      rtcp_sock(s->rtp),
-				      s->sdp);
+	if (s->mencs && mnat_ready(s)) {
+
+		err = stream_start_mediaenc(s);
 		if (err) {
 			warning("stream: mediaenc update: %m\n", err);
 		}
@@ -614,6 +733,24 @@ void stream_hold(struct stream *s, bool hold)
 
 	s->hold = hold;
 	sdp_media_set_ldir(s->sdp, hold ? SDP_SENDONLY : SDP_SENDRECV);
+	stream_reset(s);
+}
+
+
+void stream_set_ldir(struct stream *s, enum sdp_dir dir)
+{
+	if (!s)
+		return;
+
+	if (dir == SDP_INACTIVE) {
+		sdp_media_set_disabled(s->sdp, true);
+	}
+	else {
+		sdp_media_set_disabled(s->sdp, false);
+		sdp_media_set_ldir(s->sdp, dir);
+	}
+
+	stream_reset(s);
 }
 
 
@@ -622,7 +759,10 @@ void stream_set_srate(struct stream *s, uint32_t srate_tx, uint32_t srate_rx)
 	if (!s)
 		return;
 
-	rtcp_set_srate(s->rtp, srate_tx, srate_rx);
+	if (srate_tx)
+		rtcp_set_srate_tx(s->rtp, srate_tx);
+	if (srate_rx)
+		rtcp_set_srate_rx(s->rtp, srate_rx);
 }
 
 
@@ -652,7 +792,8 @@ void stream_reset(struct stream *s)
 	if (!s)
 		return;
 
-	jbuf_flush(s->jbuf);
+	if (s->jbuf && s->jbuf_started)
+		jbuf_flush(s->jbuf);
 }
 
 
@@ -685,20 +826,43 @@ void stream_enable_rtp_timeout(struct stream *strm, uint32_t timeout_ms)
 }
 
 
-void stream_set_error_handler(struct stream *strm,
-			      stream_error_h *errorh, void *arg)
+/**
+ * Set optional session handlers
+ *
+ * @param strm      Stream object
+ * @param mnatconnh Media NAT connected handler
+ * @param rtpestabh Incoming RTP established handler
+ * @param rtcph     Incoming RTCP message handler
+ * @param errorh    Error handler
+ * @param arg       Handler argument
+ */
+void stream_set_session_handlers(struct stream *strm,
+				 stream_mnatconn_h *mnatconnh,
+				 stream_rtpestab_h *rtpestabh,
+				 stream_rtcp_h *rtcph,
+				 stream_error_h *errorh, void *arg)
 {
 	if (!strm)
 		return;
 
+	strm->mnatconnh  = mnatconnh;
+	strm->rtpestabh  = rtpestabh;
+	strm->sessrtcph  = rtcph;
 	strm->errorh     = errorh;
-	strm->errorh_arg = arg;
+	strm->sess_arg   = arg;
 }
 
 
+/**
+ * Print stream debug info
+ *
+ * @param pf Print function
+ * @param s  Stream object
+ *
+ * @return 0 if success, otherwise errorcode
+ */
 int stream_debug(struct re_printf *pf, const struct stream *s)
 {
-	struct sa rrtcp;
 	int err;
 
 	if (!s)
@@ -708,10 +872,17 @@ int stream_debug(struct re_printf *pf, const struct stream *s)
 			  sdp_dir_name(sdp_media_dir(s->sdp)),
 			  s->pt_enc);
 
-	sdp_media_raddr_rtcp(s->sdp, &rrtcp);
 	err |= re_hprintf(pf, " local: %J, remote: %J/%J\n",
 			  sdp_media_laddr(s->sdp),
-			  sdp_media_raddr(s->sdp), &rrtcp);
+			  &s->raddr_rtp, &s->raddr_rtcp);
+
+	err |= re_hprintf(pf, " mnat: %s (connected=%s)\n",
+			  s->mnat ? s->mnat->id : "(none)",
+			  s->mnat_connected ? "yes" : "no");
+
+	err |= re_hprintf(pf, " menc: %s (secure=%s)\n",
+			  s->menc ? s->menc->id : "(none)",
+			  s->menc_secure ? "yes" : "no");
 
 	err |= rtp_debug(pf, s->rtp);
 	err |= jbuf_debug(pf, s->jbuf);
@@ -744,19 +915,6 @@ const struct rtcp_stats *stream_rtcp_stats(const struct stream *strm)
 }
 
 
-/**
- * Get the call object from the stream
- *
- * @param strm Stream object
- *
- * @return Call object
- */
-struct call *stream_call(const struct stream *strm)
-{
-	return strm ? strm->call : NULL;
-}
-
-
 /**
  * Get the number of transmitted RTP packets
  *
@@ -848,3 +1006,104 @@ int stream_jbuf_reset(struct stream *strm,
 
 	return 0;
 }
+
+
+bool stream_is_ready(const struct stream *strm)
+{
+	if (!strm)
+		return false;
+
+	/* Media NAT */
+	if (strm->mnat) {
+		if (!mnat_ready(strm))
+			return false;
+	}
+
+	/* Media Enc */
+	if (strm->menc && strm->menc->wait_secure) {
+
+		if (!strm->menc_secure)
+			return false;
+	}
+
+	if (!sa_isset(&strm->raddr_rtp, SA_ALL))
+		return false;
+
+	return !strm->terminated;
+}
+
+
+/**
+ * Set the secure flag on the stream object
+ *
+ * @param strm   Stream object
+ * @param secure True for secure, false for insecure
+ */
+void stream_set_secure(struct stream *strm, bool secure)
+{
+	if (!strm)
+		return;
+
+	strm->menc_secure = secure;
+}
+
+
+/**
+ * Get the secure flag on the stream object
+ *
+ * @param strm   Stream object
+ *
+ * @return True for secure, false for insecure
+ */
+bool stream_is_secure(const struct stream *strm)
+{
+	return strm ? strm->menc_secure : false;
+}
+
+
+/**
+ * Start the media stream
+ *
+ * @param strm   Stream object
+ *
+ * @return 0 if success, otherwise errorcode
+ */
+int stream_start(const struct stream *strm)
+{
+	int err;
+
+	if (!strm)
+		return EINVAL;
+
+	debug("stream: %s: starting RTCP with remote %J\n",
+	      media_name(strm->type), &strm->raddr_rtcp);
+
+	rtcp_start(strm->rtp, strm->cname, &strm->raddr_rtcp);
+
+	if (!strm->mnat) {
+		/* Send a dummy RTCP packet to open NAT pinhole */
+		err = rtcp_send_app(strm->rtp, "PING", (void *)"PONG", 4);
+		if (err) {
+			warning("stream: rtcp_send_app failed (%m)\n", err);
+			return err;
+		}
+	}
+
+	return 0;
+}
+
+
+/**
+ * Get the name of the stream type (e.g. audio or video)
+ *
+ * @param strm Stream object
+ *
+ * @return Name of stream type
+ */
+const char *stream_name(const struct stream *strm)
+{
+	if (!strm)
+		return NULL;
+
+	return media_name(strm->type);
+}
