@@ -26,7 +26,6 @@ struct ua {
 	size_t    extensionc;        /**< Number of SIP extensions           */
 	char *cuser;                 /**< SIP Contact username               */
 	char *pub_gruu;              /**< SIP Public GRUU                    */
-	int af;                      /**< Preferred Address Family           */
 	int af_media;                /**< Preferred Address Family for media */
 	enum presence_status my_status; /**< Presence Status                 */
 	bool catchall;               /**< Catch all inbound requests         */
@@ -57,7 +56,7 @@ static struct {
 	bool use_udp;                  /**< Use UDP transport               */
 	bool use_tcp;                  /**< Use TCP transport               */
 	bool use_tls;                  /**< Use TLS transport               */
-	bool prefer_ipv6;              /**< Force IPv6 transport            */
+	bool delayed_close;            /**< Module will close SIP stack     */
 	sip_msg_h *subh;               /**< Subscribe handler               */
 	ua_exit_h *exith;              /**< UA Exit handler                 */
 	void *arg;                     /**< UA Exit handler argument        */
@@ -88,6 +87,30 @@ static struct {
 };
 
 
+static void ua_destructor(void *arg)
+{
+	struct ua *ua = arg;
+
+	list_unlink(&ua->le);
+
+	if (!list_isempty(&ua->regl))
+		ua_event(ua, UA_EVENT_UNREGISTERING, NULL, NULL);
+
+	list_flush(&ua->calls);
+	list_flush(&ua->regl);
+	mem_deref(ua->cuser);
+	mem_deref(ua->pub_gruu);
+	mem_deref(ua->acc);
+
+	if (uag.delayed_close && list_isempty(&uag.ual)) {
+		sip_close(uag.sip, false);
+	}
+
+	list_flush(&ua->custom_hdrs);
+	list_flush(&ua->hdr_filter);
+}
+
+
 /* This function is called when all SIP transactions are done */
 static void exit_handler(void *arg)
 {
@@ -115,6 +138,15 @@ void ua_printf(const struct ua *ua, const char *fmt, ...)
 }
 
 
+/**
+ * Send a User-Agent event to all UA event handlers
+ *
+ * @param ua   User-Agent object (optional)
+ * @param ev   User-agent event
+ * @param call Call object (optional)
+ * @param fmt  Formatted arguments
+ * @param ...  Variable arguments
+ */
 void ua_event(struct ua *ua, enum ua_event ev, struct call *call,
 	      const char *fmt, ...)
 {
@@ -268,6 +300,38 @@ bool ua_isregistered(const struct ua *ua)
 }
 
 
+/**
+ * Destroy the user-agent, terminate all active calls and
+ * send the SHUTDOWN event.
+ *
+ * @param ua User-Agent object
+ *
+ * @return Number of remaining references
+ */
+unsigned ua_destroy(struct ua *ua)
+{
+	unsigned nrefs;
+
+	if (!ua)
+		return 0;
+
+	list_unlink(&ua->le);
+
+	/* send the shutdown event */
+	ua_event(ua, UA_EVENT_SHUTDOWN, NULL, NULL);
+
+	/* terminate all calls now */
+	list_flush(&ua->calls);
+
+	/* number of remaining references (excluding this one) */
+	nrefs = mem_nrefs(ua) - 1;
+
+	mem_deref(ua);
+
+	return nrefs;
+}
+
+
 static struct call *ua_find_call_onhold(const struct ua *ua)
 {
 	struct le *le;
@@ -331,11 +395,10 @@ static void call_event_handler(struct call *call, enum call_event ev,
 			break;
 
 		case ANSWERMODE_AUTO:
-			(void)call_answer(call, 200);
+			(void)call_answer(call, 200, VIDMODE_ON);
 			break;
 
 		case ANSWERMODE_MANUAL:
-		default:
 			ua_event(ua, UA_EVENT_CALL_INCOMING, call, peeruri);
 			break;
 		}
@@ -368,6 +431,7 @@ static void call_event_handler(struct call *call, enum call_event ev,
 
 	case CALL_EVENT_TRANSFER_FAILED:
 		ua_event(ua, UA_EVENT_CALL_TRANSFER_FAILED, call, str);
+		mem_deref(call);
 		break;
 
 	case CALL_EVENT_MENC:
@@ -384,7 +448,7 @@ static void call_dtmf_handler(struct call *call, char key, void *arg)
 
 	MAGIC_CHECK(ua);
 
-	if (key != '\0') {
+	if (key != KEYCODE_REL) {
 
 		key_str[0] = key;
 		key_str[1] = '\0';
@@ -397,6 +461,50 @@ static void call_dtmf_handler(struct call *call, char key, void *arg)
 }
 
 
+static int best_effort_af(struct ua *ua, const struct network *net)
+{
+	struct le *le;
+	const int afv[2] = { AF_INET, AF_INET6 };
+	size_t i;
+
+	for (le = ua->regl.head, i=0; le; le = le->next, i++) {
+		const struct reg *reg = le->data;
+		if (reg_isok(reg))
+			return reg_af(reg);
+	}
+
+	for (i=0; i<ARRAY_SIZE(afv); i++) {
+		int af = afv[i];
+
+		if (net_af_enabled(net, af) &&
+		    sa_isset(net_laddr_af(net, af), SA_ADDR))
+			return af;
+	}
+
+	return AF_UNSPEC;
+}
+
+
+static int sdp_af_hint(struct mbuf *mb)
+{
+	struct pl af;
+	int err;
+
+	err = re_regex((char *)mbuf_buf(mb), mbuf_get_left(mb),
+		       "IN IP[46]+", &af);
+	if (err)
+		return AF_UNSPEC;
+
+	switch (af.p[0]) {
+
+	case '4': return AF_INET;
+	case '6': return AF_INET6;
+	}
+
+	return AF_UNSPEC;
+}
+
+
 /**
  * Create a new call object
  *
@@ -417,20 +525,28 @@ int ua_call_alloc(struct call **callp, struct ua *ua,
 {
 	const struct network *net = baresip_network();
 	struct call_prm cprm;
-	int af = AF_UNSPEC;
+	int af;
+	int af_sdp;
 	int err;
 
 	if (!callp || !ua)
 		return EINVAL;
 
-	/* 1. if AF_MEDIA is set, we prefer it
-	 * 2. otherwise fall back to SIP AF
-	 */
-	if (ua->af_media) {
+	if (msg && (af_sdp = sdp_af_hint(msg->mb))) {
+		info("ua: using AF from sdp offer: af=%s\n",
+		     net_af2name(af_sdp));
+		af = af_sdp;
+	}
+	else if (ua->af_media &&
+		   sa_isset(net_laddr_af(net, ua->af_media), SA_ADDR)) {
+		info("ua: using ua's preferred AF: af=%s\n",
+		     net_af2name(ua->af_media));
 		af = ua->af_media;
 	}
-	else if (ua->af) {
-		af = ua->af;
+	else {
+		af = best_effort_af(ua, net);
+		info("ua: using best effort AF: af=%s\n",
+		     net_af2name(af));
 	}
 
 	memset(&cprm, 0, sizeof(cprm));
@@ -494,14 +610,14 @@ static void handle_options(struct ua *ua, const struct sip_msg *msg)
 
 	err = sip_treplyf(NULL, NULL, uag.sip,
 			  msg, true, 200, "OK",
-			  "Allow: %s\r\n"
+			  "Allow: %H\r\n"
 			  "%H"
 			  "%H"
 			  "%s"
 			  "Content-Length: %zu\r\n"
 			  "\r\n"
 			  "%b",
-			  ua_allowed_methods(ua),
+			  ua_print_allowed, ua,
 			  ua_print_supported, ua,
 			  sip_contact_print, &contact,
 			  desc ? "Content-Type: application/sdp\r\n" : "",
@@ -518,30 +634,6 @@ static void handle_options(struct ua *ua, const struct sip_msg *msg)
 }
 
 
-static void ua_destructor(void *arg)
-{
-	struct ua *ua = arg;
-
-	list_unlink(&ua->le);
-
-	if (!list_isempty(&ua->regl))
-		ua_event(ua, UA_EVENT_UNREGISTERING, NULL, NULL);
-
-	list_flush(&ua->calls);
-	list_flush(&ua->regl);
-	mem_deref(ua->cuser);
-	mem_deref(ua->pub_gruu);
-	mem_deref(ua->acc);
-
-	if (list_isempty(&uag.ual)) {
-		sip_close(uag.sip, false);
-	}
-
-	list_flush(&ua->custom_hdrs);
-	list_flush(&ua->hdr_filter);
-}
-
-
 static bool request_handler(const struct sip_msg *msg, void *arg)
 {
 	struct ua *ua;
@@ -585,7 +677,7 @@ static int create_register_clients(struct ua *ua)
 
 	/* Register clients */
 	if (uag.cfg && str_isset(uag.cfg->uuid))
-	        add_extension(ua, "gruu");
+		add_extension(ua, "gruu");
 
 	if (0 == str_casecmp(ua->acc->sipnat, "outbound")) {
 
@@ -644,11 +736,7 @@ int ua_alloc(struct ua **uap, const char *aor)
 
 	list_init(&ua->calls);
 
-#if HAVE_INET6
-	ua->af   = uag.prefer_ipv6 ? AF_INET6 : AF_INET;
-#else
-	ua->af   = AF_INET;
-#endif
+	ua->af_media = AF_UNSPEC;
 
 	/* Decode SIP address */
 	if (uag.eprm) {
@@ -669,11 +757,11 @@ int ua_alloc(struct ua **uap, const char *aor)
 		goto out;
 
 	if (ua->acc->sipnat) {
-		ua_printf(ua, "Using sipnat: `%s'\n", ua->acc->sipnat);
+		ua_printf(ua, "Using sipnat: '%s'\n", ua->acc->sipnat);
 	}
 
 	if (ua->acc->mnat) {
-		ua_printf(ua, "Using medianat `%s'\n",
+		ua_printf(ua, "Using medianat '%s'\n",
 			  ua->acc->mnat->id);
 
 		if (0 == str_casecmp(ua->acc->mnat->id, "ice"))
@@ -681,7 +769,7 @@ int ua_alloc(struct ua **uap, const char *aor)
 	}
 
 	if (ua->acc->menc) {
-		ua_printf(ua, "Using media encryption `%s'\n",
+		ua_printf(ua, "Using media encryption '%s'\n",
 			  ua->acc->menc->id);
 	}
 
@@ -871,10 +959,10 @@ void ua_hangup(struct ua *ua, struct call *call,
 			return;
 	}
 
-	(void)call_hangup(call, scode, reason);
+	call_hangup(call, scode, reason);
 
 	ua_event(ua, UA_EVENT_CALL_CLOSED, call,
-	         reason ? reason : "Connection reset by user");
+		 reason ? reason : "Connection reset by user");
 
 	mem_deref(call);
 
@@ -885,12 +973,13 @@ void ua_hangup(struct ua *ua, struct call *call,
 /**
  * Answer an incoming call
  *
- * @param ua   User-Agent
- * @param call Call to answer, or NULL for current call
+ * @param ua    User-Agent
+ * @param call  Call to answer, or NULL for current call
+ * @param vmode Wanted video mode
  *
  * @return 0 if success, otherwise errorcode
  */
-int ua_answer(struct ua *ua, struct call *call)
+int ua_answer(struct ua *ua, struct call *call, enum vidmode vmode)
 {
 	if (!ua)
 		return EINVAL;
@@ -901,42 +990,20 @@ int ua_answer(struct ua *ua, struct call *call)
 			return ENOENT;
 	}
 
-	return call_answer(call, 200);
-}
-
-
-/**
- * Answer an incoming call with early media
- *
- * @param ua   User-Agent
- * @param call Call to answer, or NULL for current call
- *
- * @return 0 if success, otherwise errorcode
- */
-int ua_progress(struct ua *ua, struct call *call)
-{
-	if (!ua)
-		return EINVAL;
-
-	if (!call) {
-		call = ua_call(ua);
-		if (!call)
-			return ENOENT;
-	}
-
-	return call_progress(call);
+	return call_answer(call, 200, vmode);
 }
 
 
 /**
  * Put the current call on hold and answer the incoming call
  *
- * @param ua   User-Agent
- * @param call Call to answer, or NULL for current call
+ * @param ua    User-Agent
+ * @param call  Call to answer, or NULL for current call
+ * @param vmode Wanted video mode for the incoming call
  *
  * @return 0 if success, otherwise errorcode
  */
-int ua_hold_answer(struct ua *ua, struct call *call)
+int ua_hold_answer(struct ua *ua, struct call *call, enum vidmode vmode)
 {
 	struct call *pcall;
 	int err;
@@ -961,7 +1028,7 @@ int ua_hold_answer(struct ua *ua, struct call *call)
 			return err;
 	}
 
-	return ua_answer(ua, call);
+	return ua_answer(ua, call, vmode);
 }
 
 
@@ -1161,7 +1228,7 @@ int ua_debug(struct re_printf *pf, const struct ua *ua)
 	err |= re_hprintf(pf, " nrefs:     %u\n", mem_nrefs(ua));
 	err |= re_hprintf(pf, " cuser:     %s\n", ua->cuser);
 	err |= re_hprintf(pf, " pub-gruu:  %s\n", ua->pub_gruu);
-	err |= re_hprintf(pf, " af:        %s\n", net_af2name(ua->af));
+	err |= re_hprintf(pf, " af_media:  %s\n", net_af2name(ua->af_media));
 	err |= re_hprintf(pf, " %H", ua_print_supported, ua);
 
 	err |= account_debug(pf, ua->acc);
@@ -1173,6 +1240,66 @@ int ua_debug(struct re_printf *pf, const struct ua *ua)
 }
 
 
+/**
+ * Print the user-agent information in JSON
+ *
+ * @param od  User-Agent dict
+ * @param ua  User-Agent object
+ *
+ * @return 0 if success, otherwise errorcode
+ */
+int ua_state_json_api(struct odict *od, const struct ua *ua)
+{
+	struct odict *reg = NULL;
+	struct odict *cfg = NULL;
+	struct le *le;
+	size_t i = 0;
+	int err = 0;
+
+	if (!ua)
+		return 0;
+
+	err |= odict_alloc(&reg, 8);
+	err |= odict_alloc(&cfg, 8);
+
+	/* user-agent info */
+	err |= odict_entry_add(od, "cuser", ODICT_STRING, ua->cuser);
+	err |= odict_entry_add(od, "selected_ua", ODICT_BOOL,
+			ua == uag_current());
+
+	/* account info */
+	err |= account_json_api(od, cfg, ua->acc);
+	if (err)
+		warning("ua: failed to encode json account (%m)\n", err);
+
+	/* registration info */
+	for (le = list_head(&ua->regl); le; le = le->next) {
+		struct reg *regm = le->data;
+		err |= reg_json_api(reg, regm);
+		++i;
+	}
+	if (i > 1)
+		warning("ua: multiple registrations for one account");
+
+	err |= odict_entry_add(reg, "interval", ODICT_INT,
+			(int64_t) ua->acc->regint);
+	err |= odict_entry_add(reg, "q_value", ODICT_DOUBLE, ua->acc->regq);
+
+	if (err)
+		warning("ua: failed to encode json registration (%m)\n", err);
+
+	/* package */
+	err |= odict_entry_add(od, "settings", ODICT_OBJECT, cfg);
+	err |= odict_entry_add(od, "registration", ODICT_OBJECT, reg);
+	if (err)
+		warning("ua: failed to encode json package (%m)\n", err);
+
+	mem_deref(cfg);
+	mem_deref(reg);
+	return err;
+}
+
+
 /* One instance */
 
 
@@ -1267,10 +1394,8 @@ static int ua_add_transp(struct network *net)
 {
 	int err = 0;
 
-	if (!uag.prefer_ipv6) {
-		if (sa_isset(net_laddr_af(net, AF_INET), SA_ADDR))
-			err |= add_transp_af(net_laddr_af(net, AF_INET));
-	}
+	if (sa_isset(net_laddr_af(net, AF_INET), SA_ADDR))
+		err |= add_transp_af(net_laddr_af(net, AF_INET));
 
 #if HAVE_INET6
 	if (sa_isset(net_laddr_af(net, AF_INET6), SA_ADDR))
@@ -1305,7 +1430,9 @@ static bool require_handler(const struct sip_hdr *hdr,
 static void sipsess_conn_handler(const struct sip_msg *msg, void *arg)
 {
 	struct config *config = conf_config();
+	const struct network *net = baresip_network();
 	const struct sip_hdr *hdr;
+	int af_sdp;
 	struct ua *ua;
 	struct call *call = NULL;
 	char to_uri[256];
@@ -1351,6 +1478,26 @@ static void sipsess_conn_handler(const struct sip_msg *msg, void *arg)
 		return;
 	}
 
+	/* Check if offered media AF is supported and available */
+	af_sdp = sdp_af_hint(msg->mb);
+	if (af_sdp) {
+		if (!net_af_enabled(net, af_sdp)) {
+			warning("ua: SDP offer AF not supported (%s)\n",
+				net_af2name(af_sdp));
+			af_sdp = 0;
+		}
+		else if (!sa_isset(net_laddr_af(net, af_sdp), SA_ADDR)) {
+			warning("ua: SDP offer AF not available (%s)\n",
+				net_af2name(af_sdp));
+			af_sdp = 0;
+		}
+		if (!af_sdp) {
+			(void)sip_treply(NULL, uag_sip(), msg, 488,
+					 "Not Acceptable Here");
+			return;
+		}
+	}
+
 	(void)pl_strcpy(&msg->to.auri, to_uri, sizeof(to_uri));
 
 	err = ua_call_alloc(&call, ua, VIDMODE_ON, msg, NULL, to_uri, true);
@@ -1367,16 +1514,18 @@ static void sipsess_conn_handler(const struct sip_msg *msg, void *arg)
 
 		le = list_head(&ua->hdr_filter);
 		while (le) {
-		    const struct sip_hdr *tmp_hdr;
-		    const struct ua_xhdr_filter *filter = le->data;
+			const struct sip_hdr *tmp_hdr;
+			const struct ua_xhdr_filter *filter = le->data;
 
-		    le = le->next;
-		    tmp_hdr = sip_msg_xhdr(msg, filter->hdr_name);
+			le = le->next;
+			tmp_hdr = sip_msg_xhdr(msg, filter->hdr_name);
 
-		    if (tmp_hdr) {
-		        char name[256];
-		        pl_strcpy(&tmp_hdr->name, name, sizeof(name));
-		        if (custom_hdrs_add(&hdrs, name, "%r", &tmp_hdr->val))
+			if (tmp_hdr) {
+				char name[256];
+
+				pl_strcpy(&tmp_hdr->name, name, sizeof(name));
+				if (custom_hdrs_add(&hdrs, name,
+						    "%r", &tmp_hdr->val))
 					goto error;
 			}
 		}
@@ -1434,17 +1583,6 @@ int ua_add_xhdr_filter(struct ua *ua, const char *hdr_name)
 }
 
 
-static void net_change_handler(void *arg)
-{
-	(void)arg;
-
-	info("IP-address changed: %j\n",
-	     net_laddr_af(baresip_network(), AF_INET));
-
-	(void)uag_reset_transp(true, true);
-}
-
-
 static bool sub_handler(const struct sip_msg *msg, void *arg)
 {
 	struct ua *ua;
@@ -1465,6 +1603,25 @@ static bool sub_handler(const struct sip_msg *msg, void *arg)
 }
 
 
+#ifdef LIBRE_HAVE_SIPTRACE
+static void sip_trace_handler(bool tx, enum sip_transp tp,
+			      const struct sa *src, const struct sa *dst,
+			      const uint8_t *pkt, size_t len, void *arg)
+{
+	(void)tx;
+	(void)arg;
+
+	re_printf("\x1b[36;1m"
+		  "#\n"
+		  "%s %J -> %J\n"
+		  "%b"
+		  "\x1b[;m\n"
+		  ,
+		  sip_transp_name(tp), src, dst, pkt, len);
+}
+#endif
+
+
 /**
  * Initialise the User-Agents
  *
@@ -1472,12 +1629,10 @@ static bool sub_handler(const struct sip_msg *msg, void *arg)
  * @param udp         Enable UDP transport
  * @param tcp         Enable TCP transport
  * @param tls         Enable TLS transport
- * @param prefer_ipv6 Prefer IPv6 flag
  *
  * @return 0 if success, otherwise errorcode
  */
-int ua_init(const char *software, bool udp, bool tcp, bool tls,
-	    bool prefer_ipv6)
+int ua_init(const char *software, bool udp, bool tcp, bool tls)
 {
 	struct config *cfg = conf_config();
 	struct network *net = baresip_network();
@@ -1490,12 +1645,11 @@ int ua_init(const char *software, bool udp, bool tcp, bool tls,
 	}
 
 	uag.cfg = &cfg->sip;
-	bsize = cfg->sip.trans_bsize;
+	bsize = 16;
 
 	uag.use_udp = udp;
 	uag.use_tcp = tcp;
 	uag.use_tls = tls;
-	uag.prefer_ipv6 = prefer_ipv6;
 
 	list_init(&uag.ual);
 
@@ -1524,8 +1678,6 @@ int ua_init(const char *software, bool udp, bool tcp, bool tls,
 	if (err)
 		goto out;
 
-	net_change(net, 60, net_change_handler, NULL);
-
  out:
 	if (err) {
 		warning("ua: init failed (%m)\n", err);
@@ -1574,19 +1726,14 @@ void ua_stop_all(bool forced)
 		struct ua *ua = le->data;
 		le = le->next;
 
-		ua_event(ua, UA_EVENT_SHUTDOWN, NULL, NULL);
-
-		if (mem_nrefs(ua) > 1) {
+		if (ua_destroy(ua) != 0) {
 			++ext_ref;
 		}
-
-		list_unlink(&ua->le);
-		list_flush(&ua->calls);
-		mem_deref(ua);
 	}
 
 	if (ext_ref) {
 		info("ua: in use (%u) by app module\n", ext_ref);
+		uag.delayed_close = true;
 		return;
 	}
 
@@ -1611,6 +1758,22 @@ void uag_set_exit_handler(ua_exit_h *exith, void *arg)
 }
 
 
+/**
+ * Enable SIP message tracing
+ *
+ * @param enable True to enable, false to disable
+ */
+void uag_enable_sip_trace(bool enable)
+{
+#ifdef LIBRE_HAVE_SIPTRACE
+	sip_set_trace_handler(uag.sip, enable ? sip_trace_handler : NULL);
+#else
+	(void)enable;
+	warning("no sip trace in libre\n");
+#endif
+}
+
+
 /**
  * Reset the SIP transports for all User-Agents
  *
@@ -1660,21 +1823,6 @@ int uag_reset_transp(bool reg, bool reinvite)
 }
 
 
-/**
- * Print the SIP Status for all User-Agents
- *
- * @param pf     Print handler for debug output
- * @param unused Unused parameter
- *
- * @return 0 if success, otherwise errorcode
- */
-int ua_print_sip_status(struct re_printf *pf, void *unused)
-{
-	(void)unused;
-	return sip_debug(pf, uag.sip);
-}
-
-
 /**
  * Print all calls for a given User-Agent
  *
@@ -1696,7 +1844,7 @@ int ua_print_calls(struct re_printf *pf, const struct ua *ua)
 
 	n = list_count(&ua->calls);
 
-	err |= re_hprintf(pf, "\n--- List of active calls (%u): ---\n",
+	err |= re_hprintf(pf, "\n--- Active calls (%u) ---\n",
 			  n);
 
 	for (linenum=CALL_LINENUM_MIN; linenum<CALL_LINENUM_MAX; linenum++) {
@@ -1707,8 +1855,8 @@ int ua_print_calls(struct re_printf *pf, const struct ua *ua)
 		if (call) {
 			++count;
 
-			err |= re_hprintf(pf, "  %c %H\n",
-					  call == ua_call(ua) ? '>' : ' ',
+			err |= re_hprintf(pf, "%s %H\n",
+					  call == ua_call(ua) ? ">" : " ",
 					  call_info, call);
 		}
 
@@ -1755,16 +1903,6 @@ struct sipevent_sock *uag_sipevent_sock(void)
 }
 
 
-struct tls *uag_tls(void)
-{
-#ifdef USE_TLS
-	return uag.tls;
-#else
-	return NULL;
-#endif
-}
-
-
 /**
  * Find the correct UA from the contact user
  *
@@ -1938,18 +2076,28 @@ struct list *uag_list(void)
 
 
 /**
- * Return list of methods supported by the UA
+ * Print list of methods allowed by the UA
  *
- * @param ua User-Agent
+ * @param pf  Print function
+ * @param ua  User-Agent
  *
- * @return String of supported methods
+ * @return 0 if success, otherwise errorcode
  */
-const char *ua_allowed_methods(const struct ua *ua)
+int ua_print_allowed(struct re_printf *pf, const struct ua *ua)
 {
-	return ua->acc->refer ? "INVITE,ACK,BYE,CANCEL,OPTIONS,"
-		"NOTIFY,SUBSCRIBE,INFO,MESSAGE,REFER" :
-		"INVITE,ACK,BYE,CANCEL,OPTIONS,"
-		"NOTIFY,SUBSCRIBE,INFO,MESSAGE";
+	int err;
+
+	if (!ua || !ua->acc)
+		return 0;
+
+	err = re_hprintf(pf,
+			 "INVITE,ACK,BYE,CANCEL,OPTIONS,"
+			 "NOTIFY,SUBSCRIBE,INFO,MESSAGE");
+
+	if (ua->acc->refer)
+		err |= re_hprintf(pf, ",REFER");
+
+	return err;
 }
 
 
@@ -1966,6 +2114,9 @@ int ua_print_supported(struct re_printf *pf, const struct ua *ua)
 	size_t i;
 	int err;
 
+	if (!ua)
+		return 0;
+
 	err = re_hprintf(pf, "Supported:");
 
 	for (i=0; i<ua->extensionc; i++) {
