@@ -4,6 +4,7 @@
  * Copyright (C) 2010 - 2015 Creytiv.com
  */
 #include <string.h>
+#include <stdlib.h>
 #include <re.h>
 #include <rem.h>
 #include <baresip.h>
@@ -42,6 +43,8 @@ struct agent {
 	unsigned n_dtmf_recv;
 	unsigned n_transfer;
 	unsigned n_mediaenc;
+	unsigned n_rtpestab;
+	unsigned n_rtcp;
 };
 
 struct fixture {
@@ -56,6 +59,8 @@ struct fixture {
 	int err;
 	unsigned exp_estab;
 	unsigned exp_closed;
+	bool stop_on_rtp;
+	bool stop_on_rtcp;
 };
 
 
@@ -66,13 +71,16 @@ struct fixture {
 	f->b.fix = f;							\
 	f->c.fix = f;							\
 									\
-	err = ua_init("test", true, true, false, false);		\
+	err = ua_init("test", true, true, false);			\
 	TEST_ERR(err);							\
 									\
 	f->magic = MAGIC;						\
+	f->estab_action = ACTION_RECANCEL;				\
 	f->exp_estab = 1;						\
 	f->exp_closed = 1;						\
-	mock_aucodec_register();					\
+	/* NOTE: See Makefile TEST_MODULES */				\
+	err = module_load(".", "g711");					\
+	TEST_ERR(err);							\
 									\
 	err = ua_alloc(&f->a.ua,					\
 		       "A <sip:a@127.0.0.1>;regint=0" prm);		\
@@ -113,7 +121,7 @@ struct fixture {
 	mem_deref(f->b.ua);			\
 	mem_deref(f->a.ua);			\
 						\
-	mock_aucodec_unregister();		\
+	module_unload("g711");			\
 						\
 	uag_event_unregister(event_handler);	\
 						\
@@ -129,6 +137,9 @@ struct fixture {
 static const struct list *hdrs;
 
 
+static const char dtmf_digits[] = "123";
+
+
 static void event_handler(struct ua *ua, enum ua_event ev,
 			  struct call *call, const char *prm, void *arg)
 {
@@ -165,7 +176,7 @@ static void event_handler(struct ua *ua, enum ua_event ev,
 		switch (f->behaviour) {
 
 		case BEHAVIOUR_ANSWER:
-			err = ua_answer(ua, call);
+			err = ua_answer(ua, call, VIDMODE_ON);
 			if (err) {
 				warning("ua_answer failed (%m)\n", err);
 				goto out;
@@ -173,9 +184,9 @@ static void event_handler(struct ua *ua, enum ua_event ev,
 			break;
 
 		case BEHAVIOUR_PROGRESS:
-			err = ua_progress(ua, call);
+			err = call_progress(call);
 			if (err) {
-				warning("ua_progress failed (%m)\n", err);
+				warning("call_progress failed (%m)\n", err);
 				goto out;
 			}
 			break;
@@ -188,7 +199,7 @@ static void event_handler(struct ua *ua, enum ua_event ev,
 
 		case BEHAVIOUR_GET_HDRS:
 			hdrs = call_get_custom_hdrs(call);
-			err = ua_answer(ua, call);
+			err = ua_answer(ua, call, VIDMODE_ON);
 			if (err) {
 				warning("ua_answer failed (%m)\n", err);
 				goto out;
@@ -289,6 +300,31 @@ static void event_handler(struct ua *ua, enum ua_event ev,
 
 	case UA_EVENT_CALL_MENC:
 		++ag->n_mediaenc;
+		ASSERT_TRUE(stream_is_secure(audio_strm(call_audio(call))));
+		break;
+
+	case UA_EVENT_CALL_DTMF_START:
+		ASSERT_EQ(1, str_len(prm));
+		ASSERT_EQ(dtmf_digits[ag->n_dtmf_recv], prm[0]);
+		++ag->n_dtmf_recv;
+
+		if (ag->n_dtmf_recv >= str_len(dtmf_digits)) {
+			re_cancel();
+		}
+		break;
+
+	case UA_EVENT_CALL_RTPESTAB:
+		++ag->n_rtpestab;
+
+		if (f->stop_on_rtp && ag->peer->n_rtpestab > 0)
+			re_cancel();
+		break;
+
+	case UA_EVENT_CALL_RTCP:
+		++ag->n_rtcp;
+
+		if (f->stop_on_rtcp && ag->peer->n_rtcp > 0)
+			re_cancel();
 		break;
 
 	default:
@@ -376,41 +412,6 @@ int test_call_reject(void)
 }
 
 
-int test_call_af_mismatch(void)
-{
-	struct fixture fix, *f = &fix;
-	int err = 0;
-
-	fixture_init(f);
-
-	ua_set_media_af(f->a.ua, AF_INET6);
-	ua_set_media_af(f->b.ua, AF_INET);
-
-	/* Make a call from A to B */
-	err = ua_connect(f->a.ua, 0, NULL, f->buri, VIDMODE_OFF);
-	TEST_ERR(err);
-
-	/* run main-loop with timeout, wait for events */
-	err = re_main_timeout(5000);
-	TEST_ERR(err);
-	TEST_ERR(fix.err);
-
-	ASSERT_EQ(0, fix.a.n_incoming);
-	ASSERT_EQ(0, fix.a.n_established);
-	ASSERT_EQ(1, fix.a.n_closed);
-	ASSERT_EQ(488, fix.a.close_scode);
-
-	ASSERT_EQ(0, fix.b.n_incoming);
-	ASSERT_EQ(0, fix.b.n_established);
-	ASSERT_EQ(1, fix.b.n_closed);
-
- out:
-	fixture_close(f);
-
-	return err;
-}
-
-
 int test_call_answer_hangup_a(void)
 {
 	struct fixture fix, *f = &fix;
@@ -676,33 +677,6 @@ int test_call_max(void)
 }
 
 
-static const char dtmf_digits[] = "123";
-
-
-static void dtmf_handler(struct call *call, char key, void *arg)
-{
-	struct agent *ag = arg;
-	int err = 0;
-	(void)call;
-
-	/* ignore key-release */
-	if (key == KEYCODE_REL)
-		return;
-
-	ASSERT_EQ(dtmf_digits[ag->n_dtmf_recv], key);
-	++ag->n_dtmf_recv;
-
-	if (ag->n_dtmf_recv >= str_len(dtmf_digits)) {
-		re_cancel();
-	}
-
- out:
-	if (err) {
-		fixture_abort(ag->fix, err);
-	}
-}
-
-
 int test_call_dtmf(void)
 {
 	struct fixture fix, *f = &fix;
@@ -714,7 +688,7 @@ int test_call_dtmf(void)
 	fixture_init_prm(f, ";ptime=1");
 
 	/* audio-source is needed for dtmf/telev to work */
-	err = mock_ausrc_register(&ausrc);
+	err = mock_ausrc_register(&ausrc, baresip_ausrcl());
 	TEST_ERR(err);
 
 	f->behaviour = BEHAVIOUR_ANSWER;
@@ -728,9 +702,6 @@ int test_call_dtmf(void)
 	TEST_ERR(err);
 	TEST_ERR(fix.err);
 
-	call_set_handlers(ua_call(f->a.ua), NULL, dtmf_handler, &f->a);
-	call_set_handlers(ua_call(f->b.ua), NULL, dtmf_handler, &f->b);
-
 	/* send some DTMF digits from A to B .. */
 	for (i=0; i<n; i++) {
 		err  = call_send_digit(ua_call(f->a.ua), dtmf_digits[i]);
@@ -753,17 +724,25 @@ int test_call_dtmf(void)
 }
 
 
-#ifdef USE_VIDEO
 static void mock_vidisp_handler(const struct vidframe *frame,
 				uint64_t timestamp, void *arg)
 {
 	struct fixture *fix = arg;
+	int err = 0;
 	(void)frame;
 	(void)timestamp;
 	(void)fix;
 
+	ASSERT_EQ(MAGIC, fix->magic);
+
+	ASSERT_EQ(conf_config()->video.enc_fmt, (int)frame->fmt);
+
 	/* Stop the test */
 	re_cancel();
+
+ out:
+	if (err)
+		fixture_abort(fix, err);
 }
 
 
@@ -775,6 +754,7 @@ int test_call_video(void)
 	int err = 0;
 
 	conf_config()->video.fps = 100;
+	conf_config()->video.enc_fmt = VID_FMT_YUV420P;
 
 	fixture_init(f);
 
@@ -812,7 +792,6 @@ int test_call_video(void)
 
 	return err;
 }
-#endif
 
 
 static void mock_sample_handler(const void *sampv, size_t sampc, void *arg)
@@ -844,11 +823,13 @@ int test_call_aulevel(void)
 
 	conf_config()->audio.level = true;
 
-	err = mock_ausrc_register(&ausrc);
+	err = mock_ausrc_register(&ausrc, baresip_ausrcl());
 	TEST_ERR(err);
-	err = mock_auplay_register(&auplay, mock_sample_handler, f);
+	err = mock_auplay_register(&auplay, baresip_auplayl(),
+				   mock_sample_handler, f);
 	TEST_ERR(err);
 
+	f->behaviour = BEHAVIOUR_ANSWER;
 	f->estab_action = ACTION_NOTHING;
 
 	/* Make a call from A to B */
@@ -954,9 +935,10 @@ static int test_media_base(enum audio_mode txmode)
 	conf_config()->audio.src_fmt = AUFMT_FLOAT;
 	conf_config()->audio.play_fmt = AUFMT_FLOAT;
 
-	err = mock_ausrc_register(&ausrc);
+	err = mock_ausrc_register(&ausrc, baresip_ausrcl());
 	TEST_ERR(err);
-	err = mock_auplay_register(&auplay, audio_sample_handler, f);
+	err = mock_auplay_register(&auplay, baresip_auplayl(),
+				   audio_sample_handler, f);
 	TEST_ERR(err);
 
 	f->estab_action = ACTION_NOTHING;
@@ -1003,11 +985,6 @@ int test_call_format_float(void)
 	err = test_media_base(AUDIO_MODE_POLL);
 	ASSERT_EQ(0, err);
 
-#ifdef HAVE_PTHREAD
-	err = test_media_base(AUDIO_MODE_THREAD);
-	ASSERT_EQ(0, err);
-#endif
-
 	conf_config()->audio.txmode = AUDIO_MODE_POLL;
 
  out:
@@ -1029,9 +1006,10 @@ int test_call_mediaenc(void)
 
 	ASSERT_STREQ("xrtp", account_mediaenc(ua_account(f->a.ua)));
 
-	err = mock_ausrc_register(&ausrc);
+	err = mock_ausrc_register(&ausrc, baresip_ausrcl());
 	TEST_ERR(err);
-	err = mock_auplay_register(&auplay, audio_sample_handler, f);
+	err = mock_auplay_register(&auplay, baresip_auplayl(),
+				   audio_sample_handler, f);
 	TEST_ERR(err);
 
 	f->estab_action = ACTION_NOTHING;
@@ -1078,17 +1056,17 @@ int test_call_medianat(void)
 	struct auplay *auplay = NULL;
 	int err;
 
-	err = mock_mnat_register(baresip_mnatl());
-	ASSERT_EQ(0, err);
+	mock_mnat_register(baresip_mnatl());
 
 	/* Enable a dummy media NAT-traversal protocol */
 	fixture_init_prm(f, ";medianat=XNAT;ptime=1");
 
 	ASSERT_STREQ("XNAT", account_medianat(ua_account(f->a.ua)));
 
-	err = mock_ausrc_register(&ausrc);
+	err = mock_ausrc_register(&ausrc, baresip_ausrcl());
 	TEST_ERR(err);
-	err = mock_auplay_register(&auplay, audio_sample_handler, f);
+	err = mock_auplay_register(&auplay, baresip_auplayl(),
+				   audio_sample_handler, f);
 	TEST_ERR(err);
 
 	f->estab_action = ACTION_NOTHING;
@@ -1286,3 +1264,136 @@ int test_call_transfer(void)
 
 	return err;
 }
+
+
+int test_call_rtcp(void)
+{
+	struct fixture fix, *f = &fix;
+	int err = 0;
+
+	/* Use a low packet time, so the test completes quickly */
+	fixture_init_prm(f, ";ptime=1");
+
+	f->behaviour = BEHAVIOUR_ANSWER;
+	f->estab_action = ACTION_NOTHING;
+	f->stop_on_rtcp = true;
+
+	/* Make a call from A to B */
+	err = ua_connect(f->a.ua, 0, NULL, f->buri, VIDMODE_OFF);
+	TEST_ERR(err);
+
+	/* run main-loop with timeout, wait for events */
+	err = re_main_timeout(5000);
+	TEST_ERR(err);
+	TEST_ERR(fix.err);
+
+	/* verify that one or more RTCP packets were received */
+	ASSERT_TRUE(fix.a.n_rtcp > 0);
+	ASSERT_TRUE(fix.b.n_rtcp > 0);
+
+ out:
+	fixture_close(f);
+
+	return err;
+}
+
+
+int test_call_aufilt(void)
+{
+	int err;
+
+	mock_aufilt_register(baresip_aufiltl());
+
+	err = test_media_base(AUDIO_MODE_POLL);
+	ASSERT_EQ(0, err);
+
+ out:
+	mock_aufilt_unregister();
+
+	return err;
+}
+
+
+/*
+ * Simulate a complete WebRTC testcase
+ */
+int test_call_webrtc(void)
+{
+	struct fixture fix, *f = &fix;
+	struct ausrc *ausrc = NULL;
+	struct vidsrc *vidsrc = NULL;
+	struct sdp_media *sdp_a, *sdp_b;
+	int err;
+
+	mock_mnat_register(baresip_mnatl());
+	mock_menc_register();
+
+	err = mock_ausrc_register(&ausrc, baresip_ausrcl());
+	TEST_ERR(err);
+
+	/* to enable video, we need one vidsrc and vidcodec */
+	mock_vidcodec_register();
+	err = mock_vidsrc_register(&vidsrc);
+	TEST_ERR(err);
+
+	fixture_init_prm(f, ";medianat=XNAT;mediaenc=xrtp");
+
+	f->estab_action = ACTION_NOTHING;
+	f->behaviour = BEHAVIOUR_ANSWER;
+	f->stop_on_rtp = true;
+
+	/* Make a call from A to B */
+	err = ua_connect(f->a.ua, 0, NULL, f->buri, VIDMODE_ON);
+	TEST_ERR(err);
+
+	/* run main-loop with timeout, wait for events */
+	err = re_main_timeout(5000);
+	TEST_ERR(err);
+	TEST_ERR(fix.err);
+
+	/* verify MNAT */
+
+	/* verify that MENC is secure */
+
+	ASSERT_TRUE(
+	  stream_is_secure(audio_strm(call_audio(ua_call(f->a.ua)))));
+	ASSERT_TRUE(
+	  stream_is_secure(audio_strm(call_audio(ua_call(f->b.ua)))));
+
+	ASSERT_TRUE(
+	  stream_is_secure(video_strm(call_video(ua_call(f->a.ua)))));
+	ASSERT_TRUE(
+	  stream_is_secure(video_strm(call_video(ua_call(f->b.ua)))));
+
+	/* verify that one or more RTP packets were received */
+	ASSERT_TRUE(fix.a.n_rtpestab > 0);
+	ASSERT_TRUE(fix.b.n_rtpestab > 0);
+
+	ASSERT_TRUE(call_has_video(ua_call(f->a.ua)));
+	ASSERT_TRUE(call_has_video(ua_call(f->b.ua)));
+
+	/* Verify SDP attributes */
+
+	sdp_a = stream_sdpmedia(audio_strm(call_audio(ua_call(f->a.ua))));
+	sdp_b = stream_sdpmedia(audio_strm(call_audio(ua_call(f->b.ua))));
+
+	ASSERT_TRUE(NULL != sdp_media_rattr(sdp_a, "ssrc"));
+	ASSERT_EQ(20, atoi(sdp_media_rattr(sdp_a, "ptime")));
+
+	ASSERT_TRUE(NULL != sdp_media_rattr(sdp_b, "ssrc"));
+	ASSERT_EQ(20, atoi(sdp_media_rattr(sdp_b, "ptime")));
+
+ out:
+	fixture_close(f);
+
+	mem_deref(vidsrc);
+	mem_deref(ausrc);
+	mock_vidcodec_unregister();
+	mock_menc_unregister();
+	mock_mnat_unregister();
+
+	if (fix.err)
+		return fix.err;
+
+	return err;
+}
