@@ -27,6 +27,15 @@ static struct {
 } mock;
 
 
+static void auplay_destructor(void *arg)
+{
+	struct auplay_st *st = arg;
+
+	tmr_cancel(&st->tmr);
+	mem_deref(st->sampv);
+}
+
+
 static void tmr_handler(void *arg)
 {
 	struct auplay_st *st = arg;
@@ -42,15 +51,6 @@ static void tmr_handler(void *arg)
 }
 
 
-static void auplay_destructor(void *arg)
-{
-	struct auplay_st *st = arg;
-
-	tmr_cancel(&st->tmr);
-	mem_deref(st->sampv);
-}
-
-
 static int mock_auplay_alloc(struct auplay_st **stp, const struct auplay *ap,
 			    struct auplay_prm *prm, const char *device,
 			    auplay_write_h *wh, void *arg)
@@ -91,12 +91,12 @@ static int mock_auplay_alloc(struct auplay_st **stp, const struct auplay *ap,
 }
 
 
-int mock_auplay_register(struct auplay **auplayp,
+int mock_auplay_register(struct auplay **auplayp, struct list *auplayl,
 			 mock_sample_h *sampleh, void *arg)
 {
 	mock.sampleh = sampleh;
 	mock.arg = arg;
 
-	return auplay_register(auplayp, baresip_auplayl(),
+	return auplay_register(auplayp, auplayl,
 			      "mock-auplay", mock_auplay_alloc);
 }
