@@ -21,17 +21,6 @@ struct ausrc_st {
 };
 
 
-static void tmr_handler(void *arg)
-{
-	struct ausrc_st *st = arg;
-
-	tmr_start(&st->tmr, st->prm.ptime, tmr_handler, st);
-
-	if (st->rh)
-		st->rh(st->sampv, st->sampc, st->arg);
-}
-
-
 static void ausrc_destructor(void *arg)
 {
 	struct ausrc_st *st = arg;
@@ -41,6 +30,23 @@ static void ausrc_destructor(void *arg)
 }
 
 
+static void tmr_handler(void *arg)
+{
+	struct ausrc_st *st = arg;
+	struct auframe af = {
+		.fmt   = st->prm.fmt,
+		.sampv = st->sampv,
+		.sampc = st->sampc,
+		.timestamp = tmr_jiffies_usec()
+	};
+
+	tmr_start(&st->tmr, st->prm.ptime, tmr_handler, st);
+
+	if (st->rh)
+		st->rh(&af, st->arg);
+}
+
+
 static int mock_ausrc_alloc(struct ausrc_st **stp, const struct ausrc *as,
 			    struct media_ctx **ctx,
 			    struct ausrc_prm *prm, const char *device,
@@ -84,8 +90,8 @@ static int mock_ausrc_alloc(struct ausrc_st **stp, const struct ausrc *as,
 }
 
 
-int mock_ausrc_register(struct ausrc **ausrcp)
+int mock_ausrc_register(struct ausrc **ausrcp, struct list *ausrcl)
 {
-	return ausrc_register(ausrcp, baresip_ausrcl(),
+	return ausrc_register(ausrcp, ausrcl,
 			      "mock-ausrc", mock_ausrc_alloc);
 }
