@@ -107,7 +107,7 @@ static int reg(enum sip_transp tp)
 	char aor[256];
 	int err;
 
-	memset(&t, 0, sizeof t);
+	memset(&t, 0, sizeof(t));
 
 	err = sip_server_alloc(&t.srvv[0], sip_server_exit_handler, NULL);
 	if (err) {
@@ -115,6 +115,8 @@ static int reg(enum sip_transp tp)
 		goto out;
 	}
 
+	t.srvc = 1;
+
 	err = sip_server_uri(t.srvv[0], aor, sizeof(aor), tp);
 	TEST_ERR(err);
 
@@ -155,7 +157,7 @@ int test_ua_register(void)
 {
 	int err = 0;
 
-	err = ua_init("test", true, true, true, false);
+	err = ua_init("test", true, true, true);
 	TEST_ERR(err);
 
 	err |= reg(SIP_TRANSP_UDP);
@@ -164,6 +166,7 @@ int test_ua_register(void)
 	err |= reg(SIP_TRANSP_TLS);
 #endif
 
+	ua_stop_all(true);
 	ua_close();
 
  out:
@@ -260,7 +263,7 @@ static int reg_dns(enum sip_transp tp)
 	size_t i;
 	int err;
 
-	memset(&t, 0, sizeof t);
+	memset(&t, 0, sizeof(t));
 
 	/*
 	 * Setup server-side mocks:
@@ -272,7 +275,7 @@ static int reg_dns(enum sip_transp tp)
 	info("| DNS-server on %J\n", &dnssrv->addr);
 
 	/* NOTE: must be done before ua_init() */
-	err = net_use_nameserver(net, &dnssrv->addr);
+	err = net_use_nameserver(net, &dnssrv->addr, 1);
 	TEST_ERR(err);
 
 	for (i=0; i<server_count; i++) {
@@ -320,7 +323,7 @@ static int reg_dns(enum sip_transp tp)
 	 * Start SIP client:
 	 */
 
-	err = ua_init("test", true, true, true, false);
+	err = ua_init("test", true, true, true);
 	TEST_ERR(err);
 
 	err = ua_alloc(&t.ua, aor);
@@ -394,7 +397,7 @@ static int reg_auth(enum sip_transp tp)
 	char aor[256];
 	int err;
 
-	memset(&t, 0, sizeof t);
+	memset(&t, 0, sizeof(t));
 
 	err = sip_server_alloc(&t.srvv[0], sip_server_exit_handler, NULL);
 	if (err) {
@@ -402,6 +405,8 @@ static int reg_auth(enum sip_transp tp)
 		goto out;
 	}
 
+	t.srvc = 1;
+
 	err = domain_add(t.srvv[0], DOMAIN);
 	TEST_ERR(err);
 
@@ -466,7 +471,7 @@ int test_ua_register_auth(void)
 {
 	int err;
 
-	err = ua_init("test", true, true, true, false);
+	err = ua_init("test", true, true, true);
 	TEST_ERR(err);
 
 	err |= reg_auth(SIP_TRANSP_UDP);
@@ -501,7 +506,7 @@ static int reg_auth_dns(enum sip_transp tp)
 	unsigned total_req = 0;
 	int err;
 
-	memset(&t, 0, sizeof t);
+	memset(&t, 0, sizeof(t));
 
 	/*
 	 * Setup server-side mocks:
@@ -513,7 +518,7 @@ static int reg_auth_dns(enum sip_transp tp)
 	info("| DNS-server on %J\n", &dnssrv->addr);
 
 	/* NOTE: must be done before ua_init() */
-	err = net_use_nameserver(net, &dnssrv->addr);
+	err = net_use_nameserver(net, &dnssrv->addr, 1);
 	TEST_ERR(err);
 
 	for (i=0; i<server_count; i++) {
@@ -577,7 +582,7 @@ static int reg_auth_dns(enum sip_transp tp)
 	 * Start SIP client:
 	 */
 
-	err = ua_init("test", true, true, true, false);
+	err = ua_init("test", true, true, true);
 	TEST_ERR(err);
 
 	err = ua_alloc(&t.ua, aor);
@@ -715,7 +720,7 @@ static int test_ua_options_base(enum sip_transp transp)
 	err = ua_init("test",
 		      transp == SIP_TRANSP_UDP,
 		      transp == SIP_TRANSP_TCP,
-		      false, false);
+		      false);
 	TEST_ERR(err);
 
 	err = sip_transp_laddr(uag_sip(), &laddr, transp, NULL);
@@ -738,7 +743,10 @@ static int test_ua_options_base(enum sip_transp transp)
 	if (err)
 		goto out;
 
-	TEST_ERR(t.err);
+	if (t.err) {
+		err = t.err;
+		goto out;
+	}
 
 	/* verify after test is complete */
 	ASSERT_EQ(1, t.n_resp);
